@@ -1,8 +1,8 @@
 #!/usr/bin/env perl
 #
 # Copyright (c) 2011-2014 Cisco Systems, Inc.  All rights reserved.
-# Copyright (c) 2016      Research Organization for Information Science
-#                         and Technology (RIST). All rights reserved.
+# Copyright (c) 2016-2019 Research Organization for Information Science
+#                         and Technology (RIST).  All rights reserved.
 # Copyright (c) 2016      FUJITSU LIMITED.  All rights reserved.
 # $COPYRIGHT$
 #
@@ -189,12 +189,12 @@ $handles->{MPI_INT64_T} = 64;
 $handles->{MPI_UINT64_T} = 65;
 $handles->{MPI_AINT} = 66;
 $handles->{MPI_OFFSET} = 67;
-$handles->{MPI_C_COMPLEX} = 68;
-$handles->{MPI_C_FLOAT_COMPLEX} = 69;
+$handles->{MPI_C_BOOL} = 68;
+$handles->{MPI_C_COMPLEX} = 69;
+$handles->{MPI_C_FLOAT_COMPLEX} = $handles->{MPI_C_COMPLEX};
 $handles->{MPI_C_DOUBLE_COMPLEX} = 70;
 $handles->{MPI_C_LONG_DOUBLE_COMPLEX} = 71;
 $handles->{MPI_COUNT} = 72;
-$handles->{MPI_C_BOOL} = 73;
 
 $handles->{MPI_MESSAGE_NO_PROC} = 1;
 
@@ -511,7 +511,6 @@ foreach my $key (sort(keys(%{$handles}))) {
     $output .= "#define OMPI_$key $handles->{$key}\n";
 }
 
-$output .= "\n#if OMPI_PROVIDE_MPI_FILE_INTERFACE\n";
 foreach my $key (sort(keys(%{$io_constants}))) {
     $output .= "#define OMPI_$key $io_constants->{$key}\n";
 }
@@ -522,10 +521,67 @@ $output .= "\n";
 foreach my $key (sort(keys(%{$io_handles}))) {
     $output .= "#define OMPI_$key $io_handles->{$key}\n";
 }
-$output .= "#endif /* OMPI_PROVIDE_MPI_FILE_INTERFACE */
-
-#endif /* USE_MPI_F08_CONSTANTS_H */\n";
+$output .= "\n";
+$output .= "#endif /* USE_MPI_F08_CONSTANTS_H */\n";
 
 write_file("$topdir/ompi/mpi/fortran/use-mpi-f08/constants.h", $output);
 
+$output = '! WARNING! THIS IS A GENERATED FILE!!
+! ANY EDITS YOU PUT HERE WILL BE LOST!
+! Instead, edit topdir/ompi/include/mpif-values.pl
+!
+
+!
+! Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+!                         University Research and Technology
+!                         Corporation.  All rights reserved.
+! Copyright (c) 2004-2006 The University of Tennessee and The University
+!                         of Tennessee Research Foundation.  All rights
+!                         reserved.
+! Copyright (c) 2004-2007 High Performance Computing Center Stuttgart,
+!                         University of Stuttgart.  All rights reserved.
+! Copyright (c) 2004-2005 The Regents of the University of California.
+!                         All rights reserved.
+! Copyright (c) 2007-2009 Cisco Systems, Inc.  All rights reserved.
+! Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
+! Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
+! Copyright (c) 2009-2012 Los Alamos National Security, LLC.
+!                         All rights reserved.
+! Copyright (c) 2016-2019 Research Organization for Information Science
+!                         and Technology (RIST). All rights reserved.
+! $COPYRIGHT$
+!
+! Additional copyrights may follow
+!
+! $HEADER$
+!
+
+#ifndef USE_MPI_F08_CONSTANTS_H
+#define USE_MPI_F08_CONSTANTS_H
+
+';
+
+foreach my $key (sort(keys(%{$constants}))) {
+    $output .= "#define OMPI_$key $constants->{$key}\n";
+}
+$output .= "\n";
+foreach my $key (sort(keys(%{$handles}))) {
+    $output .= "#define OMPI_$key $handles->{$key}\n";
+}
+
+foreach my $key (sort(keys(%{$io_constants}))) {
+    $output .= "#define OMPI_$key $io_constants->{$key}\n";
+}
+foreach my $key (sort(keys(%{$lio_constants}))) {
+    $output .= "#define OMPI_$key $lio_constants->{$key}\n";
+}
+$output .= "\n";
+foreach my $key (sort(keys(%{$io_handles}))) {
+    $output .= "#define OMPI_$key $io_handles->{$key}\n";
+}
+$output .= "\n";
+$output .= "#endif\n";
+
+write_file("$topdir/ompi/mpi/fortran/use-mpi-f08/mod/mpi-f08-constants.h", $output);
+
 exit(0);
