@@ -20,6 +20,7 @@
  * Copyright (c) 2014      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2016-2017 IBM Corporation.  All rights reserved.
+ * Copyright (c) 2017      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -52,7 +53,7 @@ struct avail_coll_t {
     opal_list_item_t super;
 
     int ac_priority;
-    mca_coll_base_module_2_2_0_t *ac_module;
+    mca_coll_base_module_2_3_0_t *ac_module;
     const char * ac_component_name;
 };
 typedef struct avail_coll_t avail_coll_t;
@@ -65,16 +66,16 @@ static opal_list_t *check_components(opal_list_t * components,
                                      ompi_communicator_t * comm);
 static int check_one_component(ompi_communicator_t * comm,
                                const mca_base_component_t * component,
-                               mca_coll_base_module_2_2_0_t ** module);
+                               mca_coll_base_module_2_3_0_t ** module);
 
 static int query(const mca_base_component_t * component,
                  ompi_communicator_t * comm, int *priority,
-                 mca_coll_base_module_2_2_0_t ** module);
+                 mca_coll_base_module_2_3_0_t ** module);
 
 static int query_2_0_0(const mca_coll_base_component_2_0_0_t *
                        coll_component, ompi_communicator_t * comm,
                        int *priority,
-                       mca_coll_base_module_2_2_0_t ** module);
+                       mca_coll_base_module_2_3_0_t ** module);
 
 /*
  * Stuff for the OBJ interface
@@ -190,6 +191,24 @@ int mca_coll_base_comm_select(ompi_communicator_t * comm)
             COPY(avail->ac_module, comm, iscatter);
             COPY(avail->ac_module, comm, iscatterv);
 
+            COPY(avail->ac_module, comm, allgather_init);
+            COPY(avail->ac_module, comm, allgatherv_init);
+            COPY(avail->ac_module, comm, allreduce_init);
+            COPY(avail->ac_module, comm, alltoall_init);
+            COPY(avail->ac_module, comm, alltoallv_init);
+            COPY(avail->ac_module, comm, alltoallw_init);
+            COPY(avail->ac_module, comm, barrier_init);
+            COPY(avail->ac_module, comm, bcast_init);
+            COPY(avail->ac_module, comm, exscan_init);
+            COPY(avail->ac_module, comm, gather_init);
+            COPY(avail->ac_module, comm, gatherv_init);
+            COPY(avail->ac_module, comm, reduce_init);
+            COPY(avail->ac_module, comm, reduce_scatter_block_init);
+            COPY(avail->ac_module, comm, reduce_scatter_init);
+            COPY(avail->ac_module, comm, scan_init);
+            COPY(avail->ac_module, comm, scatter_init);
+            COPY(avail->ac_module, comm, scatterv_init);
+
             /* We can not reliably check if this comm has a topology
              * at this time. The flags are set *after* coll_select */
             COPY(avail->ac_module, comm, neighbor_allgather);
@@ -204,6 +223,12 @@ int mca_coll_base_comm_select(ompi_communicator_t * comm)
             COPY(avail->ac_module, comm, ineighbor_alltoallv);
             COPY(avail->ac_module, comm, ineighbor_alltoallw);
 
+            COPY(avail->ac_module, comm, neighbor_allgather_init);
+            COPY(avail->ac_module, comm, neighbor_allgatherv_init);
+            COPY(avail->ac_module, comm, neighbor_alltoall_init);
+            COPY(avail->ac_module, comm, neighbor_alltoallv_init);
+            COPY(avail->ac_module, comm, neighbor_alltoallw_init);
+
             COPY(avail->ac_module, comm, reduce_local);
         }
         /* release the original module reference and the list item */
@@ -249,6 +274,23 @@ int mca_coll_base_comm_select(ompi_communicator_t * comm)
         ((OMPI_COMM_IS_INTRA(comm)) && CHECK_NULL(which_func, comm, iscan)) ||
         CHECK_NULL(which_func, comm, iscatter) ||
         CHECK_NULL(which_func, comm, iscatterv) ||
+        CHECK_NULL(which_func, comm, allgather_init) ||
+        CHECK_NULL(which_func, comm, allgatherv_init) ||
+        CHECK_NULL(which_func, comm, allreduce_init) ||
+        CHECK_NULL(which_func, comm, alltoall_init) ||
+        CHECK_NULL(which_func, comm, alltoallv_init) ||
+        CHECK_NULL(which_func, comm, alltoallw_init) ||
+        CHECK_NULL(which_func, comm, barrier_init) ||
+        CHECK_NULL(which_func, comm, bcast_init) ||
+        ((OMPI_COMM_IS_INTRA(comm)) && CHECK_NULL(which_func, comm, exscan_init)) ||
+        CHECK_NULL(which_func, comm, gather_init) ||
+        CHECK_NULL(which_func, comm, gatherv_init) ||
+        CHECK_NULL(which_func, comm, reduce_init) ||
+        CHECK_NULL(which_func, comm, reduce_scatter_block_init) ||
+        CHECK_NULL(which_func, comm, reduce_scatter_init) ||
+        ((OMPI_COMM_IS_INTRA(comm)) && CHECK_NULL(which_func, comm, scan_init)) ||
+        CHECK_NULL(which_func, comm, scatter_init) ||
+        CHECK_NULL(which_func, comm, scatterv_init) ||
         CHECK_NULL(which_func, comm, reduce_local) ) {
         /* TODO -- Once the topology flags are set before coll_select then
          * check if neighborhood collectives have been set. */
@@ -288,7 +330,7 @@ static opal_list_t *check_components(opal_list_t * components,
     int priority;
     const mca_base_component_t *component;
     mca_base_component_list_item_t *cli;
-    mca_coll_base_module_2_2_0_t *module;
+    mca_coll_base_module_2_3_0_t *module;
     opal_list_t *selectable;
     avail_coll_t *avail;
 
@@ -344,7 +386,7 @@ static opal_list_t *check_components(opal_list_t * components,
  */
 static int check_one_component(ompi_communicator_t * comm,
                                const mca_base_component_t * component,
-                               mca_coll_base_module_2_2_0_t ** module)
+                               mca_coll_base_module_2_3_0_t ** module)
 {
     int err;
     int priority = -1;
@@ -378,7 +420,7 @@ static int check_one_component(ompi_communicator_t * comm,
  */
 static int query(const mca_base_component_t * component,
                  ompi_communicator_t * comm,
-                 int *priority, mca_coll_base_module_2_2_0_t ** module)
+                 int *priority, mca_coll_base_module_2_3_0_t ** module)
 {
     *module = NULL;
     if (2 == component->mca_type_major_version &&
@@ -398,9 +440,9 @@ static int query(const mca_base_component_t * component,
 
 static int query_2_0_0(const mca_coll_base_component_2_0_0_t * component,
                        ompi_communicator_t * comm, int *priority,
-                       mca_coll_base_module_2_2_0_t ** module)
+                       mca_coll_base_module_2_3_0_t ** module)
 {
-    mca_coll_base_module_2_2_0_t *ret;
+    mca_coll_base_module_2_3_0_t *ret;
 
     /* There's currently no need for conversion */
 
