@@ -5,11 +5,12 @@
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2006      The Technical University of Chemnitz. All
  *                         rights reserved.
- * Copyright (c) 2014-2017 Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2014-2018 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2016-2017 IBM Corporation.  All rights reserved.
+ * Copyright (c) 2018      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -44,9 +45,9 @@ int NBC_Bcast_args_compare(NBC_Bcast_args *a, NBC_Bcast_args *b, void *param) {
 }
 #endif
 
-int ompi_coll_libnbc_ibcast(void *buffer, int count, MPI_Datatype datatype, int root,
-                            struct ompi_communicator_t *comm, ompi_request_t ** request,
-                            struct mca_coll_base_module_2_2_0_t *module)
+static int nbc_bcast_init(void *buffer, int count, MPI_Datatype datatype, int root,
+                          struct ompi_communicator_t *comm, ompi_request_t ** request,
+                          struct mca_coll_base_module_2_3_0_t *module, bool persistent)
 {
   int rank, p, res, segsize;
   size_t size;
@@ -61,8 +62,7 @@ int ompi_coll_libnbc_ibcast(void *buffer, int count, MPI_Datatype datatype, int
   p = ompi_comm_size (comm);
 
   if (1 == p) {
-    *request = &ompi_request_empty;
-    return OMPI_SUCCESS;
+    return nbc_get_noop_request(persistent, request);
   }
 
   res = ompi_datatype_type_size(datatype, &size);
@@ -162,7 +162,7 @@ int ompi_coll_libnbc_ibcast(void *buffer, int count, MPI_Datatype datatype, int
   }
 #endif
 
-  res = NBC_Schedule_request(schedule, comm, libnbc_module, request, NULL);
+  res = NBC_Schedule_request(schedule, comm, libnbc_module, persistent, request, NULL);
   if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
     OBJ_RELEASE(schedule);
     return res;
@@ -171,6 +171,25 @@ int ompi_coll_libnbc_ibcast(void *buffer, int count, MPI_Datatype datatype, int
   return OMPI_SUCCESS;
 }
 
+int ompi_coll_libnbc_ibcast(void *buffer, int count, MPI_Datatype datatype, int root,
+                            struct ompi_communicator_t *comm, ompi_request_t ** request,
+                            struct mca_coll_base_module_2_3_0_t *module)
+{
+    int res = nbc_bcast_init(buffer, count, datatype, root,
+                             comm, request, module, false);
+    if (OPAL_LIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+    res = NBC_Start(*(ompi_coll_libnbc_request_t **)request);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        NBC_Return_handle (*(ompi_coll_libnbc_request_t **)request);
+        *request = &ompi_request_null.request;
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
+
 /* better binomial bcast
  * working principle:
  * - each node gets a virtual rank vrank
@@ -323,9 +342,9 @@ static inline int bcast_sched_chain(int rank, int p, int root, NBC_Schedule *sch
   return OMPI_SUCCESS;
 }
 
-int ompi_coll_libnbc_ibcast_inter(void *buffer, int count, MPI_Datatype datatype, int root,
-                                  struct ompi_communicator_t *comm, ompi_request_t ** request,
-                                  struct mca_coll_base_module_2_2_0_t *module) {
+static int nbc_bcast_inter_init(void *buffer, int count, MPI_Datatype datatype, int root,
+                                struct ompi_communicator_t *comm, ompi_request_t ** request,
+                                struct mca_coll_base_module_2_3_0_t *module, bool persistent) {
   int res;
   NBC_Schedule *schedule;
   ompi_coll_libnbc_module_t *libnbc_module = (ompi_coll_libnbc_module_t*) module;
@@ -366,7 +385,7 @@ int ompi_coll_libnbc_ibcast_inter(void *buffer, int count, MPI_Datatype datatype
     return res;
   }
 
-  res = NBC_Schedule_request(schedule, comm, libnbc_module, request, NULL);
+  res = NBC_Schedule_request(schedule, comm, libnbc_module, persistent, request, NULL);
   if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
     OBJ_RELEASE(schedule);
     return res;
@@ -374,3 +393,46 @@ int ompi_coll_libnbc_ibcast_inter(void *buffer, int count, MPI_Datatype datatype
 
   return OMPI_SUCCESS;
 }
+
+int ompi_coll_libnbc_ibcast_inter(void *buffer, int count, MPI_Datatype datatype, int root,
+                                  struct ompi_communicator_t *comm, ompi_request_t ** request,
+                                  struct mca_coll_base_module_2_3_0_t *module) {
+    int res = nbc_bcast_inter_init(buffer, count, datatype, root,
+                                   comm, request, module, false);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+  
+    res = NBC_Start(*(ompi_coll_libnbc_request_t **)request);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        NBC_Return_handle (*(ompi_coll_libnbc_request_t **)request);
+        *request = &ompi_request_null.request;
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
+
+int ompi_coll_libnbc_bcast_init(void *buffer, int count, MPI_Datatype datatype, int root,
+                                struct ompi_communicator_t *comm, MPI_Info info, ompi_request_t ** request,
+                                struct mca_coll_base_module_2_3_0_t *module) {
+    int res = nbc_bcast_init(buffer, count, datatype, root,
+                             comm, request, module, true);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
+
+int ompi_coll_libnbc_bcast_inter_init(void *buffer, int count, MPI_Datatype datatype, int root,
+                                      struct ompi_communicator_t *comm, MPI_Info info, ompi_request_t ** request,
+                                      struct mca_coll_base_module_2_3_0_t *module) {
+    int res = nbc_bcast_inter_init(buffer, count, datatype, root,
+                                   comm, request, module, true);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
