@@ -10,9 +10,10 @@
  * Copyright (c) 2013      The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2014-2017 Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2014-2018 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2017      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2018      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -44,10 +45,10 @@ int NBC_Scatter_args_compare(NBC_Scatter_args *a, NBC_Scatter_args *b, void *par
 #endif
 
 /* simple linear MPI_Iscatter */
-int ompi_coll_libnbc_iscatter (const void* sendbuf, int sendcount, MPI_Datatype sendtype,
-                               void* recvbuf, int recvcount, MPI_Datatype recvtype, int root,
-                               struct ompi_communicator_t *comm, ompi_request_t ** request,
-                               struct mca_coll_base_module_2_2_0_t *module) {
+static int nbc_scatter_init (const void* sendbuf, int sendcount, MPI_Datatype sendtype,
+                             void* recvbuf, int recvcount, MPI_Datatype recvtype, int root,
+                             struct ompi_communicator_t *comm, ompi_request_t ** request,
+                             struct mca_coll_base_module_2_3_0_t *module, bool persistent) {
   int rank, p, res;
   MPI_Aint sndext = 0;
   NBC_Schedule *schedule;
@@ -69,15 +70,6 @@ int ompi_coll_libnbc_iscatter (const void* sendbuf, int sendcount, MPI_Datatype
     }
   }
 
-  if ((rank == root) && (!inplace)) {
-    sbuf = (char *) sendbuf + rank * sendcount * sndext;
-    /* if I am the root - just copy the message (not for MPI_IN_PLACE) */
-    res = NBC_Copy (sbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
-    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
-      return res;
-    }
-  }
-
 #ifdef NBC_CACHE_SCHEDULE
   NBC_Scatter_args *args, *found, search;
 
@@ -108,7 +100,17 @@ int ompi_coll_libnbc_iscatter (const void* sendbuf, int sendcount, MPI_Datatype
     } else {
       for (int i = 0 ; i < p ; ++i) {
         sbuf = (char *) sendbuf + i * sendcount * sndext;
-        if (i != root) {
+        if (i == root) {
+          if (!inplace) {
+            /* if I am the root - just copy the message */
+            res = NBC_Sched_copy (sbuf, false, sendcount, sendtype,
+                                  recvbuf, false, recvcount, recvtype, schedule, false);
+            if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+              OBJ_RELEASE(schedule);
+              return res;
+            }
+          }
+        } else {
           /* root sends the right buffer to the right receiver */
           res = NBC_Sched_send (sbuf, false, sendcount, sendtype, i, schedule, false);
           if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
@@ -157,7 +159,7 @@ int ompi_coll_libnbc_iscatter (const void* sendbuf, int sendcount, MPI_Datatype
   }
 #endif
 
-  res = NBC_Schedule_request(schedule, comm, libnbc_module, request, NULL);
+  res = NBC_Schedule_request(schedule, comm, libnbc_module, persistent, request, NULL);
   if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
     OBJ_RELEASE(schedule);
     return res;
@@ -166,10 +168,29 @@ int ompi_coll_libnbc_iscatter (const void* sendbuf, int sendcount, MPI_Datatype
   return OMPI_SUCCESS;
 }
 
-int ompi_coll_libnbc_iscatter_inter (const void* sendbuf, int sendcount, MPI_Datatype sendtype,
-                                     void* recvbuf, int recvcount, MPI_Datatype recvtype, int root,
-                                     struct ompi_communicator_t *comm, ompi_request_t ** request,
-                                     struct mca_coll_base_module_2_2_0_t *module) {
+int ompi_coll_libnbc_iscatter (const void* sendbuf, int sendcount, MPI_Datatype sendtype,
+                               void* recvbuf, int recvcount, MPI_Datatype recvtype, int root,
+                               struct ompi_communicator_t *comm, ompi_request_t ** request,
+                               struct mca_coll_base_module_2_3_0_t *module) {
+    int res = nbc_scatter_init(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
+                               comm, request, module, false);
+    if (OPAL_LIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+    res = NBC_Start(*(ompi_coll_libnbc_request_t **)request);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        NBC_Return_handle (*(ompi_coll_libnbc_request_t **)request);
+        *request = &ompi_request_null.request;
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
+
+static int nbc_scatter_inter_init (const void* sendbuf, int sendcount, MPI_Datatype sendtype,
+                                   void* recvbuf, int recvcount, MPI_Datatype recvtype, int root,
+                                   struct ompi_communicator_t *comm, ompi_request_t ** request,
+                                   struct mca_coll_base_module_2_3_0_t *module, bool persistent) {
     int res, rsize;
     MPI_Aint sndext;
     NBC_Schedule *schedule;
@@ -217,7 +238,7 @@ int ompi_coll_libnbc_iscatter_inter (const void* sendbuf, int sendcount, MPI_Dat
         return res;
     }
 
-    res = NBC_Schedule_request(schedule, comm, libnbc_module, request, NULL);
+    res = NBC_Schedule_request(schedule, comm, libnbc_module, persistent, request, NULL);
     if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
         OBJ_RELEASE(schedule);
         return res;
@@ -225,3 +246,48 @@ int ompi_coll_libnbc_iscatter_inter (const void* sendbuf, int sendcount, MPI_Dat
 
     return OMPI_SUCCESS;
 }
+
+int ompi_coll_libnbc_iscatter_inter (const void* sendbuf, int sendcount, MPI_Datatype sendtype,
+                                     void* recvbuf, int recvcount, MPI_Datatype recvtype, int root,
+                                     struct ompi_communicator_t *comm, ompi_request_t ** request,
+                                     struct mca_coll_base_module_2_3_0_t *module) {
+    int res = nbc_scatter_inter_init(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
+                                     comm, request, module, false);
+    if (OPAL_LIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+    res = NBC_Start(*(ompi_coll_libnbc_request_t **)request);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        NBC_Return_handle (*(ompi_coll_libnbc_request_t **)request);
+        *request = &ompi_request_null.request;
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
+
+int ompi_coll_libnbc_scatter_init(const void* sendbuf, int sendcount, MPI_Datatype sendtype,
+                                  void* recvbuf, int recvcount, MPI_Datatype recvtype, int root,
+                                  struct ompi_communicator_t *comm, MPI_Info info, ompi_request_t ** request,
+                                  struct mca_coll_base_module_2_3_0_t *module) {
+    int res = nbc_scatter_init(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
+                               comm, request, module, true);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
+
+int ompi_coll_libnbc_scatter_inter_init(const void* sendbuf, int sendcount, MPI_Datatype sendtype,
+                                        void* recvbuf, int recvcount, MPI_Datatype recvtype, int root,
+                                        struct ompi_communicator_t *comm, MPI_Info info, ompi_request_t ** request,
+                                        struct mca_coll_base_module_2_3_0_t *module) {
+    int res = nbc_scatter_inter_init(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root,
+                                     comm, request, module, true);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
