@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
@@ -386,6 +386,58 @@ int ompi_coll_tuned_reduce_scatter_intra_dec_dynamic(const void *sbuf, void *rbu
                                                            dtype, op, comm, module);
 }
 
+/*
+ *    reduce_scatter_block_intra_dec
+ *
+ *    Function:   - seletects reduce_scatter_block algorithm to use
+ *    Accepts:    - same arguments as MPI_Reduce_scatter_block()
+ *    Returns:    - MPI_SUCCESS or error code (passed from
+ *                  the reduce_scatter implementation)
+ *
+ */
+int ompi_coll_tuned_reduce_scatter_block_intra_dec_dynamic(const void *sbuf, void *rbuf,
+                                                           int rcount,
+                                                           struct ompi_datatype_t *dtype,
+                                                           struct ompi_op_t *op,
+                                                           struct ompi_communicator_t *comm,
+                                                           mca_coll_base_module_t *module)
+{
+    mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
+
+    OPAL_OUTPUT((ompi_coll_tuned_stream, "coll:tuned:reduce_scatter_block_intra_dec_dynamic"));
+
+    /* check to see if we have some filebased rules */
+    if (tuned_module->com_rules[REDUCESCATTERBLOCK]) {
+        /* we do, so calc the message size or what ever we need and use
+           this for the evaluation */
+        int alg, faninout, segsize, ignoreme, size;
+        size_t dsize;
+        size = ompi_comm_size(comm);
+        ompi_datatype_type_size (dtype, &dsize);
+        dsize *= rcount * size;
+
+        alg = ompi_coll_tuned_get_target_method_params(tuned_module->com_rules[REDUCESCATTERBLOCK],
+                                                       dsize, &faninout,
+                                                       &segsize, &ignoreme);
+        if (alg) {
+            /* we have found a valid choice from the file based rules for this message size */
+            return  ompi_coll_tuned_reduce_scatter_block_intra_do_this (sbuf, rbuf, rcount, dtype,
+                                                                        op, comm, module,
+                                                                        alg, faninout, segsize);
+        } /* found a method */
+    } /* end if any com rules to check */
+
+    if (tuned_module->user_forced[REDUCESCATTERBLOCK].algorithm) {
+        return ompi_coll_tuned_reduce_scatter_block_intra_do_this(sbuf, rbuf, rcount, dtype,
+                                                                  op, comm, module,
+                                                                  tuned_module->user_forced[REDUCESCATTERBLOCK].algorithm,
+                                                                  tuned_module->user_forced[REDUCESCATTERBLOCK].chain_fanout,
+                                                                  tuned_module->user_forced[REDUCESCATTERBLOCK].segsize);
+    }
+    return ompi_coll_tuned_reduce_scatter_block_intra_dec_fixed (sbuf, rbuf, rcount,
+                                                                 dtype, op, comm, module);
+}
+
 /*
  *    allgather_intra_dec
  *
@@ -610,3 +662,89 @@ int ompi_coll_tuned_scatter_intra_dec_dynamic(const void *sbuf, int scount,
                                                     rbuf, rcount, rdtype,
                                                     root, comm, module);
 }
+
+int ompi_coll_tuned_exscan_intra_dec_dynamic(const void *sbuf, void* rbuf, int count,
+                                              struct ompi_datatype_t *dtype,
+                                              struct ompi_op_t *op,
+                                              struct ompi_communicator_t *comm,
+                                              mca_coll_base_module_t *module)
+{
+    mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
+
+    OPAL_OUTPUT((ompi_coll_tuned_stream,
+                 "ompi_coll_tuned_exscan_intra_dec_dynamic"));
+
+    /**
+     * check to see if we have some filebased rules.
+     */
+    if (tuned_module->com_rules[EXSCAN]) {
+        int comsize, alg, faninout, segsize, max_requests;
+        size_t dsize;
+
+        comsize = ompi_comm_size(comm);
+        ompi_datatype_type_size (dtype, &dsize);
+        dsize *= comsize;
+
+        alg = ompi_coll_tuned_get_target_method_params (tuned_module->com_rules[EXSCAN],
+                                                        dsize, &faninout, &segsize, &max_requests);
+
+        if (alg) {
+            /* we have found a valid choice from the file based rules for this message size */
+            return ompi_coll_tuned_exscan_intra_do_this (sbuf, rbuf, count, dtype,
+                                                         op, comm, module,
+                                                         alg);
+        } /* found a method */
+    } /*end if any com rules to check */
+
+    if (tuned_module->user_forced[EXSCAN].algorithm) {
+        return ompi_coll_tuned_exscan_intra_do_this(sbuf, rbuf, count, dtype,
+                                                    op, comm, module,
+                                                    tuned_module->user_forced[EXSCAN].algorithm);
+    }
+
+    return ompi_coll_base_exscan_intra_linear(sbuf, rbuf, count, dtype,
+                                              op, comm, module);
+}
+
+int ompi_coll_tuned_scan_intra_dec_dynamic(const void *sbuf, void* rbuf, int count,
+                                           struct ompi_datatype_t *dtype,
+                                           struct ompi_op_t *op,
+                                           struct ompi_communicator_t *comm,
+                                           mca_coll_base_module_t *module)
+{
+    mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
+
+    OPAL_OUTPUT((ompi_coll_tuned_stream,
+                 "ompi_coll_tuned_scan_intra_dec_dynamic"));
+
+    /**
+     * check to see if we have some filebased rules.
+     */
+    if (tuned_module->com_rules[SCAN]) {
+        int comsize, alg, faninout, segsize, max_requests;
+        size_t dsize;
+
+        comsize = ompi_comm_size(comm);
+        ompi_datatype_type_size (dtype, &dsize);
+        dsize *= comsize;
+
+        alg = ompi_coll_tuned_get_target_method_params (tuned_module->com_rules[SCAN],
+                                                        dsize, &faninout, &segsize, &max_requests);
+
+        if (alg) {
+            /* we have found a valid choice from the file based rules for this message size */
+            return ompi_coll_tuned_scan_intra_do_this (sbuf, rbuf, count, dtype,
+                                                       op, comm, module,
+                                                       alg);
+        } /* found a method */
+    } /*end if any com rules to check */
+
+    if (tuned_module->user_forced[SCAN].algorithm) {
+        return ompi_coll_tuned_scan_intra_do_this(sbuf, rbuf, count, dtype,
+                                                  op, comm, module,
+                                                  tuned_module->user_forced[SCAN].algorithm);
+    }
+
+    return ompi_coll_base_scan_intra_linear(sbuf, rbuf, count, dtype,
+                                            op, comm, module);
+}
