@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2015 University of Houston. All rights reserved.
- * Copyright (c) 2015-2017 Research Organization for Information Science
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
@@ -27,7 +27,7 @@
 #include "ompi/constants.h"
 #include "ompi/mca/fcoll/fcoll.h"
 #include "ompi/mca/fcoll/base/fcoll_base_coll_array.h"
-#include "ompi/mca/io/ompio/io_ompio.h"
+#include "ompi/mca/common/ompio/common_ompio.h"
 #include "ompi/mca/io/io.h"
 #include "math.h"
 #include "ompi/mca/pml/pml.h"
@@ -51,7 +51,7 @@ static int local_heap_sort (mca_io_ompio_local_io_array *io_array,
 
 
 int
-mca_fcoll_dynamic_file_write_all (mca_io_ompio_file_t *fh,
+mca_fcoll_dynamic_file_write_all (ompio_file_t *fh,
                                   const void *buf,
                                   int count,
                                   struct ompi_datatype_t *datatype,
@@ -127,13 +127,13 @@ mca_fcoll_dynamic_file_write_all (mca_io_ompio_file_t *fh,
 
 
     if (! sendbuf_is_contiguous ) {
-        ret =   fh->f_decode_datatype ((struct mca_io_ompio_file_t *) fh,
-                                       datatype,
-                                       count,
-                                       buf,
-                                       &max_data,
-                                       &decoded_iov,
-                                       &iov_count);
+        ret =   mca_common_ompio_decode_datatype ((struct ompio_file_t *) fh,
+                                                  datatype,
+                                                  count,
+                                                  buf,
+                                                  &max_data,
+                                                  &decoded_iov,
+                                                  &iov_count);
         if (OMPI_SUCCESS != ret ){
             goto exit;
         }
@@ -146,15 +146,19 @@ mca_fcoll_dynamic_file_write_all (mca_io_ompio_file_t *fh,
 	status->_ucount = max_data;
     }
 
-    fh->f_get_num_aggregators ( &dynamic_num_io_procs );
-    ret = fh->f_set_aggregator_props ((struct mca_io_ompio_file_t *) fh,
-				      dynamic_num_io_procs,
-				      max_data);
+    dynamic_num_io_procs = fh->f_get_mca_parameter_value ( "num_aggregators", strlen ("num_aggregators"));
+    if ( OMPI_ERR_MAX == dynamic_num_io_procs ) {
+        ret = OMPI_ERROR;
+        goto exit;
+    }
+    ret = mca_common_ompio_set_aggregator_props ((struct ompio_file_t *) fh,
+				                 dynamic_num_io_procs,
+				                 max_data);
 
     if (OMPI_SUCCESS != ret){
 	goto exit;
     }
-    my_aggregator = fh->f_procs_in_group[fh->f_aggregator_index];
+    my_aggregator = fh->f_procs_in_group[0];
     /**************************************************************************
      ** 2. Determine the total amount of data to be written
      **************************************************************************/
@@ -175,7 +179,7 @@ mca_fcoll_dynamic_file_write_all (mca_io_ompio_file_t *fh,
                                            total_bytes_per_process,
                                            1,
                                            MPI_LONG,
-                                           fh->f_aggregator_index,
+                                           0,
                                            fh->f_procs_in_group,
                                            fh->f_procs_per_group,
                                            fh->f_comm);
@@ -201,7 +205,7 @@ mca_fcoll_dynamic_file_write_all (mca_io_ompio_file_t *fh,
      *** 3. Generate the local offsets/lengths array corresponding to
      ***    this write operation
      ********************************************************************/
-    ret = fh->f_generate_current_file_view( (struct mca_io_ompio_file_t *) fh,
+    ret = fh->f_generate_current_file_view( (struct ompio_file_t *) fh,
 					    max_data,
 					    &local_iov_array,
 					    &local_count);
@@ -238,7 +242,7 @@ mca_fcoll_dynamic_file_write_all (mca_io_ompio_file_t *fh,
                                            fview_count,
                                            1,
                                            MPI_INT,
-                                           fh->f_aggregator_index,
+                                           0,
                                            fh->f_procs_in_group,
                                            fh->f_procs_per_group,
                                            fh->f_comm);
@@ -301,7 +305,7 @@ mca_fcoll_dynamic_file_write_all (mca_io_ompio_file_t *fh,
                                             fview_count,
                                             displs,
                                             fh->f_iov_type,
-                                            fh->f_aggregator_index,
+                                            0,
                                             fh->f_procs_in_group,
                                             fh->f_procs_per_group,
                                             fh->f_comm);
@@ -357,7 +361,7 @@ mca_fcoll_dynamic_file_write_all (mca_io_ompio_file_t *fh,
      *** 6. Determine the number of cycles required to execute this
      ***    operation
      *************************************************************/
-    fh->f_get_bytes_per_agg ( (int *)&bytes_per_cycle );
+    bytes_per_cycle = fh->f_bytes_per_agg;
     cycles = ceil((double)total_bytes/bytes_per_cycle);
 
     if (my_aggregator == fh->f_rank) {
@@ -903,8 +907,8 @@ mca_fcoll_dynamic_file_write_all (mca_io_ompio_file_t *fh,
 	    start_write_time = MPI_Wtime();
 #endif
 
-            fh->f_io_array = (mca_io_ompio_io_array_t *) malloc
-                (entries_per_aggregator * sizeof (mca_io_ompio_io_array_t));
+            fh->f_io_array = (mca_common_ompio_io_array_t *) malloc
+                (entries_per_aggregator * sizeof (mca_common_ompio_io_array_t));
             if (NULL == fh->f_io_array) {
                 opal_output(1, "OUT OF MEMORY\n");
                 ret = OMPI_ERR_OUT_OF_RESOURCE;
