@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2016 University of Houston. All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
@@ -45,21 +45,21 @@ OMPI_MODULE_DECLSPEC extern mca_fcoll_base_component_2_0_0_t mca_fcoll_dynamic_g
 int mca_fcoll_dynamic_gen2_component_init_query(bool enable_progress_threads,
                                            bool enable_mpi_threads);
 struct mca_fcoll_base_module_1_0_0_t *
-mca_fcoll_dynamic_gen2_component_file_query (mca_io_ompio_file_t *fh, int *priority);
+mca_fcoll_dynamic_gen2_component_file_query (ompio_file_t *fh, int *priority);
 
-int mca_fcoll_dynamic_gen2_component_file_unquery (mca_io_ompio_file_t *file);
+int mca_fcoll_dynamic_gen2_component_file_unquery (ompio_file_t *file);
 
-int mca_fcoll_dynamic_gen2_module_init (mca_io_ompio_file_t *file);
-int mca_fcoll_dynamic_gen2_module_finalize (mca_io_ompio_file_t *file);
+int mca_fcoll_dynamic_gen2_module_init (ompio_file_t *file);
+int mca_fcoll_dynamic_gen2_module_finalize (ompio_file_t *file);
 
-int mca_fcoll_dynamic_gen2_file_read_all (mca_io_ompio_file_t *fh,
+int mca_fcoll_dynamic_gen2_file_read_all (ompio_file_t *fh,
                                      void *buf,
                                      int count,
                                      struct ompi_datatype_t *datatype,
                                      ompi_status_public_t * status);
 
 
-int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
+int mca_fcoll_dynamic_gen2_file_write_all (ompio_file_t *fh,
                                       const void *buf,
                                       int count,
                                       struct ompi_datatype_t *datatype,
