@@ -10,9 +10,10 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2016 University of Houston. All rights reserved.
- * Copyright (c) 2015-2017 Research Organization for Information Science
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
+ * Copyright (c) 2018      Cisco Systems, Inc.  All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -27,7 +28,7 @@
 #include "ompi/constants.h"
 #include "ompi/mca/fcoll/fcoll.h"
 #include "ompi/mca/fcoll/base/fcoll_base_coll_array.h"
-#include "ompi/mca/io/ompio/io_ompio.h"
+#include "ompi/mca/common/ompio/common_ompio.h"
 #include "ompi/mca/io/io.h"
 #include "math.h"
 #include "ompi/mca/pml/pml.h"
@@ -36,6 +37,7 @@
 
 #define DEBUG_ON 0
 #define FCOLL_DYNAMIC_GEN2_SHUFFLE_TAG   123
+#define INIT_LEN 10
 
 /*Used for loading file-offsets per aggregator*/
 typedef struct mca_io_ompio_local_io_array{
@@ -46,6 +48,7 @@ typedef struct mca_io_ompio_local_io_array{
 
 typedef struct mca_io_ompio_aggregator_data {
     int *disp_index, *sorted, *fview_count, n;
+    int *max_disp_index;
     int **blocklen_per_process;
     MPI_Aint **displs_per_process, total_bytes, bytes_per_cycle, total_bytes_written;
     MPI_Comm comm;
@@ -55,13 +58,11 @@ typedef struct mca_io_ompio_aggregator_data {
     int current_index, current_position;
     int bytes_to_write_in_cycle, bytes_remaining, procs_per_group;    
     int *procs_in_group, iov_index;
-    bool sendbuf_is_contiguous, prev_sendbuf_is_contiguous;
     int bytes_sent, prev_bytes_sent;
     struct iovec *decoded_iov;
     int bytes_to_write, prev_bytes_to_write;
-    mca_io_ompio_io_array_t *io_array, *prev_io_array;
+    mca_common_ompio_io_array_t *io_array, *prev_io_array;
     int num_io_entries, prev_num_io_entries;
-    char *send_buf, *prev_send_buf;
 } mca_io_ompio_aggregator_data;
 
 
@@ -76,9 +77,7 @@ typedef struct mca_io_ompio_aggregator_data {
     for (_i=0; _i<_num; _i++ ) {                                \
         _aggr[_i]->prev_io_array=_aggr[_i]->io_array;             \
         _aggr[_i]->prev_num_io_entries=_aggr[_i]->num_io_entries; \
-        _aggr[_i]->prev_send_buf=_aggr[_i]->send_buf;             \
         _aggr[_i]->prev_bytes_sent=_aggr[_i]->bytes_sent;         \
-        _aggr[_i]->prev_sendbuf_is_contiguous=_aggr[_i]->sendbuf_is_contiguous;   \
         _aggr[_i]->prev_bytes_to_write=_aggr[_i]->bytes_to_write; \
         _t=_aggr[_i]->prev_global_buf;                            \
         _aggr[_i]->prev_global_buf=_aggr[_i]->global_buf;         \
@@ -93,7 +92,7 @@ typedef struct mca_io_ompio_aggregator_data {
 static int shuffle_init ( int index, int cycles, int aggregator, int rank, 
                           mca_io_ompio_aggregator_data *data, 
                           ompi_request_t **reqs );
-static int write_init (mca_io_ompio_file_t *fh, int aggregator, mca_io_ompio_aggregator_data *aggr_data, int write_chunksize );
+static int write_init (ompio_file_t *fh, int aggregator, mca_io_ompio_aggregator_data *aggr_data, int write_chunksize );
 
 int mca_fcoll_dynamic_gen2_break_file_view ( struct iovec *decoded_iov, int iov_count, 
                                         struct iovec *local_iov_array, int local_count, 
@@ -103,7 +102,7 @@ int mca_fcoll_dynamic_gen2_break_file_view ( struct iovec *decoded_iov, int iov_
                                         int stripe_count, int stripe_size); 
 
 
-int mca_fcoll_dynamic_gen2_get_configuration (mca_io_ompio_file_t *fh, int *dynamic_gen2_num_io_procs, 
+int mca_fcoll_dynamic_gen2_get_configuration (ompio_file_t *fh, int *dynamic_gen2_num_io_procs, 
                                               int **ret_aggregators);
 
 
@@ -111,12 +110,12 @@ static int local_heap_sort (mca_io_ompio_local_io_array *io_array,
 			    int num_entries,
 			    int *sorted);
 
-int mca_fcoll_dynamic_gen2_split_iov_array ( mca_io_ompio_file_t *fh, mca_io_ompio_io_array_t *work_array,
+int mca_fcoll_dynamic_gen2_split_iov_array ( ompio_file_t *fh, mca_common_ompio_io_array_t *work_array,
                                              int num_entries, int *last_array_pos, int *last_pos_in_field,
                                              int chunk_size );
 
 
-int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
+int mca_fcoll_dynamic_gen2_file_write_all (ompio_file_t *fh,
                                       const void *buf,
                                       int count,
                                       struct ompi_datatype_t *datatype,
@@ -160,18 +159,19 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
     /**************************************************************************
      ** 1.  In case the data is not contigous in memory, decode it into an iovec
      **************************************************************************/
-    fh->f_get_bytes_per_agg ( (int *)&bytes_per_cycle );
+    bytes_per_cycle = fh->f_bytes_per_agg;
+
     /* since we want to overlap 2 iterations, define the bytes_per_cycle to be half of what
        the user requested */
     bytes_per_cycle =bytes_per_cycle/2;
         
-    ret =   fh->f_decode_datatype ((struct mca_io_ompio_file_t *) fh,
-                                   datatype,
-                                   count,
-                                   buf,
-                                   &max_data,
-                                   &decoded_iov,
-                                   &iov_count);
+    ret =   mca_common_ompio_decode_datatype ((struct ompio_file_t *) fh,
+                                              datatype,
+                                              count,
+                                              buf,
+                                              &max_data,
+                                              &decoded_iov,
+                                              &iov_count);
     if (OMPI_SUCCESS != ret ){
         goto exit;
     }
@@ -188,7 +188,11 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
         dynamic_gen2_num_io_procs =  fh->f_stripe_count;
     }
     else {
-        fh->f_get_num_aggregators ( &dynamic_gen2_num_io_procs );
+        dynamic_gen2_num_io_procs = fh->f_get_mca_parameter_value ( "num_aggregators", strlen ("num_aggregators"));
+        if ( OMPI_ERR_MAX == dynamic_gen2_num_io_procs ) {
+            ret = OMPI_ERROR;
+            goto exit;
+        }
     }
 
 
@@ -221,15 +225,13 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
         aggr_data[i]->procs_in_group  = fh->f_procs_in_group;
         aggr_data[i]->comm = fh->f_comm;
         aggr_data[i]->buf  = (char *)buf;             // should not be used in the new version.
-        aggr_data[i]->sendbuf_is_contiguous      = false;  //safe assumption for right now
-        aggr_data[i]->prev_sendbuf_is_contiguous = false;  //safe assumption for right now
     }
 
     /*********************************************************************
      *** 2. Generate the local offsets/lengths array corresponding to
      ***    this write operation
      ********************************************************************/
-    ret = fh->f_generate_current_file_view( (struct mca_io_ompio_file_t *) fh,
+    ret = fh->f_generate_current_file_view( (struct ompio_file_t *) fh,
 					    max_data,
 					    &local_iov_array,
 					    &local_count);
@@ -253,42 +255,53 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
     /**************************************************************************
      ** 3. Determine the total amount of data to be written and no. of cycles
      **************************************************************************/
-    total_bytes_per_process = (MPI_Aint*)malloc
-        (dynamic_gen2_num_io_procs * fh->f_procs_per_group*sizeof(MPI_Aint));
-    if (NULL == total_bytes_per_process) {
-        opal_output (1, "OUT OF MEMORY\n");
-        ret = OMPI_ERR_OUT_OF_RESOURCE;
-	goto exit;
-    }
-    
 #if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
     start_comm_time = MPI_Wtime();
 #endif
     if ( 1 == mca_fcoll_dynamic_gen2_num_groups ) {
-        ret = fh->f_comm->c_coll->coll_allgather (broken_total_lengths,
-                                                 dynamic_gen2_num_io_procs,
-                                                 MPI_LONG,
-                                                 total_bytes_per_process,
-                                                 dynamic_gen2_num_io_procs,
-                                                 MPI_LONG,
-                                                 fh->f_comm,
-                                                 fh->f_comm->c_coll->coll_allgather_module);
+        ret = fh->f_comm->c_coll->coll_allreduce (MPI_IN_PLACE,
+                                                  broken_total_lengths,
+                                                  dynamic_gen2_num_io_procs,
+                                                  MPI_LONG,
+                                                  MPI_SUM,
+                                                  fh->f_comm,
+                                                  fh->f_comm->c_coll->coll_allreduce_module);
+        if( OMPI_SUCCESS != ret){
+            goto exit;
+        }
     }
     else {
-        ret = ompi_fcoll_base_coll_allgather_array (broken_total_lengths,
-                                               dynamic_gen2_num_io_procs,
-                                               MPI_LONG,
-                                               total_bytes_per_process,
-                                               dynamic_gen2_num_io_procs,
-                                               MPI_LONG,
-                                               0,
-                                               fh->f_procs_in_group,
-                                               fh->f_procs_per_group,
-                                               fh->f_comm);
-    }
+        total_bytes_per_process = (MPI_Aint*)malloc
+            (dynamic_gen2_num_io_procs * fh->f_procs_per_group*sizeof(MPI_Aint));
+        if (NULL == total_bytes_per_process) {
+            opal_output (1, "OUT OF MEMORY\n");
+            ret = OMPI_ERR_OUT_OF_RESOURCE;
+            goto exit;
+        }
     
-    if( OMPI_SUCCESS != ret){
-	goto exit;
+        ret = ompi_fcoll_base_coll_allgather_array (broken_total_lengths,
+                                                    dynamic_gen2_num_io_procs,
+                                                    MPI_LONG,
+                                                    total_bytes_per_process,
+                                                    dynamic_gen2_num_io_procs,
+                                                    MPI_LONG,
+                                                    0,
+                                                    fh->f_procs_in_group,
+                                                    fh->f_procs_per_group,
+                                                    fh->f_comm);
+        if( OMPI_SUCCESS != ret){
+            goto exit;
+        }
+        for ( i=0; i<dynamic_gen2_num_io_procs; i++ ) {
+            broken_total_lengths[i] = 0;
+            for (j=0 ; j<fh->f_procs_per_group ; j++) {
+                broken_total_lengths[i] += total_bytes_per_process[j*dynamic_gen2_num_io_procs + i];
+            }
+        }
+        if (NULL != total_bytes_per_process) {
+            free (total_bytes_per_process);
+            total_bytes_per_process = NULL;
+        }
     }
 #if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
     end_comm_time = MPI_Wtime();
@@ -297,10 +310,6 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
 
     cycles=0;
     for ( i=0; i<dynamic_gen2_num_io_procs; i++ ) {
-        broken_total_lengths[i] = 0;
-        for (j=0 ; j<fh->f_procs_per_group ; j++) {
-            broken_total_lengths[i] += total_bytes_per_process[j*dynamic_gen2_num_io_procs + i];
-        }
 #if DEBUG_ON
         printf("%d: Overall broken_total_lengths[%d] = %ld\n", fh->f_rank, i, broken_total_lengths[i]);
 #endif
@@ -309,10 +318,6 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
         }
     }
     
-    if (NULL != total_bytes_per_process) {
-        free (total_bytes_per_process);
-        total_bytes_per_process = NULL;
-    }
 
     result_counts = (int *) malloc ( dynamic_gen2_num_io_procs * fh->f_procs_per_group * sizeof(int) );
     if ( NULL == result_counts ) {
@@ -496,6 +501,13 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
                 goto exit;
             }
         
+            aggr_data[i]->max_disp_index = (int *)calloc (fh->f_procs_per_group,  sizeof (int));
+            if (NULL == aggr_data[i]->max_disp_index) {
+                opal_output (1, "OUT OF MEMORY\n");
+                ret = OMPI_ERR_OUT_OF_RESOURCE;
+                goto exit;
+            }
+
             aggr_data[i]->blocklen_per_process = (int **)calloc (fh->f_procs_per_group, sizeof (int*));
             if (NULL == aggr_data[i]->blocklen_per_process) {
                 opal_output (1, "OUT OF MEMORY\n");
@@ -603,10 +615,6 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
             end_write_time = MPI_Wtime();
             write_time += end_write_time - start_write_time;
 #endif
-
-            if (!aggr_data[i]->prev_sendbuf_is_contiguous && aggr_data[i]->prev_bytes_sent) {
-                free (aggr_data[i]->prev_send_buf);
-            }
         }
         
     } /* end  for (index = 0; index < cycles; index++) */
@@ -636,10 +644,6 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
             end_write_time = MPI_Wtime();
             write_time += end_write_time - start_write_time;
 #endif
-            
-            if (!aggr_data[i]->prev_sendbuf_is_contiguous && aggr_data[i]->prev_bytes_sent) {
-                free (aggr_data[i]->prev_send_buf);
-            }
         }
     }
 
@@ -684,6 +688,7 @@ exit :
                 }
                 
                 free (aggr_data[i]->disp_index);
+                free (aggr_data[i]->max_disp_index);
                 free (aggr_data[i]->global_buf);
                 free (aggr_data[i]->prev_global_buf);
                 for(l=0;l<aggr_data[i]->procs_per_group;l++){
@@ -703,6 +708,7 @@ exit :
         }
         free (aggr_data);
     }
+    free(local_iov_array);
     free(displs);
     free(decoded_iov);
     free(broken_counts);
@@ -728,7 +734,7 @@ exit :
 }
 
 
-static int write_init (mca_io_ompio_file_t *fh, int aggregator, mca_io_ompio_aggregator_data *aggr_data, int write_chunksize )
+static int write_init (ompio_file_t *fh, int aggregator, mca_io_ompio_aggregator_data *aggr_data, int write_chunksize )
 {
     int ret=OMPI_SUCCESS;
     int last_array_pos=0;
@@ -773,11 +779,12 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
     MPI_Aint *memory_displacements=NULL;
     int *temp_disp_index=NULL;
     MPI_Aint global_count = 0;
+    int* blocklength_proc=NULL;
+    ptrdiff_t* displs_proc=NULL;
 
     data->num_io_entries = 0;
     data->bytes_sent = 0;
     data->io_array=NULL;
-    data->send_buf=NULL;
     /**********************************************************************
      ***  7a. Getting ready for next cycle: initializing and freeing buffers
      **********************************************************************/
@@ -794,16 +801,20 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
         
         for(l=0;l<data->procs_per_group;l++){
             data->disp_index[l] =  1;
-            
-            free(data->blocklen_per_process[l]);
-            free(data->displs_per_process[l]);
-            
-            data->blocklen_per_process[l] = (int *) calloc (1, sizeof(int));
-            data->displs_per_process[l] = (MPI_Aint *) calloc (1, sizeof(MPI_Aint));
-            if (NULL == data->displs_per_process[l] || NULL == data->blocklen_per_process[l]){
-                opal_output (1, "OUT OF MEMORY for displs\n");
-                ret = OMPI_ERR_OUT_OF_RESOURCE;
-                goto exit;
+
+            if(data->max_disp_index[l] == 0) {
+                data->blocklen_per_process[l]   = (int *)       calloc (INIT_LEN, sizeof(int));
+                data->displs_per_process[l]     = (MPI_Aint *)  calloc (INIT_LEN, sizeof(MPI_Aint));
+                if (NULL == data->displs_per_process[l] || NULL == data->blocklen_per_process[l]){
+                    opal_output (1, "OUT OF MEMORY for displs\n");
+                    ret = OMPI_ERR_OUT_OF_RESOURCE;
+                    goto exit;
+                }
+                data->max_disp_index[l] = INIT_LEN;
+            }
+            else {
+                memset ( data->blocklen_per_process[l], 0, data->max_disp_index[l]*sizeof(int) );
+                memset ( data->displs_per_process[l], 0, data->max_disp_index[l]*sizeof(MPI_Aint) );
             }
         }
     } /* (aggregator == rank */
@@ -874,15 +885,22 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
                         (data->global_iov_array[data->sorted[data->current_index]].iov_len
                          - data->bytes_remaining);
                     
+                    data->disp_index[data->n] += 1;
+
                     /* In this cases the length is consumed so allocating for
                        next displacement and blocklength*/
-                    data->blocklen_per_process[data->n] = (int *) realloc
-                        ((void *)data->blocklen_per_process[data->n], (data->disp_index[data->n]+1)*sizeof(int));
-                    data->displs_per_process[data->n] = (MPI_Aint *) realloc
-                        ((void *)data->displs_per_process[data->n], (data->disp_index[data->n]+1)*sizeof(MPI_Aint));
+                    if ( data->disp_index[data->n] == data->max_disp_index[data->n] ) {
+                        data->max_disp_index[data->n] *= 2;
+                        data->blocklen_per_process[data->n] = (int *) realloc(
+                            (void *)data->blocklen_per_process[data->n], 
+                            (data->max_disp_index[data->n])*sizeof(int));
+                        data->displs_per_process[data->n] = (MPI_Aint *) realloc(
+                            (void *)data->displs_per_process[data->n], 
+                            (data->max_disp_index[data->n])*sizeof(MPI_Aint));
+                    }
+
                     data->blocklen_per_process[data->n][data->disp_index[data->n]] = 0;
                     data->displs_per_process[data->n][data->disp_index[data->n]] = 0;
-                    data->disp_index[data->n] += 1;
                 }
                 if (data->procs_in_group[data->n] == rank) {
                     bytes_sent += data->bytes_remaining;
@@ -890,7 +908,6 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
                 data->current_index ++;
                 data->bytes_to_write_in_cycle -= data->bytes_remaining;
                 data->bytes_remaining = 0;
-//                continue;
             }
             else {
                 /* the remaining data from the previous cycle is larger than the
@@ -938,16 +955,22 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
                     data->displs_per_process[data->n][data->disp_index[data->n] - 1] = (ptrdiff_t)
                         data->global_iov_array[data->sorted[data->current_index]].iov_base;
                     
+                    data->disp_index[data->n] += 1;
+
                     /*realloc for next blocklength
                       and assign this displacement and check for next displs as
                       the total length of this entry has been consumed!*/
-                    data->blocklen_per_process[data->n] =
-                        (int *) realloc ((void *)data->blocklen_per_process[data->n], (data->disp_index[data->n]+1)*sizeof(int));
-                    data->displs_per_process[data->n] = (MPI_Aint *)realloc
-                        ((void *)data->displs_per_process[data->n], (data->disp_index[data->n]+1)*sizeof(MPI_Aint));
+                    if ( data->disp_index[data->n] == data->max_disp_index[data->n] ) {
+                        data->max_disp_index[data->n] *= 2;
+                        data->blocklen_per_process[data->n] = (int *) realloc(
+                            (void *)data->blocklen_per_process[data->n], 
+                            (data->max_disp_index[data->n]*sizeof(int)));
+                        data->displs_per_process[data->n] = (MPI_Aint *)realloc(
+                            (void *)data->displs_per_process[data->n], 
+                            (data->max_disp_index[data->n]*sizeof(MPI_Aint)));
+                    }
                     data->blocklen_per_process[data->n][data->disp_index[data->n]] = 0;
                     data->displs_per_process[data->n][data->disp_index[data->n]] = 0;
-                    data->disp_index[data->n] += 1;
                 }
                 if (data->procs_in_group[data->n] == rank) {
                     bytes_sent += data->global_iov_array[data->sorted[data->current_index]].iov_len;
@@ -955,7 +978,6 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
                 data->bytes_to_write_in_cycle -=
                     data->global_iov_array[data->sorted[data->current_index]].iov_len;
                 data->current_index ++;
-//                continue;
             }
         }
     }
@@ -1135,73 +1157,86 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
             }
         }  /* end if (entries_per_aggr > 0 ) */
     }/* end if (aggregator == rank ) */
-    
-    if ( data->sendbuf_is_contiguous ) {
-        data->send_buf = &((char*)data->buf)[data->total_bytes_written];
-    }
-    else if (bytes_sent) {
-        /* allocate a send buffer and copy the data that needs
-           to be sent into it in case the data is non-contigous
-           in memory */
-        ptrdiff_t mem_address;
-        size_t remaining = 0;
-        size_t temp_position = 0;
-        
-        data->send_buf = malloc (bytes_sent);
-        if (NULL == data->send_buf) {
+
+    if (bytes_sent) {
+        size_t remaining      = bytes_sent;
+        int block_index       = -1;
+        int blocklength_size  = INIT_LEN;
+
+        ptrdiff_t send_mem_address  = (ptrdiff_t) NULL;
+        ompi_datatype_t *newType    = MPI_DATATYPE_NULL;
+        blocklength_proc            = (int *)       calloc (blocklength_size, sizeof (int));
+        displs_proc                 = (ptrdiff_t *) calloc (blocklength_size, sizeof (ptrdiff_t));
+
+        if (NULL == blocklength_proc || NULL == displs_proc ) {
             opal_output (1, "OUT OF MEMORY\n");
             ret = OMPI_ERR_OUT_OF_RESOURCE;
             goto exit;
         }
-        
-        remaining = bytes_sent;
-        
+
         while (remaining) {
-            mem_address = (ptrdiff_t)
-                (data->decoded_iov[data->iov_index].iov_base) + data->current_position;
-            
+            block_index++;
+
+            if(0 == block_index) {
+                send_mem_address = (ptrdiff_t) (data->decoded_iov[data->iov_index].iov_base) +
+                                                data->current_position;
+            }
+            else {
+                // Reallocate more memory if blocklength_size is not enough
+                if(0 == block_index % INIT_LEN) {
+                    blocklength_size += INIT_LEN;
+                    blocklength_proc = (int *)       realloc(blocklength_proc, blocklength_size * sizeof(int));
+                    displs_proc      = (ptrdiff_t *) realloc(displs_proc, blocklength_size * sizeof(ptrdiff_t));
+                }
+                displs_proc[block_index] = (ptrdiff_t) (data->decoded_iov[data->iov_index].iov_base) +
+                                                        data->current_position - send_mem_address;
+            }
+
             if (remaining >=
                 (data->decoded_iov[data->iov_index].iov_len - data->current_position)) {
-                memcpy (data->send_buf+temp_position,
-                        (IOVBASE_TYPE *)mem_address,
-                        data->decoded_iov[data->iov_index].iov_len - data->current_position);
+
+                blocklength_proc[block_index] = data->decoded_iov[data->iov_index].iov_len -
+                                                data->current_position;
                 remaining = remaining -
-                    (data->decoded_iov[data->iov_index].iov_len - data->current_position);
-                temp_position = temp_position +
-                    (data->decoded_iov[data->iov_index].iov_len - data->current_position);
+                            (data->decoded_iov[data->iov_index].iov_len - data->current_position);
                 data->iov_index = data->iov_index + 1;
                 data->current_position = 0;
             }
             else {
-                memcpy (data->send_buf+temp_position,
-                        (IOVBASE_TYPE *) mem_address,
-                        remaining);
+                blocklength_proc[block_index] = remaining;
                 data->current_position += remaining;
                 remaining = 0;
             }
         }
-    }
-    data->total_bytes_written += bytes_sent;
-    data->bytes_sent = bytes_sent;
-    /* Gather the sendbuf from each process in appropritate locations in
-       aggregators*/
-    
-    if (bytes_sent){
-        ret = MCA_PML_CALL(isend(data->send_buf,
-                                 bytes_sent,
-                                 MPI_BYTE,
-                                 aggregator,
-                                 FCOLL_DYNAMIC_GEN2_SHUFFLE_TAG+index,
-                                 MCA_PML_BASE_SEND_STANDARD,
-                                 data->comm,
-                                 &reqs[data->procs_per_group]));
-        
-        
-        if ( OMPI_SUCCESS != ret ){
-            goto exit;
+
+        data->total_bytes_written += bytes_sent;
+        data->bytes_sent = bytes_sent;
+
+        if ( 0 <= block_index ) {
+            ompi_datatype_create_hindexed(block_index+1,
+                                          blocklength_proc,
+                                          displs_proc,
+                                          MPI_BYTE,
+                                          &newType);
+            ompi_datatype_commit(&newType);
+
+            ret = MCA_PML_CALL(isend((char *)send_mem_address,
+                                     1,
+                                     newType,
+                                     aggregator,
+                                     FCOLL_DYNAMIC_GEN2_SHUFFLE_TAG+index,
+                                     MCA_PML_BASE_SEND_STANDARD,
+                                     data->comm,
+                                     &reqs[data->procs_per_group]));
+            if ( MPI_DATATYPE_NULL != newType ) {
+                ompi_datatype_destroy(&newType);
+            }
+            if (OMPI_SUCCESS != ret){
+                goto exit;
+            }
         }
-        
     }
+
     
 #if DEBUG_ON
     if (aggregator == rank){
@@ -1223,8 +1258,8 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
     if (aggregator == rank && entries_per_aggregator>0) {
         
         
-        data->io_array = (mca_io_ompio_io_array_t *) malloc
-            (entries_per_aggregator * sizeof (mca_io_ompio_io_array_t));
+        data->io_array = (mca_common_ompio_io_array_t *) malloc
+            (entries_per_aggregator * sizeof (mca_common_ompio_io_array_t));
         if (NULL == data->io_array) {
             opal_output(1, "OUT OF MEMORY\n");
             ret = OMPI_ERR_OUT_OF_RESOURCE;
@@ -1278,6 +1313,8 @@ exit:
     free(sorted_file_offsets);
     free(file_offsets_for_agg);
     free(memory_displacements);
+    free(blocklength_proc);
+    free(displs_proc);
     
     return OMPI_SUCCESS;
 }
@@ -1342,7 +1379,8 @@ int mca_fcoll_dynamic_gen2_break_file_view ( struct iovec *mem_iov, int mem_coun
     }
     
     /* Step 1: separate the local_iov_array per aggregator */
-    int owner, rest, len, temp_len, blocklen, memlen=0;
+    int owner;
+    size_t rest, len, temp_len, blocklen, memlen=0;
     off_t offset, temp_offset, start_offset, memoffset=0;
 
     i=j=0;
@@ -1511,7 +1549,7 @@ exit:
 }
 
 
-int mca_fcoll_dynamic_gen2_get_configuration (mca_io_ompio_file_t *fh, int *dynamic_gen2_num_io_procs, int **ret_aggregators)
+int mca_fcoll_dynamic_gen2_get_configuration (ompio_file_t *fh, int *dynamic_gen2_num_io_procs, int **ret_aggregators)
 {
     int *aggregators=NULL;
     int num_io_procs = *dynamic_gen2_num_io_procs;
@@ -1553,7 +1591,7 @@ int mca_fcoll_dynamic_gen2_get_configuration (mca_io_ompio_file_t *fh, int *dyna
 }    
     
 
-int mca_fcoll_dynamic_gen2_split_iov_array ( mca_io_ompio_file_t *fh, mca_io_ompio_io_array_t *io_array, int num_entries,
+int mca_fcoll_dynamic_gen2_split_iov_array ( ompio_file_t *fh, mca_common_ompio_io_array_t *io_array, int num_entries,
                                              int *ret_array_pos, int *ret_pos,  int chunk_size )
 {
 
@@ -1563,7 +1601,7 @@ int mca_fcoll_dynamic_gen2_split_iov_array ( mca_io_ompio_file_t *fh, mca_io_omp
     size_t bytes_to_write = chunk_size;
 
     if ( 0 == array_pos && 0 == pos ) {
-        fh->f_io_array = (mca_io_ompio_io_array_t *) malloc ( num_entries * sizeof(mca_io_ompio_io_array_t));
+        fh->f_io_array = (mca_common_ompio_io_array_t *) malloc ( num_entries * sizeof(mca_common_ompio_io_array_t));
         if ( NULL == fh->f_io_array ){
             opal_output (1,"Could not allocate memory\n");
             return -1;
