@@ -11,7 +11,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2014 University of Houston. All rights reserved.
- * Copyright (c) 2015-2017 Research Organization for Information Science
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2015-2016 Los Alamos National Security, LLC. All rights
  *                         reserved.
@@ -29,7 +29,7 @@
 #include "ompi/constants.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/mca/fcoll/fcoll.h"
-#include "ompi/mca/io/ompio/io_ompio.h"
+#include "ompi/mca/common/ompio/common_ompio.h"
 #include "ompi/mca/io/io.h"
 #include "opal/mca/base/base.h"
 #include "math.h"
@@ -56,10 +56,10 @@ typedef struct flat_list_node {
 
 
 /* local function declarations  */
-static int two_phase_exch_and_write(mca_io_ompio_file_t *fh,
+static int two_phase_exch_and_write(ompio_file_t *fh,
 				    const void *buf,
 				    MPI_Datatype datatype,
-				    mca_io_ompio_access_array_t *others_req,
+				    mca_common_ompio_access_array_t *others_req,
 				    struct iovec *offset_len,
 				    int contig_access_count,
 				    OMPI_MPI_OFFSET_TYPE min_st_offset,
@@ -72,7 +72,7 @@ static int two_phase_exch_and_write(mca_io_ompio_file_t *fh,
 
 
 
-static int  two_phase_exchage_data(mca_io_ompio_file_t *fh,
+static int  two_phase_exchage_data(ompio_file_t *fh,
 				   const void *buf,
 				   char *write_buf,
 				   struct iovec *offset_length,
@@ -87,7 +87,7 @@ static int  two_phase_exchage_data(mca_io_ompio_file_t *fh,
 				   OMPI_MPI_OFFSET_TYPE *fd_start,
 				   OMPI_MPI_OFFSET_TYPE *fd_end,
 				   Flatlist_node *flat_buf,
-				   mca_io_ompio_access_array_t *others_req,
+				   mca_common_ompio_access_array_t *others_req,
 				   int *send_buf_idx, int *curr_to_proc,
 				   int *done_to_proc, int iter,
 				   size_t *buf_idx, MPI_Aint buftype_extent,
@@ -95,7 +95,7 @@ static int  two_phase_exchage_data(mca_io_ompio_file_t *fh,
 				   int *aggregator_list,  int *hole);
 
 
-static int two_phase_fill_send_buffer(mca_io_ompio_file_t *fh,
+static int two_phase_fill_send_buffer(ompio_file_t *fh,
 				      const void *buf,
 				      Flatlist_node *flat_buf,
 				      char **send_buf,
@@ -120,7 +120,7 @@ static int is_aggregator(int rank,
 			 int *aggregator_list);
 #endif
 
-void two_phase_heap_merge(mca_io_ompio_access_array_t *others_req,
+void two_phase_heap_merge(mca_common_ompio_access_array_t *others_req,
 			  int *count,
 			  OMPI_MPI_OFFSET_TYPE *srt_off,
 			  int *srt_len,
@@ -141,7 +141,7 @@ double exch_write = 0.0, start_exch = 0.0, end_exch = 0.0;
 #endif
 
 int
-mca_fcoll_two_phase_file_write_all (mca_io_ompio_file_t *fh,
+mca_fcoll_two_phase_file_write_all (ompio_file_t *fh,
                                     const void *buf,
                                     int count,
                                     struct ompi_datatype_t *datatype,
@@ -166,7 +166,7 @@ mca_fcoll_two_phase_file_write_all (mca_io_ompio_file_t *fh,
     OMPI_MPI_OFFSET_TYPE *start_offsets=NULL, *end_offsets=NULL;
     OMPI_MPI_OFFSET_TYPE *fd_start=NULL, *fd_end=NULL, min_st_offset;
     Flatlist_node *flat_buf=NULL;
-    mca_io_ompio_access_array_t *my_req=NULL, *others_req=NULL;
+    mca_common_ompio_access_array_t *my_req=NULL, *others_req=NULL;
     MPI_Aint send_buf_addr;
 #if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
     mca_common_ompio_print_entry nentry;
@@ -180,13 +180,13 @@ mca_fcoll_two_phase_file_write_all (mca_io_ompio_file_t *fh,
 
     if (! (fh->f_flags & OMPIO_CONTIGUOUS_MEMORY)) {
 
-	ret =   fh->f_decode_datatype ((struct mca_io_ompio_file_t *)fh,
-				       datatype,
-				       count,
-				       buf,
-				       &max_data,
-				       &temp_iov,
-				       &iov_count);
+	ret =   mca_common_ompio_decode_datatype ((struct ompio_file_t *)fh,
+				                  datatype,
+				                  count,
+				                  buf,
+				                  &max_data,
+				                  &temp_iov,
+				                  &iov_count);
 	if (OMPI_SUCCESS != ret ){
 	    goto exit;
 	}
@@ -222,16 +222,20 @@ mca_fcoll_two_phase_file_write_all (mca_io_ompio_file_t *fh,
 	status->_ucount = max_data;
     }
 
-    fh->f_get_num_aggregators ( &two_phase_num_io_procs );
+    two_phase_num_io_procs = fh->f_get_mca_parameter_value ( "num_aggregators", strlen ("num_aggregators"));
+    if ( OMPI_ERR_MAX == two_phase_num_io_procs ) {
+        ret = OMPI_ERROR;
+        goto exit;
+    }
     if(-1 == two_phase_num_io_procs){
-	ret = fh->f_set_aggregator_props ((struct mca_io_ompio_file_t *)fh,
-					  two_phase_num_io_procs,
-					  max_data);
+	ret = mca_common_ompio_set_aggregator_props ((struct ompio_file_t *)fh,
+					             two_phase_num_io_procs,
+					             max_data);
 	if ( OMPI_SUCCESS != ret){
             goto exit;
 	}
 
-	two_phase_num_io_procs = fh->f_final_num_aggrs;
+	two_phase_num_io_procs = fh->f_num_aggrs;
 
     }
 
@@ -239,7 +243,6 @@ mca_fcoll_two_phase_file_write_all (mca_io_ompio_file_t *fh,
         two_phase_num_io_procs = fh->f_size;
     }
 
-
 #if DEBUG_ON
     printf("Number of aggregators : %ld\n", two_phase_num_io_procs);
 #endif
@@ -261,7 +264,7 @@ mca_fcoll_two_phase_file_write_all (mca_io_ompio_file_t *fh,
         }
     }        
 
-    ret = fh->f_generate_current_file_view ((struct mca_io_ompio_file_t*)fh,
+    ret = fh->f_generate_current_file_view ((struct ompio_file_t*)fh,
 					    max_data,
 					    &iov,
 					    &local_count);
@@ -587,10 +590,10 @@ exit :
 }
 
 
-static int two_phase_exch_and_write(mca_io_ompio_file_t *fh,
+static int two_phase_exch_and_write(ompio_file_t *fh,
 				    const void *buf,
 				    MPI_Datatype datatype,
-				    mca_io_ompio_access_array_t *others_req,
+				    mca_common_ompio_access_array_t *others_req,
 				    struct iovec *offset_len,
 				    int contig_access_count,
 				    OMPI_MPI_OFFSET_TYPE min_st_offset,
@@ -643,7 +646,7 @@ static int two_phase_exch_and_write(mca_io_ompio_file_t *fh,
 	}
     }
 
-    fh->f_get_bytes_per_agg ( &two_phase_cycle_buffer_size );
+    two_phase_cycle_buffer_size = fh->f_bytes_per_agg;
     ntimes = (int) ((end_loc - st_loc + two_phase_cycle_buffer_size)/two_phase_cycle_buffer_size);
 
     if ((st_loc == -1) && (end_loc == -1)) {
@@ -774,7 +777,7 @@ static int two_phase_exch_and_write(mca_io_ompio_file_t *fh,
 			       size,i,
 			       count[i]);
 #endif
-			PMPI_Address(write_buf+req_off-off,
+			PMPI_Get_address(write_buf+req_off-off,
 				     &(others_req[i].mem_ptrs[j]));
 #if DEBUG_ON
 			printf("%d : mem_ptrs : %ld\n", fh->f_rank,
@@ -835,8 +838,8 @@ static int two_phase_exch_and_write(mca_io_ompio_file_t *fh,
 	    }
 #endif
 	    len = size * byte_size;
-	    fh->f_io_array = (mca_io_ompio_io_array_t *)malloc
-		(sizeof(mca_io_ompio_io_array_t));
+	    fh->f_io_array = (mca_common_ompio_io_array_t *)malloc
+		(sizeof(mca_common_ompio_io_array_t));
 	    if (NULL == fh->f_io_array) {
 		opal_output(1, "OUT OF MEMORY\n");
 		ret = OMPI_ERR_OUT_OF_RESOURCE;
@@ -921,7 +924,7 @@ exit:
     return ret;
 }
 
-static int two_phase_exchage_data(mca_io_ompio_file_t *fh,
+static int two_phase_exchage_data(ompio_file_t *fh,
 				  const void *buf,
 				  char *write_buf,
 				  struct iovec *offset_length,
@@ -936,7 +939,7 @@ static int two_phase_exchage_data(mca_io_ompio_file_t *fh,
 				  OMPI_MPI_OFFSET_TYPE *fd_start,
 				  OMPI_MPI_OFFSET_TYPE *fd_end,
 				  Flatlist_node *flat_buf,
-				  mca_io_ompio_access_array_t *others_req,
+				  mca_common_ompio_access_array_t *others_req,
 				  int *send_buf_idx, int *curr_to_proc,
 				  int *done_to_proc, int iter,
 				  size_t *buf_idx,MPI_Aint buftype_extent,
@@ -1062,8 +1065,8 @@ static int two_phase_exchage_data(mca_io_ompio_file_t *fh,
     if (nprocs_recv){
 	if (*hole){
 	    if (off >= 0){
-		fh->f_io_array = (mca_io_ompio_io_array_t *)malloc
-		    (sizeof(mca_io_ompio_io_array_t));
+		fh->f_io_array = (mca_common_ompio_io_array_t *)malloc
+		    (sizeof(mca_common_ompio_io_array_t));
 		if (NULL == fh->f_io_array) {
 		    opal_output(1, "OUT OF MEMORY\n");
                     ret = OMPI_ERR_OUT_OF_RESOURCE;
@@ -1074,6 +1077,23 @@ static int two_phase_exchage_data(mca_io_ompio_file_t *fh,
 		fh->f_io_array[0].length = size;
 		fh->f_io_array[0].memory_address = write_buf;
 		if (fh->f_num_of_io_entries){
+                    int amode_overwrite;
+                    amode_overwrite = fh->f_get_mca_parameter_value ("overwrite_amode", strlen("overwrite_amode"));
+                    if ( OMPI_ERR_MAX == amode_overwrite ) {
+                        ret = OMPI_ERROR;
+                        goto exit;
+                    }
+                    if ( fh->f_amode & MPI_MODE_WRONLY && !amode_overwrite ){
+                        if ( 0 == fh->f_rank ) {
+                            printf("\n File not opened in RDWR mode, can not continue."
+                                   "\n To resolve this problem, you can either \n"
+                                   "  a. open the file with MPI_MODE_RDWR instead of MPI_MODE_WRONLY\n"
+                                   "  b. ensure that the mca parameter mca_io_ompio_overwrite_amode is set to 1\n"
+                                   "  c. use an fcoll component that does not use data sieving (e.g. dynamic)\n");
+                        }
+                        ret = MPI_ERR_FILE;
+                        goto exit;
+                    }
 		    if ( 0 >  fh->f_fbtl->fbtl_preadv (fh)) {
 			opal_output(1, "READ FAILED\n");
                         ret = OMPI_ERROR;
@@ -1263,7 +1283,7 @@ exit:
 
 
 
-static int two_phase_fill_send_buffer(mca_io_ompio_file_t *fh,
+static int two_phase_fill_send_buffer(ompio_file_t *fh,
 				      const void *buf,
 				      Flatlist_node *flat_buf,
 				      char **send_buf,
@@ -1383,7 +1403,7 @@ static int two_phase_fill_send_buffer(mca_io_ompio_file_t *fh,
 
 
 
-void two_phase_heap_merge( mca_io_ompio_access_array_t *others_req,
+void two_phase_heap_merge( mca_common_ompio_access_array_t *others_req,
 			   int *count,
 			   OMPI_MPI_OFFSET_TYPE *srt_off,
 			   int *srt_len,
