@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2016 University of Houston. All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
@@ -41,11 +41,11 @@ BEGIN_C_DECLS
 int mca_fs_ufs_component_init_query(bool enable_progress_threads,
                                         bool enable_mpi_threads);
 struct mca_fs_base_module_1_0_0_t *
-mca_fs_ufs_component_file_query (mca_io_ompio_file_t *fh, int *priority);
-int mca_fs_ufs_component_file_unquery (mca_io_ompio_file_t *file);
+mca_fs_ufs_component_file_query (ompio_file_t *fh, int *priority);
+int mca_fs_ufs_component_file_unquery (ompio_file_t *file);
 
-int mca_fs_ufs_module_init (mca_io_ompio_file_t *file);
-int mca_fs_ufs_module_finalize (mca_io_ompio_file_t *file);
+int mca_fs_ufs_module_init (ompio_file_t *file);
+int mca_fs_ufs_module_finalize (ompio_file_t *file);
 
 OMPI_MODULE_DECLSPEC extern mca_fs_base_component_2_0_0_t mca_fs_ufs_component;
 /*
@@ -58,22 +58,22 @@ int mca_fs_ufs_file_open (struct ompi_communicator_t *comm,
                           const char *filename,
                           int amode,
                           struct opal_info_t *info,
-                          mca_io_ompio_file_t *fh);
+                          ompio_file_t *fh);
 
-int mca_fs_ufs_file_close (mca_io_ompio_file_t *fh);
+int mca_fs_ufs_file_close (ompio_file_t *fh);
 
 int mca_fs_ufs_file_delete (char *filename,
                             struct opal_info_t *info);
 
-int mca_fs_ufs_file_set_size (mca_io_ompio_file_t *fh,
+int mca_fs_ufs_file_set_size (ompio_file_t *fh,
                               OMPI_MPI_OFFSET_TYPE size);
 
-int mca_fs_ufs_file_get_size (mca_io_ompio_file_t *fh,
+int mca_fs_ufs_file_get_size (ompio_file_t *fh,
                               OMPI_MPI_OFFSET_TYPE *size);
 
-int mca_fs_ufs_file_sync (mca_io_ompio_file_t *fh);
+int mca_fs_ufs_file_sync (ompio_file_t *fh);
 
-int mca_fs_ufs_file_seek (mca_io_ompio_file_t *fh,
+int mca_fs_ufs_file_seek (ompio_file_t *fh,
                           OMPI_MPI_OFFSET_TYPE offset,
                           int whence);
 /*
