@@ -14,7 +14,7 @@
 
 #include "mtl_ofi.h"
 #include "opal/util/argv.h"
-#include "opal/util/show_help.h"
+#include "opal/util/printf.h"
 
 static int ompi_mtl_ofi_component_open(void);
 static int ompi_mtl_ofi_component_query(mca_base_module_t **module, int *priority);
@@ -293,7 +293,7 @@ is_in_list(char **list, char *item)
     }
 
     while (NULL != list[i]) {
-        if (0 == strncmp(item, list[i], strlen(list[i]))) {
+        if (0 == strncasecmp(item, list[i], strlen(list[i]))) {
             return 1;
         } else {
             i++;
@@ -576,6 +576,8 @@ ompi_mtl_ofi_component_init(bool enable_progress_threads,
         ompi_mtl_ofi_define_tag_mode(ofi_tag_mode);
     }
 
+    ompi_mtl_ofi.num_peers = 0;
+
     /**
      * Open fabric
      * The getinfo struct returns a fabric attribute struct that can be used to
@@ -630,9 +632,10 @@ ompi_mtl_ofi_component_init(bool enable_progress_threads,
     }
 
     /**
-     * Save the maximum inject size.
+     * Save the maximum sizes.
      */
     ompi_mtl_ofi.max_inject_size = prov->tx_attr->inject_size;
+    ompi_mtl_ofi.max_msg_size = prov->ep_attr->max_msg_size;
 
     /**
      * Create the objects that will be bound to the endpoint.
@@ -708,6 +711,8 @@ ompi_mtl_ofi_component_init(bool enable_progress_threads,
         goto error;
     }
 
+    ompi_mtl_ofi.provider_name = strdup(prov->fabric_attr->prov_name);
+
     /**
      * Free providers info since it's not needed anymore.
      */
