@@ -75,6 +75,10 @@ static mca_base_var_enum_value_t long_protocol_values[] = {
     {0, NULL}
 };
 
+OBJ_CLASS_INSTANCE(ompi_mtl_portals4_rndv_get_frag_t,
+                   opal_free_list_item_t,
+                   NULL, NULL);
+
 static int
 ompi_mtl_portals4_component_register(void)
 {
@@ -198,6 +202,16 @@ ompi_mtl_portals4_component_register(void)
                                            MCA_BASE_VAR_SCOPE_READONLY,
                                            &ompi_mtl_portals4.max_msg_size_mtl);
 
+    ompi_mtl_portals4.get_retransmit_timeout=10000;
+    (void) mca_base_component_var_register(&mca_mtl_portals4_component.mtl_version,
+                                           "get_retransmit_timeout",
+                                           "PtlGET retransmission timeout in usec",
+                                           MCA_BASE_VAR_TYPE_INT,
+                                           NULL, 0, 0,
+                                           OPAL_INFO_LVL_5,
+                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           &ompi_mtl_portals4.get_retransmit_timeout);
+
     OBJ_RELEASE(new_enum);
     if (0 > ret) {
         return OMPI_ERR_NOT_SUPPORTED;
@@ -251,6 +265,13 @@ ompi_mtl_portals4_component_open(void)
                         OBJ_CLASS(ompi_mtl_portals4_message_t),
                         0, 0, 1, -1, 1, NULL, 0, NULL, NULL, NULL);
 
+    OBJ_CONSTRUCT(&ompi_mtl_portals4.fl_rndv_get_frag, opal_free_list_t);
+    opal_free_list_init(&ompi_mtl_portals4.fl_rndv_get_frag,
+                        sizeof(ompi_mtl_portals4_rndv_get_frag_t),
+                        opal_cache_line_size,
+                        OBJ_CLASS(ompi_mtl_portals4_rndv_get_frag_t),
+                        0, 0, 1, -1, 1, NULL, 0, NULL, NULL, NULL);
+
     ompi_mtl_portals4.ni_h = PTL_INVALID_HANDLE;
     ompi_mtl_portals4.send_eq_h = PTL_INVALID_HANDLE;
     ompi_mtl_portals4.recv_eq_h = PTL_INVALID_HANDLE;
@@ -478,6 +499,7 @@ ompi_mtl_portals4_progress(void)
     unsigned int which;
     ptl_event_t ev;
     ompi_mtl_portals4_base_request_t *ptl_request;
+    ompi_mtl_portals4_rndv_get_frag_t *rndv_get_frag;
 
     while (true) {
 	ret = PtlEQPoll(ompi_mtl_portals4.eqs_h, 2, 0, &ev, &which);
@@ -489,7 +511,6 @@ ompi_mtl_portals4_progress(void)
             case PTL_EVENT_GET:
             case PTL_EVENT_PUT:
             case PTL_EVENT_PUT_OVERFLOW:
-            case PTL_EVENT_REPLY:
             case PTL_EVENT_SEND:
             case PTL_EVENT_ACK:
             case PTL_EVENT_AUTO_FREE:
@@ -507,6 +528,18 @@ ompi_mtl_portals4_progress(void)
                 }
                 break;
 
+            case PTL_EVENT_REPLY:
+                if (NULL != ev.user_ptr) {
+                    rndv_get_frag = ev.user_ptr;
+                    ret = rndv_get_frag->event_callback(&ev, rndv_get_frag);
+                    if (OMPI_SUCCESS != ret) {
+                        opal_output(ompi_mtl_base_framework.framework_output,
+                                    "Error returned from target event callback: %d", ret);
+                        abort();
+                    }
+                }
+                break;
+
             case PTL_EVENT_PT_DISABLED:
 #if OMPI_MTL_PORTALS4_FLOW_CONTROL
                 OPAL_OUTPUT_VERBOSE((10, ompi_mtl_base_framework.framework_output,
