@@ -7,6 +7,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -41,7 +42,7 @@ int ompi_osc_base_select(ompi_win_t *win,
                          size_t size,
                          int disp_unit,
                          ompi_communicator_t *comm,
-                         ompi_info_t *info,
+                         opal_info_t *info,
                          int flavor,
                          int *model);
 
