@@ -12,10 +12,18 @@
 #include "ompi/mca/osc/osc.h"
 #include "ompi/mca/osc/base/base.h"
 #include "ompi/mca/osc/base/osc_base_obj_convert.h"
+#include "opal/mca/common/ucx/common_ucx.h"
 
 #include "osc_ucx.h"
 #include "osc_ucx_request.h"
 
+
+#define CHECK_VALID_RKEY(_module, _target, _count)                               \
+    if (!((_module)->win_info_array[_target]).rkey_init && ((_count) > 0)) {     \
+        OSC_UCX_VERBOSE(1, "window with non-zero length does not have an rkey"); \
+        return OMPI_ERROR;                                                       \
+    }
+
 typedef struct ucx_iovec {
     void *addr;
     size_t len;
@@ -60,19 +68,14 @@ static inline int check_sync_state(ompi_osc_ucx_module_t *module, int target,
 
 static inline int incr_and_check_ops_num(ompi_osc_ucx_module_t *module, int target,
                                          ucp_ep_h ep) {
-    ucs_status_t status;
+    int status;
 
     module->global_ops_num++;
     module->per_target_ops_nums[target]++;
     if (module->global_ops_num >= OSC_UCX_OPS_THRESHOLD) {
-        /* TODO: ucp_ep_flush needs to be replaced with its non-blocking counterpart
-         * when it is implemented in UCX */
-        status = ucp_ep_flush(ep);
-        if (status != UCS_OK) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_ep_flush failed: %d\n",
-                                __FILE__, __LINE__, status);
-            return OMPI_ERROR;
+        status = opal_common_ucx_ep_flush(ep, mca_osc_ucx_component.ucp_worker);
+        if (status != OMPI_SUCCESS) {
+            return status;
         }
         module->global_ops_num -= module->per_target_ops_nums[target];
         module->per_target_ops_nums[target] = 0;
@@ -169,18 +172,14 @@ static inline int ddt_put_get(ompi_osc_ucx_module_t *module,
                 status = ucp_put_nbi(ep, origin_ucx_iov[origin_ucx_iov_idx].addr, curr_len,
                                      remote_addr + (uint64_t)(target_ucx_iov[target_ucx_iov_idx].addr), rkey);
                 if (status != UCS_OK && status != UCS_INPROGRESS) {
-                    opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                        "%s:%d: ucp_put_nbi failed: %d\n",
-                                        __FILE__, __LINE__, status);
+                    OSC_UCX_VERBOSE(1, "ucp_put_nbi failed: %d", status);
                     return OMPI_ERROR;
                 }
             } else {
                 status = ucp_get_nbi(ep, origin_ucx_iov[origin_ucx_iov_idx].addr, curr_len,
                                      remote_addr + (uint64_t)(target_ucx_iov[target_ucx_iov_idx].addr), rkey);
                 if (status != UCS_OK && status != UCS_INPROGRESS) {
-                    opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                        "%s:%d: ucp_get_nbi failed: %d\n",
-                                        __FILE__, __LINE__, status);
+                    OSC_UCX_VERBOSE(1, "ucp_get_nbi failed: %d",status);
                     return OMPI_ERROR;
                 }
             }
@@ -214,9 +213,7 @@ static inline int ddt_put_get(ompi_osc_ucx_module_t *module,
                                      origin_ucx_iov[origin_ucx_iov_idx].len,
                                      remote_addr + target_lb + prev_len, rkey);
                 if (status != UCS_OK && status != UCS_INPROGRESS) {
-                    opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                        "%s:%d: ucp_put_nbi failed: %d\n",
-                                        __FILE__, __LINE__, status);
+                    OSC_UCX_VERBOSE(1, "ucp_put_nbi failed: %d", status);
                     return OMPI_ERROR;
                 }
             } else {
@@ -224,9 +221,7 @@ static inline int ddt_put_get(ompi_osc_ucx_module_t *module,
                                      origin_ucx_iov[origin_ucx_iov_idx].len,
                                      remote_addr + target_lb + prev_len, rkey);
                 if (status != UCS_OK && status != UCS_INPROGRESS) {
-                    opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                        "%s:%d: ucp_get_nbi failed: %d\n",
-                                        __FILE__, __LINE__, status);
+                    OSC_UCX_VERBOSE(1, "ucp_get_nbi failed: %d", status);
                     return OMPI_ERROR;
                 }
             }
@@ -247,9 +242,7 @@ static inline int ddt_put_get(ompi_osc_ucx_module_t *module,
                                      target_ucx_iov[target_ucx_iov_idx].len,
                                      remote_addr + (uint64_t)(target_ucx_iov[target_ucx_iov_idx].addr), rkey);
                 if (status != UCS_OK && status != UCS_INPROGRESS) {
-                    opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                        "%s:%d: ucp_put_nbi failed: %d\n",
-                                        __FILE__, __LINE__, status);
+                    OSC_UCX_VERBOSE(1, "ucp_put_nbi failed: %d", status);
                     return OMPI_ERROR;
                 }
             } else {
@@ -257,9 +250,7 @@ static inline int ddt_put_get(ompi_osc_ucx_module_t *module,
                                      target_ucx_iov[target_ucx_iov_idx].len,
                                      remote_addr + (uint64_t)(target_ucx_iov[target_ucx_iov_idx].addr), rkey);
                 if (status != UCS_OK && status != UCS_INPROGRESS) {
-                    opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                        "%s:%d: ucp_get_nbi failed: %d\n",
-                                        __FILE__, __LINE__, status);
+                    OSC_UCX_VERBOSE(1, "ucp_get_nbi failed: %d", status);
                     return OMPI_ERROR;
                 }
             }
@@ -290,34 +281,35 @@ static inline int start_atomicity(ompi_osc_ucx_module_t *module, ucp_ep_h ep, in
     uint64_t remote_addr = (module->state_info_array)[target].addr + OSC_UCX_STATE_ACC_LOCK_OFFSET;
     ucs_status_t status;
 
-    while (result_value != TARGET_LOCK_UNLOCKED) {
-        status = ucp_atomic_cswap64(ep, TARGET_LOCK_UNLOCKED,
-                                    TARGET_LOCK_EXCLUSIVE,
-                                    remote_addr, rkey, &result_value);
+    for (;;) {
+        status = opal_common_ucx_atomic_cswap(ep, TARGET_LOCK_UNLOCKED, TARGET_LOCK_EXCLUSIVE,
+                                              &result_value, sizeof(result_value),
+                                              remote_addr, rkey,
+                                              mca_osc_ucx_component.ucp_worker);
         if (status != UCS_OK) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_atomic_cswap64 failed: %d\n",
-                                __FILE__, __LINE__, status);
+            OSC_UCX_VERBOSE(1, "ucp_atomic_cswap64 failed: %d", status);
             return OMPI_ERROR;
         }
+        if (result_value == TARGET_LOCK_UNLOCKED) {
+            return OMPI_SUCCESS;
+        }
+
+        ucp_worker_progress(mca_osc_ucx_component.ucp_worker);
     }
 
-    return OMPI_SUCCESS;
 }
 
 static inline int end_atomicity(ompi_osc_ucx_module_t *module, ucp_ep_h ep, int target) {
     uint64_t result_value = 0;
     ucp_rkey_h rkey = (module->state_info_array)[target].rkey;
     uint64_t remote_addr = (module->state_info_array)[target].addr + OSC_UCX_STATE_ACC_LOCK_OFFSET;
-    ucs_status_t status;
+    int ret;
 
-    status = ucp_atomic_swap64(ep, TARGET_LOCK_UNLOCKED,
-                               remote_addr, rkey, &result_value);
-    if (status != UCS_OK) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_atomic_swap64 failed: %d\n",
-                            __FILE__, __LINE__, status);
-        return OMPI_ERROR;
+    ret = opal_common_ucx_atomic_fetch(ep, UCP_ATOMIC_FETCH_OP_SWAP, TARGET_LOCK_UNLOCKED,
+                                       &result_value, sizeof(result_value),
+                                       remote_addr, rkey, mca_osc_ucx_component.ucp_worker);
+    if (OMPI_SUCCESS != ret) {
+        return ret;
     }
 
     assert(result_value == TARGET_LOCK_EXCLUSIVE);
@@ -332,28 +324,25 @@ static inline int get_dynamic_win_info(uint64_t remote_addr, ompi_osc_ucx_module
     size_t len = sizeof(uint64_t) + sizeof(ompi_osc_dynamic_win_info_t) * OMPI_OSC_UCX_ATTACH_MAX;
     char *temp_buf = malloc(len);
     ompi_osc_dynamic_win_info_t *temp_dynamic_wins;
-    int win_count, contain, insert = -1;
+    uint64_t win_count;
+    int contain, insert = -1;
     ucs_status_t status;
+    int ret;
 
     if ((module->win_info_array[target]).rkey_init == true) {
         ucp_rkey_destroy((module->win_info_array[target]).rkey);
-        (module->win_info_array[target]).rkey_init == false;
+        (module->win_info_array[target]).rkey_init = false;
     }
 
     status = ucp_get_nbi(ep, (void *)temp_buf, len, remote_state_addr, state_rkey);
     if (status != UCS_OK && status != UCS_INPROGRESS) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_get_nbi failed: %d\n",
-                            __FILE__, __LINE__, status);
+        OSC_UCX_VERBOSE(1, "ucp_get_nbi failed: %d", status);
         return OMPI_ERROR;
     }
 
-    status = ucp_ep_flush(ep);
-    if (status != UCS_OK) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_ep_flush failed: %d\n",
-                            __FILE__, __LINE__, status);
-        return OMPI_ERROR;
+    ret = opal_common_ucx_ep_flush(ep, mca_osc_ucx_component.ucp_worker);
+    if (ret != OMPI_SUCCESS) {
+        return ret;
     }
 
     memcpy(&win_count, temp_buf, sizeof(uint64_t));
@@ -362,14 +351,12 @@ static inline int get_dynamic_win_info(uint64_t remote_addr, ompi_osc_ucx_module
     temp_dynamic_wins = (ompi_osc_dynamic_win_info_t *)(temp_buf + sizeof(uint64_t));
     contain = ompi_osc_find_attached_region_position(temp_dynamic_wins, 0, win_count,
                                                      remote_addr, 1, &insert);
-    assert(contain >= 0 && contain < win_count);
+    assert(contain >= 0 && (uint64_t)contain < win_count);
 
     status = ucp_ep_rkey_unpack(ep, temp_dynamic_wins[contain].rkey_buffer,
                                 &((module->win_info_array[target]).rkey));
     if (status != UCS_OK) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_ep_rkey_unpack failed: %d\n",
-                            __FILE__, __LINE__, status);
+        OSC_UCX_VERBOSE(1, "ucp_ep_rkey_unpack failed: %d", status);
         return OMPI_ERROR;
     }
 
@@ -404,6 +391,12 @@ int ompi_osc_ucx_put(const void *origin_addr, int origin_count, struct ompi_data
         }
     }
 
+    CHECK_VALID_RKEY(module, target, target_count);
+
+    if (!target_count) {
+        return OMPI_SUCCESS;
+    }
+
     rkey = (module->win_info_array[target]).rkey;
 
     ompi_datatype_get_true_extent(origin_dt, &origin_lb, &origin_extent);
@@ -422,9 +415,7 @@ int ompi_osc_ucx_put(const void *origin_addr, int origin_count, struct ompi_data
         status = ucp_put_nbi(ep, (void *)((intptr_t)origin_addr + origin_lb), origin_len,
                              remote_addr + target_lb, rkey);
         if (status != UCS_OK && status != UCS_INPROGRESS) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_put_nbi failed: %d\n",
-                                __FILE__, __LINE__, status);
+            OSC_UCX_VERBOSE(1, "ucp_put_nbi failed: %d", status);
             return OMPI_ERROR;
         }
         return incr_and_check_ops_num(module, target, ep);
@@ -460,6 +451,12 @@ int ompi_osc_ucx_get(void *origin_addr, int origin_count,
         }
     }
 
+    CHECK_VALID_RKEY(module, target, target_count);
+
+    if (!target_count) {
+        return OMPI_SUCCESS;
+    }
+
     rkey = (module->win_info_array[target]).rkey;
 
     ompi_datatype_get_true_extent(origin_dt, &origin_lb, &origin_extent);
@@ -478,9 +475,7 @@ int ompi_osc_ucx_get(void *origin_addr, int origin_count,
         status = ucp_get_nbi(ep, (void *)((intptr_t)origin_addr + origin_lb), origin_len,
                              remote_addr + target_lb, rkey);
         if (status != UCS_OK && status != UCS_INPROGRESS) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_get_nbi failed: %d\n",
-                                __FILE__, __LINE__, status);
+            OSC_UCX_VERBOSE(1, "ucp_get_nbi failed: %d", status);
             return OMPI_ERROR;
         }
 
@@ -522,11 +517,11 @@ int ompi_osc_ucx_accumulate(const void *origin_addr, int origin_count,
             return ret;
         }
     } else {
+        void *temp_addr_holder = NULL;
         void *temp_addr = NULL;
         uint32_t temp_count;
         ompi_datatype_t *temp_dt;
         ptrdiff_t temp_lb, temp_extent;
-        ucs_status_t status;
         bool is_origin_contig = ompi_datatype_is_contiguous_memory_layout(origin_dt, origin_count);
 
         if (ompi_datatype_is_predefined(target_dt)) {
@@ -539,7 +534,7 @@ int ompi_osc_ucx_accumulate(const void *origin_addr, int origin_count,
             }
         }
         ompi_datatype_get_true_extent(temp_dt, &temp_lb, &temp_extent);
-        temp_addr = malloc(temp_extent * temp_count);
+        temp_addr = temp_addr_holder = malloc(temp_extent * temp_count);
         if (temp_addr == NULL) {
             return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
         }
@@ -550,12 +545,9 @@ int ompi_osc_ucx_accumulate(const void *origin_addr, int origin_count,
             return ret;
         }
 
-        status = ucp_ep_flush(ep);
-        if (status != UCS_OK) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_ep_flush failed: %d\n",
-                                __FILE__, __LINE__, status);
-            return OMPI_ERROR;
+        ret = opal_common_ucx_ep_flush(ep, mca_osc_ucx_component.ucp_worker);
+        if (ret != OMPI_SUCCESS) {
+            return ret;
         }
 
         if (ompi_datatype_is_predefined(origin_dt) || is_origin_contig) {
@@ -574,12 +566,13 @@ int ompi_osc_ucx_accumulate(const void *origin_addr, int origin_count,
             if ((op != &ompi_mpi_op_maxloc.op && op != &ompi_mpi_op_minloc.op) ||
                 ompi_datatype_is_contiguous_memory_layout(temp_dt, temp_count)) {
                 size_t temp_size;
+                char *curr_temp_addr = (char *)temp_addr;
                 ompi_datatype_type_size(temp_dt, &temp_size);
                 while (origin_ucx_iov_idx < origin_ucx_iov_count) {
                     int curr_count = origin_ucx_iov[origin_ucx_iov_idx].len / temp_size;
                     ompi_op_reduce(op, origin_ucx_iov[origin_ucx_iov_idx].addr,
-                                   temp_addr, curr_count, temp_dt);
-                    temp_addr = (void *)((char *)temp_addr + curr_count * temp_size);
+                                   curr_temp_addr, curr_count, temp_dt);
+                    curr_temp_addr += curr_count * temp_size;
                     origin_ucx_iov_idx++;
                 }
             } else {
@@ -607,15 +600,12 @@ int ompi_osc_ucx_accumulate(const void *origin_addr, int origin_count,
             return ret;
         }
 
-        status = ucp_ep_flush(ep);
-        if (status != UCS_OK) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_ep_flush failed: %d\n",
-                                __FILE__, __LINE__, status);
-            return OMPI_ERROR;
+        ret = opal_common_ucx_ep_flush(ep, mca_osc_ucx_component.ucp_worker);
+        if (ret != OMPI_SUCCESS) {
+            return ret;
         }
 
-        free(temp_addr);
+        free(temp_addr_holder);
     }
 
     ret = end_atomicity(module, ep, target);
@@ -688,7 +678,7 @@ int ompi_osc_ucx_fetch_and_op(const void *origin_addr, void *result_addr,
         ucp_ep_h ep = OSC_UCX_GET_EP(module->comm, target);
         uint64_t remote_addr = (module->win_info_array[target]).addr + target_disp * OSC_UCX_GET_DISP(module, target);
         ucp_rkey_h rkey;
-        uint64_t value = *(uint64_t *)origin_addr;
+        uint64_t value = origin_addr ? *(uint64_t *)origin_addr : 0;
         ucp_atomic_fetch_op_t opcode;
         size_t dt_bytes;
         ompi_osc_ucx_internal_request_t *req = NULL;
@@ -773,11 +763,11 @@ int ompi_osc_ucx_get_accumulate(const void *origin_addr, int origin_count,
                 return ret;
             }
         } else {
+            void *temp_addr_holder = NULL;
             void *temp_addr = NULL;
             uint32_t temp_count;
             ompi_datatype_t *temp_dt;
             ptrdiff_t temp_lb, temp_extent;
-            ucs_status_t status;
             bool is_origin_contig = ompi_datatype_is_contiguous_memory_layout(origin_dt, origin_count);
 
             if (ompi_datatype_is_predefined(target_dt)) {
@@ -790,7 +780,7 @@ int ompi_osc_ucx_get_accumulate(const void *origin_addr, int origin_count,
                 }
             }
             ompi_datatype_get_true_extent(temp_dt, &temp_lb, &temp_extent);
-            temp_addr = malloc(temp_extent * temp_count);
+            temp_addr = temp_addr_holder = malloc(temp_extent * temp_count);
             if (temp_addr == NULL) {
                 return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
             }
@@ -801,12 +791,9 @@ int ompi_osc_ucx_get_accumulate(const void *origin_addr, int origin_count,
                 return ret;
             }
 
-            status = ucp_ep_flush(ep);
-            if (status != UCS_OK) {
-                opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                    "%s:%d: ucp_ep_flush failed: %d\n",
-                                    __FILE__, __LINE__, status);
-                return OMPI_ERROR;
+            ret = opal_common_ucx_ep_flush(ep, mca_osc_ucx_component.ucp_worker);
+            if (ret != OMPI_SUCCESS) {
+                return ret;
             }
 
             if (ompi_datatype_is_predefined(origin_dt) || is_origin_contig) {
@@ -825,12 +812,13 @@ int ompi_osc_ucx_get_accumulate(const void *origin_addr, int origin_count,
                 if ((op != &ompi_mpi_op_maxloc.op && op != &ompi_mpi_op_minloc.op) ||
                     ompi_datatype_is_contiguous_memory_layout(temp_dt, temp_count)) {
                     size_t temp_size;
+                    char *curr_temp_addr = (char *)temp_addr;
                     ompi_datatype_type_size(temp_dt, &temp_size);
                     while (origin_ucx_iov_idx < origin_ucx_iov_count) {
                         int curr_count = origin_ucx_iov[origin_ucx_iov_idx].len / temp_size;
                         ompi_op_reduce(op, origin_ucx_iov[origin_ucx_iov_idx].addr,
-                                       temp_addr, curr_count, temp_dt);
-                        temp_addr = (void *)((char *)temp_addr + curr_count * temp_size);
+                                       curr_temp_addr, curr_count, temp_dt);
+                        curr_temp_addr += curr_count * temp_size;
                         origin_ucx_iov_idx++;
                     }
                 } else {
@@ -857,15 +845,12 @@ int ompi_osc_ucx_get_accumulate(const void *origin_addr, int origin_count,
                 return ret;
             }
 
-            status = ucp_ep_flush(ep);
-            if (status != UCS_OK) {
-                opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                    "%s:%d: ucp_ep_flush failed: %d\n",
-                                    __FILE__, __LINE__, status);
-                return OMPI_ERROR;
+            ret = opal_common_ucx_ep_flush(ep, mca_osc_ucx_component.ucp_worker);
+            if (ret != OMPI_SUCCESS) {
+                return ret;
             }
 
-            free(temp_addr);
+            free(temp_addr_holder);
         }
     }
 
@@ -900,12 +885,12 @@ int ompi_osc_ucx_rput(const void *origin_addr, int origin_count,
         }
     }
 
-    rkey = (module->win_info_array[target]).rkey;
+    CHECK_VALID_RKEY(module, target, target_count);
+
+    rkey = (module->state_info_array[target]).rkey;
 
     OMPI_OSC_UCX_REQUEST_ALLOC(win, ucx_req);
-    if (NULL == ucx_req) {
-        return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-    }
+    assert(NULL != ucx_req);
 
     ret = ompi_osc_ucx_put(origin_addr, origin_count, origin_dt, target, target_disp,
                            target_count, target_dt, win);
@@ -915,9 +900,7 @@ int ompi_osc_ucx_rput(const void *origin_addr, int origin_count,
 
     status = ucp_worker_fence(mca_osc_ucx_component.ucp_worker);
     if (status != UCS_OK) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_worker_fence failed: %d\n",
-                            __FILE__, __LINE__, status);
+        OSC_UCX_VERBOSE(1, "ucp_worker_fence failed: %d", status);
         return OMPI_ERROR;
     }
 
@@ -963,12 +946,12 @@ int ompi_osc_ucx_rget(void *origin_addr, int origin_count,
         }
     }
 
-    rkey = (module->win_info_array[target]).rkey;
+    CHECK_VALID_RKEY(module, target, target_count);
+
+    rkey = (module->state_info_array[target]).rkey;
 
     OMPI_OSC_UCX_REQUEST_ALLOC(win, ucx_req);
-    if (NULL == ucx_req) {
-        return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-    }
+    assert(NULL != ucx_req);
 
     ret = ompi_osc_ucx_get(origin_addr, origin_count, origin_dt, target, target_disp,
                            target_count, target_dt, win);
@@ -978,9 +961,7 @@ int ompi_osc_ucx_rget(void *origin_addr, int origin_count,
 
     status = ucp_worker_fence(mca_osc_ucx_component.ucp_worker);
     if (status != UCS_OK) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_worker_fence failed: %d\n",
-                            __FILE__, __LINE__, status);
+        OSC_UCX_VERBOSE(1, "ucp_worker_fence failed: %d", status);
         return OMPI_ERROR;
     }
 
@@ -1015,9 +996,7 @@ int ompi_osc_ucx_raccumulate(const void *origin_addr, int origin_count,
     }
 
     OMPI_OSC_UCX_REQUEST_ALLOC(win, ucx_req);
-    if (NULL == ucx_req) {
-        return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-    }
+    assert(NULL != ucx_req);
 
     ret = ompi_osc_ucx_accumulate(origin_addr, origin_count, origin_dt, target, target_disp,
                                   target_count, target_dt, op, win);
@@ -1049,9 +1028,7 @@ int ompi_osc_ucx_rget_accumulate(const void *origin_addr, int origin_count,
     }
 
     OMPI_OSC_UCX_REQUEST_ALLOC(win, ucx_req);
-    if (NULL == ucx_req) {
-        return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-    }
+    assert(NULL != ucx_req);
 
     ret = ompi_osc_ucx_get_accumulate(origin_addr, origin_count, origin_datatype,
                                       result_addr, result_count, result_datatype,
