@@ -12,10 +12,29 @@
 #include "ompi/mca/osc/osc.h"
 #include "ompi/mca/osc/base/base.h"
 #include "ompi/mca/osc/base/osc_base_obj_convert.h"
+#include "opal/mca/common/ucx/common_ucx.h"
 
 #include "osc_ucx.h"
 #include "osc_ucx_request.h"
 
+#define memcpy_off(_dst, _src, _len, _off)        \
+    memcpy(((char*)(_dst)) + (_off), _src, _len); \
+    (_off) += (_len);
+
+opal_mutex_t mca_osc_service_mutex = OPAL_MUTEX_STATIC_INIT;
+static void _osc_ucx_init_lock(void)
+{
+    if(mca_osc_ucx_component.enable_mpi_threads) {
+        opal_mutex_lock(&mca_osc_service_mutex);
+    }
+}
+static void _osc_ucx_init_unlock(void)
+{
+    if(mca_osc_ucx_component.enable_mpi_threads) {
+        opal_mutex_unlock(&mca_osc_service_mutex);
+    }
+}
+
 static int component_open(void);
 static int component_register(void);
 static int component_init(bool enable_progress_threads, bool enable_mpi_threads);
@@ -25,6 +44,7 @@ static int component_query(struct ompi_win_t *win, void **base, size_t size, int
 static int component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit,
                             struct ompi_communicator_t *comm, struct opal_info_t *info,
                             int flavor, int *model);
+static void ompi_osc_ucx_unregister_progress(void);
 
 ompi_osc_ucx_component_t mca_osc_ucx_component = {
     { /* ompi_osc_base_component_t */
@@ -44,7 +64,12 @@ ompi_osc_ucx_component_t mca_osc_ucx_component = {
         .osc_query = component_query,
         .osc_select = component_select,
         .osc_finalize = component_finalize,
-    }
+    },
+    .ucp_context            = NULL,
+    .ucp_worker             = NULL,
+    .env_initialized        = false,
+    .num_incomplete_req_ops = 0,
+    .num_modules            = 0
 };
 
 ompi_osc_ucx_module_t ompi_osc_ucx_module_template = {
@@ -91,8 +116,15 @@ static int component_open(void) {
 }
 
 static int component_register(void) {
+    unsigned major          = 0;
+    unsigned minor          = 0;
+    unsigned release_number = 0;
     char *description_str;
-    mca_osc_ucx_component.priority = 0;
+
+    ucp_get_version(&major, &minor, &release_number);
+
+    mca_osc_ucx_component.priority = UCX_VERSION(major, minor, release_number) >= UCX_VERSION(1, 5, 0) ? 60 : 0;
+
     asprintf(&description_str, "Priority of the osc/ucx component (default: %d)",
              mca_osc_ucx_component.priority);
     (void) mca_base_component_var_register(&mca_osc_ucx_component.super.osc_version, "priority", description_str,
@@ -100,91 +132,21 @@ static int component_register(void) {
                                            MCA_BASE_VAR_SCOPE_GROUP, &mca_osc_ucx_component.priority);
     free(description_str);
 
+    opal_common_ucx_mca_var_register(&mca_osc_ucx_component.super.osc_version);
+
     return OMPI_SUCCESS;
 }
 
 static int progress_callback(void) {
-    if (mca_osc_ucx_component.ucp_worker != NULL &&
-        mca_osc_ucx_component.num_incomplete_req_ops > 0) {
-        ucp_worker_progress(mca_osc_ucx_component.ucp_worker);
-    }
+    ucp_worker_progress(mca_osc_ucx_component.ucp_worker);
     return 0;
 }
 
 static int component_init(bool enable_progress_threads, bool enable_mpi_threads) {
-    ucp_config_t *config = NULL;
-    ucp_params_t context_params;
-    bool progress_registered = false, requests_created = false;
-    int ret = OMPI_SUCCESS;
-    ucs_status_t status;
-
-    mca_osc_ucx_component.ucp_context = NULL;
-    mca_osc_ucx_component.ucp_worker = NULL;
     mca_osc_ucx_component.enable_mpi_threads = enable_mpi_threads;
 
-    status = ucp_config_read("MPI", NULL, &config);
-    if (UCS_OK != status) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_config_read failed: %d\n",
-                            __FILE__, __LINE__, status);
-        return OMPI_ERROR;
-    }
-
-    OBJ_CONSTRUCT(&mca_osc_ucx_component.requests, opal_free_list_t);
-    requests_created = true;
-    ret = opal_free_list_init (&mca_osc_ucx_component.requests,
-                               sizeof(ompi_osc_ucx_request_t),
-                               opal_cache_line_size,
-                               OBJ_CLASS(ompi_osc_ucx_request_t),
-                               0, 0, 8, 0, 8, NULL, 0, NULL, NULL, NULL);
-    if (OMPI_SUCCESS != ret) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: opal_free_list_init failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-
-    mca_osc_ucx_component.num_incomplete_req_ops = 0;
-
-    ret = opal_progress_register(progress_callback);
-    progress_registered = true;
-    if (OMPI_SUCCESS != ret) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: opal_progress_register failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-
-    /* initialize UCP context */
-
-    memset(&context_params, 0, sizeof(ucp_context_h));
-    context_params.field_mask = UCP_PARAM_FIELD_FEATURES |
-                                UCP_PARAM_FIELD_MT_WORKERS_SHARED |
-                                UCP_PARAM_FIELD_ESTIMATED_NUM_EPS |
-                                UCP_PARAM_FIELD_REQUEST_INIT |
-                                UCP_PARAM_FIELD_REQUEST_SIZE;
-    context_params.features = UCP_FEATURE_RMA | UCP_FEATURE_AMO32 | UCP_FEATURE_AMO64;
-    context_params.mt_workers_shared = 0;
-    context_params.estimated_num_eps = ompi_proc_world_size();
-    context_params.request_init = internal_req_init;
-    context_params.request_size = sizeof(ompi_osc_ucx_internal_request_t);
-
-    status = ucp_init(&context_params, config, &mca_osc_ucx_component.ucp_context);
-    ucp_config_release(config);
-    if (UCS_OK != status) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_init failed: %d\n",
-                            __FILE__, __LINE__, status);
-        ret = OMPI_ERROR;
-        goto error;
-    }
-
-    return ret;
- error:
-    if (progress_registered) opal_progress_unregister(progress_callback);
-    if (requests_created) OBJ_DESTRUCT(&mca_osc_ucx_component.requests);
-    if (mca_osc_ucx_component.ucp_context) ucp_cleanup(mca_osc_ucx_component.ucp_context);
-    return ret;
+    opal_common_ucx_mca_register();
+    return OMPI_SUCCESS;
 }
 
 static int component_finalize(void) {
@@ -201,9 +163,12 @@ static int component_finalize(void) {
     }
 
     assert(mca_osc_ucx_component.num_incomplete_req_ops == 0);
-    OBJ_DESTRUCT(&mca_osc_ucx_component.requests);
-    opal_progress_unregister(progress_callback);
-    ucp_cleanup(mca_osc_ucx_component.ucp_context);
+    if (mca_osc_ucx_component.env_initialized == true) {
+        OBJ_DESTRUCT(&mca_osc_ucx_component.requests);
+        ucp_cleanup(mca_osc_ucx_component.ucp_context);
+        mca_osc_ucx_component.env_initialized = false;
+    }
+    opal_common_ucx_mca_deregister();
     return OMPI_SUCCESS;
 }
 
@@ -273,9 +238,7 @@ static inline int mem_map(void **base, size_t size, ucp_mem_h *memh_ptr,
 
     status = ucp_mem_map(mca_osc_ucx_component.ucp_context, &mem_params, memh_ptr);
     if (status != UCS_OK) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_mem_map failed: %d\n",
-                            __FILE__, __LINE__, status);
+        OSC_UCX_VERBOSE(1, "ucp_mem_map failed: %d", status);
         ret = OMPI_ERROR;
         goto error;
     }
@@ -283,9 +246,7 @@ static inline int mem_map(void **base, size_t size, ucp_mem_h *memh_ptr,
     mem_attrs.field_mask = UCP_MEM_ATTR_FIELD_ADDRESS | UCP_MEM_ATTR_FIELD_LENGTH;
     status = ucp_mem_query((*memh_ptr), &mem_attrs);
     if (status != UCS_OK) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_mem_query failed: %d\n",
-                            __FILE__, __LINE__, status);
+        OSC_UCX_VERBOSE(1, "ucp_mem_query failed: %d", status);
         ret = OMPI_ERROR;
         goto error;
     }
@@ -303,6 +264,25 @@ static inline int mem_map(void **base, size_t size, ucp_mem_h *memh_ptr,
     return ret;
 }
 
+static void ompi_osc_ucx_unregister_progress()
+{
+    int ret;
+
+    /* May be called concurrently - protect */
+    _osc_ucx_init_lock();
+
+    mca_osc_ucx_component.num_modules--;
+    OSC_UCX_ASSERT(mca_osc_ucx_component.num_modules >= 0);
+    if (0 == mca_osc_ucx_component.num_modules) {
+        ret = opal_progress_unregister(progress_callback);
+        if (OMPI_SUCCESS != ret) {
+            OSC_UCX_VERBOSE(1, "opal_progress_unregister failed: %d", ret);
+        }
+    }
+
+    _osc_ucx_init_unlock();
+}
+
 static int component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit,
                             struct ompi_communicator_t *comm, struct opal_info_t *info,
                             int flavor, int *model) {
@@ -313,7 +293,7 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
     ucs_status_t status;
     int i, comm_size = ompi_comm_size(comm);
     int is_eps_ready;
-    bool eps_created = false, worker_created = false;
+    bool eps_created = false, env_initialized = false;
     ucp_address_t *my_addr = NULL;
     size_t my_addr_len;
     char *recv_buf = NULL;
@@ -325,6 +305,8 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
     int disps[comm_size];
     int rkey_sizes[comm_size];
     uint64_t zero = 0;
+    size_t info_offset;
+    uint64_t size_u64;
 
     /* the osc/sm component is the exclusive provider for support for
      * shared memory windows */
@@ -332,55 +314,119 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
         return OMPI_ERR_NOT_SUPPORTED;
     }
 
-    /* if UCP worker has never been initialized before, init it first */
-    if (mca_osc_ucx_component.ucp_worker == NULL) {
+    _osc_ucx_init_lock();
+
+    if (mca_osc_ucx_component.env_initialized == false) {
+        ucp_config_t *config = NULL;
+        ucp_params_t context_params;
         ucp_worker_params_t worker_params;
         ucp_worker_attr_t worker_attr;
 
-        memset(&worker_params, 0, sizeof(ucp_worker_h));
+        status = ucp_config_read("MPI", NULL, &config);
+        if (UCS_OK != status) {
+            OSC_UCX_VERBOSE(1, "ucp_config_read failed: %d", status);
+            ret = OMPI_ERROR;
+            goto select_unlock;
+        }
+
+        OBJ_CONSTRUCT(&mca_osc_ucx_component.requests, opal_free_list_t);
+        ret = opal_free_list_init (&mca_osc_ucx_component.requests,
+                                   sizeof(ompi_osc_ucx_request_t),
+                                   opal_cache_line_size,
+                                   OBJ_CLASS(ompi_osc_ucx_request_t),
+                                   0, 0, 8, 0, 8, NULL, 0, NULL, NULL, NULL);
+        if (OMPI_SUCCESS != ret) {
+            OSC_UCX_VERBOSE(1, "opal_free_list_init failed: %d", ret);
+            goto select_unlock;
+        }
+
+        /* initialize UCP context */
+
+        memset(&context_params, 0, sizeof(context_params));
+        context_params.field_mask = UCP_PARAM_FIELD_FEATURES |
+                                    UCP_PARAM_FIELD_MT_WORKERS_SHARED |
+                                    UCP_PARAM_FIELD_ESTIMATED_NUM_EPS |
+                                    UCP_PARAM_FIELD_REQUEST_INIT |
+                                    UCP_PARAM_FIELD_REQUEST_SIZE;
+        context_params.features = UCP_FEATURE_RMA | UCP_FEATURE_AMO32 | UCP_FEATURE_AMO64;
+        context_params.mt_workers_shared = 0;
+        context_params.estimated_num_eps = ompi_proc_world_size();
+        context_params.request_init = internal_req_init;
+        context_params.request_size = sizeof(ompi_osc_ucx_internal_request_t);
+
+        status = ucp_init(&context_params, config, &mca_osc_ucx_component.ucp_context);
+        ucp_config_release(config);
+        if (UCS_OK != status) {
+            OSC_UCX_VERBOSE(1, "ucp_init failed: %d", status);
+            ret = OMPI_ERROR;
+            goto select_unlock;
+        }
+
+        assert(mca_osc_ucx_component.ucp_worker == NULL);
+        memset(&worker_params, 0, sizeof(worker_params));
         worker_params.field_mask = UCP_WORKER_PARAM_FIELD_THREAD_MODE;
         worker_params.thread_mode = (mca_osc_ucx_component.enable_mpi_threads == true)
                                     ? UCS_THREAD_MODE_MULTI : UCS_THREAD_MODE_SINGLE;
         status = ucp_worker_create(mca_osc_ucx_component.ucp_context, &worker_params,
                                    &(mca_osc_ucx_component.ucp_worker));
         if (UCS_OK != status) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_worker_create failed: %d\n",
-                                __FILE__, __LINE__, status);
-            ret = OMPI_ERROR;
-            goto error;
+            OSC_UCX_VERBOSE(1, "ucp_worker_create failed: %d", status);
+            ret = OMPI_ERR_TEMP_OUT_OF_RESOURCE;
+            goto select_unlock;
         }
 
         /* query UCP worker attributes */
         worker_attr.field_mask = UCP_WORKER_ATTR_FIELD_THREAD_MODE;
         status = ucp_worker_query(mca_osc_ucx_component.ucp_worker, &worker_attr);
         if (UCS_OK != status) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_worker_query failed: %d\n",
-                                __FILE__, __LINE__, status);
-            ret = OMPI_ERROR;
-            goto error;
+            OSC_UCX_VERBOSE(1, "ucp_worker_query failed: %d", status);
+            ret = OMPI_ERR_TEMP_OUT_OF_RESOURCE;
+            goto select_unlock;
         }
 
         if (mca_osc_ucx_component.enable_mpi_threads == true &&
             worker_attr.thread_mode != UCS_THREAD_MODE_MULTI) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucx does not support multithreading\n",
-                                __FILE__, __LINE__);
-            ret = OMPI_ERROR;
-            goto error;
+            OSC_UCX_VERBOSE(1, "ucx does not support multithreading");
+            ret = OMPI_ERR_TEMP_OUT_OF_RESOURCE;
+            goto select_unlock;
         }
 
-        worker_created = true;
+        mca_osc_ucx_component.env_initialized = true;
+        env_initialized = true;
+    }
+    
+    mca_osc_ucx_component.num_modules++;
+
+    OSC_UCX_ASSERT(mca_osc_ucx_component.num_modules > 0);
+    if (1 == mca_osc_ucx_component.num_modules) {
+        ret = opal_progress_register(progress_callback);
+        if (OMPI_SUCCESS != ret) {
+            OSC_UCX_VERBOSE(1, "opal_progress_register failed: %d", ret);
+            goto select_unlock;
+        }
+    }
+
+select_unlock:
+    _osc_ucx_init_unlock();
+    switch(ret) {
+    case OMPI_SUCCESS:
+        break;
+    case OMPI_ERROR:
+        goto error;
+    case OMPI_ERR_TEMP_OUT_OF_RESOURCE:
+        goto error_nomem;
+    default:
+        goto error;
     }
 
     /* create module structure */
     module = (ompi_osc_ucx_module_t *)calloc(1, sizeof(ompi_osc_ucx_module_t));
     if (module == NULL) {
         ret = OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-        goto error;
+        goto error_nomem;
     }
 
+
     /* fill in the function pointer part */
     memcpy(module, &ompi_osc_ucx_module_template, sizeof(ompi_osc_base_module_t));
 
@@ -449,9 +495,7 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
         status = ucp_worker_get_address(mca_osc_ucx_component.ucp_worker,
                                         &my_addr, &my_addr_len);
         if (status != UCS_OK) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_worker_get_address failed: %d\n",
-                                __FILE__, __LINE__, status);
+            OSC_UCX_VERBOSE(1, "ucp_worker_get_address failed: %d", status);
             ret = OMPI_ERROR;
             goto error;
         }
@@ -471,9 +515,7 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
                 ep_params.address = (ucp_address_t *)&(recv_buf[disps[i]]);
                 status = ucp_ep_create(mca_osc_ucx_component.ucp_worker, &ep_params, &ep);
                 if (status != UCS_OK) {
-                    opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                        "%s:%d: ucp_ep_create failed: %d\n",
-                                        __FILE__, __LINE__, status);
+                    OSC_UCX_VERBOSE(1, "ucp_ep_create failed: %d", status);
                     ret = OMPI_ERROR;
                     goto error;
                 }
@@ -518,9 +560,7 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
         status = ucp_rkey_pack(mca_osc_ucx_component.ucp_context, module->memh,
                                &rkey_buffer, &rkey_buffer_size);
         if (status != UCS_OK) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_rkey_pack failed: %d\n",
-                                __FILE__, __LINE__, status);
+            OSC_UCX_VERBOSE(1, "ucp_rkey_pack failed: %d", status);
             ret = OMPI_ERROR;
             goto error;
         }
@@ -531,29 +571,32 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
     status = ucp_rkey_pack(mca_osc_ucx_component.ucp_context, module->state_memh,
                            &state_rkey_buffer, &state_rkey_buffer_size);
     if (status != UCS_OK) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_rkey_pack failed: %d\n",
-                            __FILE__, __LINE__, status);
+        OSC_UCX_VERBOSE(1, "ucp_rkey_pack failed: %d", status);
         ret = OMPI_ERROR;
         goto error;
     }
 
-    my_info_len = 2 * sizeof(uint64_t) + rkey_buffer_size + state_rkey_buffer_size;
+    size_u64 = (uint64_t)size;
+    my_info_len = 3 * sizeof(uint64_t) + rkey_buffer_size + state_rkey_buffer_size;
     my_info = malloc(my_info_len);
     if (my_info == NULL) {
         ret = OMPI_ERR_TEMP_OUT_OF_RESOURCE;
         goto error;
     }
 
+    info_offset = 0;
+
     if (flavor == MPI_WIN_FLAVOR_ALLOCATE || flavor == MPI_WIN_FLAVOR_CREATE) {
-        memcpy(my_info, base, sizeof(uint64_t));
+        memcpy_off(my_info, base, sizeof(uint64_t), info_offset);
     } else {
-        memcpy(my_info, &zero, sizeof(uint64_t));
+        memcpy_off(my_info, &zero, sizeof(uint64_t), info_offset);
     }
-    memcpy((void *)((char *)my_info + sizeof(uint64_t)), &state_base, sizeof(uint64_t));
-    memcpy((void *)((char *)my_info + 2 * sizeof(uint64_t)), rkey_buffer, rkey_buffer_size);
-    memcpy((void *)((char *)my_info + 2 * sizeof(uint64_t) + rkey_buffer_size),
-           state_rkey_buffer, state_rkey_buffer_size);
+    memcpy_off(my_info, &state_base, sizeof(uint64_t), info_offset);
+    memcpy_off(my_info, &size_u64, sizeof(uint64_t), info_offset);
+    memcpy_off(my_info, rkey_buffer, rkey_buffer_size, info_offset);
+    memcpy_off(my_info, state_rkey_buffer, state_rkey_buffer_size, info_offset);
+
+    assert(my_info_len == info_offset);
 
     ret = allgather_len_and_info(my_info, (int)my_info_len, &recv_buf, disps, module->comm);
     if (ret != OMPI_SUCCESS) {
@@ -569,32 +612,35 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
 
     for (i = 0; i < comm_size; i++) {
         ucp_ep_h ep = OSC_UCX_GET_EP(module->comm, i);
+        uint64_t dest_size;
         assert(ep != NULL);
 
-        memcpy(&(module->win_info_array[i]).addr, &recv_buf[disps[i]], sizeof(uint64_t));
-        memcpy(&(module->state_info_array[i]).addr, &recv_buf[disps[i] + sizeof(uint64_t)], 
-               sizeof(uint64_t));
+        info_offset = disps[i];
+
+        memcpy(&(module->win_info_array[i]).addr, &recv_buf[info_offset], sizeof(uint64_t));
+        info_offset += sizeof(uint64_t);
+        memcpy(&(module->state_info_array[i]).addr, &recv_buf[info_offset], sizeof(uint64_t));
+        info_offset += sizeof(uint64_t);
+        memcpy(&dest_size, &recv_buf[info_offset], sizeof(uint64_t));
+        info_offset += sizeof(uint64_t);
 
         (module->win_info_array[i]).rkey_init = false;
-        if (size > 0 && (flavor == MPI_WIN_FLAVOR_ALLOCATE || flavor == MPI_WIN_FLAVOR_CREATE)) {
-            status = ucp_ep_rkey_unpack(ep, &(recv_buf[disps[i] + 2 * sizeof(uint64_t)]),
+        if (dest_size > 0 && (flavor == MPI_WIN_FLAVOR_ALLOCATE || flavor == MPI_WIN_FLAVOR_CREATE)) {
+            status = ucp_ep_rkey_unpack(ep, &recv_buf[info_offset],
                                         &((module->win_info_array[i]).rkey));
             if (status != UCS_OK) {
-                opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                    "%s:%d: ucp_ep_rkey_unpack failed: %d\n",
-                                    __FILE__, __LINE__, status);
+                OSC_UCX_VERBOSE(1, "ucp_ep_rkey_unpack failed: %d", status);
                 ret = OMPI_ERROR;
                 goto error;
             }
+            info_offset += rkey_sizes[i];
             (module->win_info_array[i]).rkey_init = true;
         }
 
-        status = ucp_ep_rkey_unpack(ep, &(recv_buf[disps[i] + 2 * sizeof(uint64_t) + rkey_sizes[i]]),
+        status = ucp_ep_rkey_unpack(ep, &recv_buf[info_offset],
                                     &((module->state_info_array[i]).rkey));
         if (status != UCS_OK) {
-            opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                "%s:%d: ucp_ep_rkey_unpack failed: %d\n",
-                                __FILE__, __LINE__, status);
+            OSC_UCX_VERBOSE(1, "ucp_ep_rkey_unpack failed: %d", status);
             ret = OMPI_ERROR;
             goto error;
         }
@@ -674,8 +720,18 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
             ucp_ep_destroy(ep);
         }
     }
-    if (worker_created) ucp_worker_destroy(mca_osc_ucx_component.ucp_worker);
-    if (module) free(module);
+    if (module) {
+        free(module);
+        ompi_osc_ucx_unregister_progress();
+    }
+
+error_nomem:
+    if (env_initialized == true) {
+        OBJ_DESTRUCT(&mca_osc_ucx_component.requests);
+        ucp_worker_destroy(mca_osc_ucx_component.ucp_worker);
+        ucp_cleanup(mca_osc_ucx_component.ucp_context);
+        mca_osc_ucx_component.env_initialized = false;
+    }
     return ret;
 }
 
@@ -721,7 +777,7 @@ int ompi_osc_ucx_win_attach(struct ompi_win_t *win, void *base, size_t len) {
             return ret;
         }
 
-        assert(insert_index >= 0 && insert_index < module->state.dynamic_win_count);
+        assert(insert_index >= 0 && (uint64_t)insert_index < module->state.dynamic_win_count);
 
         memmove((void *)&module->local_dynamic_win_info[insert_index+1],
                 (void *)&module->local_dynamic_win_info[insert_index],
@@ -746,9 +802,7 @@ int ompi_osc_ucx_win_attach(struct ompi_win_t *win, void *base, size_t len) {
                            module->local_dynamic_win_info[insert_index].memh,
                            &rkey_buffer, (size_t *)&rkey_buffer_size);
     if (status != UCS_OK) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ucp_rkey_pack failed: %d\n",
-                            __FILE__, __LINE__, status);
+        OSC_UCX_VERBOSE(1, "ucp_rkey_pack failed: %d", status);
         return OMPI_ERROR;
     }
 
@@ -773,7 +827,12 @@ int ompi_osc_ucx_win_detach(struct ompi_win_t *win, const void *base) {
     contain = ompi_osc_find_attached_region_position((ompi_osc_dynamic_win_info_t *)module->state.dynamic_wins,
                                                      0, (int)module->state.dynamic_win_count,
                                                      (uint64_t)base, 1, &insert);
-    assert(contain >= 0 && contain < module->state.dynamic_win_count);
+    assert(contain >= 0 && (uint64_t)contain < module->state.dynamic_win_count);
+
+    /* if we can't find region - just exit */
+    if (contain < 0) {
+        return OMPI_SUCCESS;
+    }
 
     module->local_dynamic_win_info[contain].refcnt--;
     if (module->local_dynamic_win_info[contain].refcnt == 0) {
@@ -794,16 +853,7 @@ int ompi_osc_ucx_win_detach(struct ompi_win_t *win, const void *base) {
 
 int ompi_osc_ucx_free(struct ompi_win_t *win) {
     ompi_osc_ucx_module_t *module = (ompi_osc_ucx_module_t*) win->w_osc_module;
-    int i, ret = OMPI_SUCCESS;
-
-    if ((module->epoch_type.access != NONE_EPOCH && module->epoch_type.access != FENCE_EPOCH)
-        || module->epoch_type.exposure != NONE_EPOCH) {
-        ret = OMPI_ERR_RMA_SYNC;
-    }
-
-    if (module->start_group != NULL || module->post_group != NULL) {
-        ret = OMPI_ERR_RMA_SYNC;
-    }
+    int i, ret;
 
     assert(module->global_ops_num == 0);
     assert(module->lock_count == 0);
@@ -816,13 +866,18 @@ int ompi_osc_ucx_free(struct ompi_win_t *win) {
         ucp_worker_progress(mca_osc_ucx_component.ucp_worker);
     }
 
+    ret = opal_common_ucx_worker_flush(mca_osc_ucx_component.ucp_worker);
+    if (OMPI_SUCCESS != ret) {
+        OSC_UCX_VERBOSE(1, "opal_common_ucx_worker_flush failed: %d", ret);
+    }
+
     ret = module->comm->c_coll->coll_barrier(module->comm,
                                              module->comm->c_coll->coll_barrier_module);
 
     for (i = 0; i < ompi_comm_size(module->comm); i++) {
         if ((module->win_info_array[i]).rkey_init == true) {
             ucp_rkey_destroy((module->win_info_array[i]).rkey);
-            (module->win_info_array[i]).rkey_init == false;
+            (module->win_info_array[i]).rkey_init = false;
         }
         ucp_rkey_destroy((module->state_info_array[i]).rkey);
     }
@@ -841,6 +896,7 @@ int ompi_osc_ucx_free(struct ompi_win_t *win) {
     ompi_comm_free(&module->comm);
 
     free(module);
+    ompi_osc_ucx_unregister_progress();
 
     return ret;
 }
