@@ -1,8 +1,10 @@
 /*
- * Copyright (C) Mellanox Technologies Ltd. 2001-2011.  ALL RIGHTS RESERVED.
+ * Copyright (C) 2001-2011 Mellanox Technologies Ltd. 2001-2011.  ALL RIGHTS RESERVED.
  * Copyright (c) 2016      The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
+ * Copyright (c) 2018      Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -14,8 +16,10 @@
 
 #include "opal/runtime/opal.h"
 #include "opal/mca/pmix/pmix.h"
+#include "ompi/attribute/attribute.h"
 #include "ompi/message/message.h"
 #include "ompi/mca/pml/base/pml_base_bsend.h"
+#include "opal/mca/common/ucx/common_ucx.h"
 #include "pml_ucx_request.h"
 
 #include <inttypes.h>
@@ -46,43 +50,78 @@
 #define MODEX_KEY "pml-ucx"
 
 mca_pml_ucx_module_t ompi_pml_ucx = {
-    {
-        mca_pml_ucx_add_procs,
-        mca_pml_ucx_del_procs,
-        mca_pml_ucx_enable,
-        NULL,
-        mca_pml_ucx_add_comm,
-        mca_pml_ucx_del_comm,
-        mca_pml_ucx_irecv_init,
-        mca_pml_ucx_irecv,
-        mca_pml_ucx_recv,
-        mca_pml_ucx_isend_init,
-        mca_pml_ucx_isend,
-        mca_pml_ucx_send,
-        mca_pml_ucx_iprobe,
-        mca_pml_ucx_probe,
-        mca_pml_ucx_start,
-        mca_pml_ucx_improbe,
-        mca_pml_ucx_mprobe,
-        mca_pml_ucx_imrecv,
-        mca_pml_ucx_mrecv,
-        mca_pml_ucx_dump,
-        NULL, /* FT */
-        1ul << (PML_UCX_CONTEXT_BITS),
-        1ul << (PML_UCX_TAG_BITS - 1),
+    .super = {
+        .pml_add_procs     = mca_pml_ucx_add_procs,
+        .pml_del_procs     = mca_pml_ucx_del_procs,
+        .pml_enable        = mca_pml_ucx_enable,
+        .pml_progress      = NULL,
+        .pml_add_comm      = mca_pml_ucx_add_comm,
+        .pml_del_comm      = mca_pml_ucx_del_comm,
+        .pml_irecv_init    = mca_pml_ucx_irecv_init,
+        .pml_irecv         = mca_pml_ucx_irecv,
+        .pml_recv          = mca_pml_ucx_recv,
+        .pml_isend_init    = mca_pml_ucx_isend_init,
+        .pml_isend         = mca_pml_ucx_isend,
+        .pml_send          = mca_pml_ucx_send,
+        .pml_iprobe        = mca_pml_ucx_iprobe,
+        .pml_probe         = mca_pml_ucx_probe,
+        .pml_start         = mca_pml_ucx_start,
+        .pml_improbe       = mca_pml_ucx_improbe,
+        .pml_mprobe        = mca_pml_ucx_mprobe,
+        .pml_imrecv        = mca_pml_ucx_imrecv,
+        .pml_mrecv         = mca_pml_ucx_mrecv,
+        .pml_dump          = mca_pml_ucx_dump,
+        .pml_ft_event      = NULL,
+        .pml_max_contextid = (1ul << (PML_UCX_CONTEXT_BITS)) - 1,
+        .pml_max_tag       = (1ul << (PML_UCX_TAG_BITS - 1)) - 1
     },
-    NULL,   /* ucp_context */
-    NULL    /* ucp_worker */
+    .ucp_context           = NULL,
+    .ucp_worker            = NULL
 };
 
 #define PML_UCX_REQ_ALLOCA() \
     ((char *)alloca(ompi_pml_ucx.request_size) + ompi_pml_ucx.request_size);
 
+#if HAVE_UCP_WORKER_ADDRESS_FLAGS
+static int mca_pml_ucx_send_worker_address_type(int addr_flags, int modex_scope)
+{
+    ucs_status_t status;
+    ucp_worker_attr_t attrs;
+    int rc;
+
+    attrs.field_mask    = UCP_WORKER_ATTR_FIELD_ADDRESS |
+                          UCP_WORKER_ATTR_FIELD_ADDRESS_FLAGS;
+    attrs.address_flags = addr_flags;
+
+    status = ucp_worker_query(ompi_pml_ucx.ucp_worker, &attrs);
+    if (UCS_OK != status) {
+        PML_UCX_ERROR("Failed to query UCP worker address");
+        return OMPI_ERROR;
+    }
+
+    OPAL_MODEX_SEND(rc, modex_scope, &mca_pml_ucx_component.pmlm_version,
+                    (void*)attrs.address, attrs.address_length);
+
+    ucp_worker_release_address(ompi_pml_ucx.ucp_worker, attrs.address);
+
+    if (OMPI_SUCCESS != rc) {
+        return OMPI_ERROR;
+    }
+
+    PML_UCX_VERBOSE(2, "Pack %s worker address, size %ld",
+                    (modex_scope == OPAL_PMIX_LOCAL) ? "local" : "remote",
+                    attrs.address_length);
+
+    return OMPI_SUCCESS;
+}
+#endif
 
 static int mca_pml_ucx_send_worker_address(void)
 {
-    ucp_address_t *address;
     ucs_status_t status;
+
+#if !HAVE_UCP_WORKER_ADDRESS_FLAGS
+    ucp_address_t *address;
     size_t addrlen;
     int rc;
 
@@ -92,16 +131,35 @@ static int mca_pml_ucx_send_worker_address(void)
         return OMPI_ERROR;
     }
 
+    PML_UCX_VERBOSE(2, "Pack worker address, size %ld", addrlen);
+
     OPAL_MODEX_SEND(rc, OPAL_PMIX_GLOBAL,
                     &mca_pml_ucx_component.pmlm_version, (void*)address, addrlen);
-    if (OMPI_SUCCESS != rc) {
-        PML_UCX_ERROR("Open MPI couldn't distribute EP connection details");
-        return OMPI_ERROR;
-    }
 
     ucp_worker_release_address(ompi_pml_ucx.ucp_worker, address);
 
+    if (OMPI_SUCCESS != rc) {
+        goto err;
+    }
+#else
+    /* Pack just network device addresses for remote node peers */
+    status = mca_pml_ucx_send_worker_address_type(UCP_WORKER_ADDRESS_FLAG_NET_ONLY,
+                                                  OPAL_PMIX_REMOTE);
+    if (UCS_OK != status) {
+        goto err;
+    }
+
+    status = mca_pml_ucx_send_worker_address_type(0, OPAL_PMIX_LOCAL);
+    if (UCS_OK != status) {
+        goto err;
+    }
+#endif
+
     return OMPI_SUCCESS;
+
+err:
+    PML_UCX_ERROR("Open MPI couldn't distribute EP connection details");
+    return OMPI_ERROR;
 }
 
 static int mca_pml_ucx_recv_worker_address(ompi_proc_t *proc,
@@ -117,6 +175,9 @@ static int mca_pml_ucx_recv_worker_address(ompi_proc_t *proc,
         PML_UCX_ERROR("Failed to receive UCX worker address: %s (%d)",
                       opal_strerror(ret), ret);
     }
+
+    PML_UCX_VERBOSE(2, "Got proc %d address, size %ld",
+                    proc->super.proc_name.vpid, *addrlen_p);
     return ret;
 }
 
@@ -136,22 +197,27 @@ int mca_pml_ucx_open(void)
     }
 
     /* Initialize UCX context */
-    params.field_mask      = UCP_PARAM_FIELD_FEATURES |
-                             UCP_PARAM_FIELD_REQUEST_SIZE |
-                             UCP_PARAM_FIELD_REQUEST_INIT |
-                             UCP_PARAM_FIELD_REQUEST_CLEANUP |
-                             UCP_PARAM_FIELD_TAG_SENDER_MASK |
-                             UCP_PARAM_FIELD_MT_WORKERS_SHARED |
-                             UCP_PARAM_FIELD_ESTIMATED_NUM_EPS;
-    params.features        = UCP_FEATURE_TAG;
-    params.request_size    = sizeof(ompi_request_t);
-    params.request_init    = mca_pml_ucx_request_init;
-    params.request_cleanup = mca_pml_ucx_request_cleanup;
-    params.tag_sender_mask = PML_UCX_SPECIFIC_SOURCE_MASK;
+    params.field_mask        = UCP_PARAM_FIELD_FEATURES |
+                               UCP_PARAM_FIELD_REQUEST_SIZE |
+                               UCP_PARAM_FIELD_REQUEST_INIT |
+                               UCP_PARAM_FIELD_REQUEST_CLEANUP |
+                               UCP_PARAM_FIELD_TAG_SENDER_MASK |
+                               UCP_PARAM_FIELD_MT_WORKERS_SHARED |
+                               UCP_PARAM_FIELD_ESTIMATED_NUM_EPS;
+    params.features          = UCP_FEATURE_TAG;
+    params.request_size      = sizeof(ompi_request_t);
+    params.request_init      = mca_pml_ucx_request_init;
+    params.request_cleanup   = mca_pml_ucx_request_cleanup;
+    params.tag_sender_mask   = PML_UCX_SPECIFIC_SOURCE_MASK;
     params.mt_workers_shared = 0; /* we do not need mt support for context
                                      since it will be protected by worker */
     params.estimated_num_eps = ompi_proc_world_size();
 
+#if HAVE_DECL_UCP_PARAM_FIELD_ESTIMATED_NUM_PPN
+    params.estimated_num_ppn = opal_process_info.num_local_peers + 1;
+    params.field_mask       |= UCP_PARAM_FIELD_ESTIMATED_NUM_PPN;
+#endif
+
     status = ucp_init(&params, config, &ompi_pml_ucx.ucp_context);
     ucp_config_release(config);
 
@@ -184,19 +250,18 @@ int mca_pml_ucx_close(void)
     return OMPI_SUCCESS;
 }
 
-int mca_pml_ucx_init(void)
+int mca_pml_ucx_init(int enable_mpi_threads)
 {
     ucp_worker_params_t params;
-    ucs_status_t status;
     ucp_worker_attr_t attr;
-    int rc;
+    ucs_status_t status;
+    int i, rc;
 
     PML_UCX_VERBOSE(1, "mca_pml_ucx_init");
 
     /* TODO check MPI thread mode */
     params.field_mask  = UCP_WORKER_PARAM_FIELD_THREAD_MODE;
-    params.thread_mode = UCS_THREAD_MODE_SINGLE;
-    if (ompi_mpi_thread_multiple) {
+    if (enable_mpi_threads) {
         params.thread_mode = UCS_THREAD_MODE_MULTI;
     } else {
         params.thread_mode = UCS_THREAD_MODE_SINGLE;
@@ -206,30 +271,35 @@ int mca_pml_ucx_init(void)
                                &ompi_pml_ucx.ucp_worker);
     if (UCS_OK != status) {
         PML_UCX_ERROR("Failed to create UCP worker");
-        return OMPI_ERROR;
+        rc = OMPI_ERROR;
+        goto err;
     }
 
     attr.field_mask = UCP_WORKER_ATTR_FIELD_THREAD_MODE;
     status = ucp_worker_query(ompi_pml_ucx.ucp_worker, &attr);
     if (UCS_OK != status) {
-        ucp_worker_destroy(ompi_pml_ucx.ucp_worker);
-        ompi_pml_ucx.ucp_worker = NULL;
         PML_UCX_ERROR("Failed to query UCP worker thread level");
-        return OMPI_ERROR;
+        rc = OMPI_ERROR;
+        goto err_destroy_worker;
     }
 
-    if (ompi_mpi_thread_multiple && attr.thread_mode != UCS_THREAD_MODE_MULTI) {
+    if (enable_mpi_threads && (attr.thread_mode != UCS_THREAD_MODE_MULTI)) {
         /* UCX does not support multithreading, disqualify current PML for now */
         /* TODO: we should let OMPI to fallback to THREAD_SINGLE mode */
-        ucp_worker_destroy(ompi_pml_ucx.ucp_worker);
-        ompi_pml_ucx.ucp_worker = NULL;
-        PML_UCX_ERROR("UCP worker does not support MPI_THREAD_MULTIPLE");
-        return OMPI_ERROR;
+        PML_UCX_VERBOSE(1, "UCP worker does not support MPI_THREAD_MULTIPLE. "
+                           "PML UCX could not be selected");
+        rc = OMPI_ERR_NOT_SUPPORTED;
+        goto err_destroy_worker;
     }
 
     rc = mca_pml_ucx_send_worker_address();
     if (rc < 0) {
-        return rc;
+        goto err_destroy_worker;
+    }
+
+    ompi_pml_ucx.datatype_attr_keyval = MPI_KEYVAL_INVALID;
+    for (i = 0; i < OMPI_DATATYPE_MAX_PREDEFINED; ++i) {
+        ompi_pml_ucx.predefined_types[i] = PML_UCX_DATATYPE_INVALID;
     }
 
     /* Initialize the free lists */
@@ -245,15 +315,34 @@ int mca_pml_ucx_init(void)
     PML_UCX_VERBOSE(2, "created ucp context %p, worker %p",
                     (void *)ompi_pml_ucx.ucp_context,
                     (void *)ompi_pml_ucx.ucp_worker);
-    return OMPI_SUCCESS;
+    return rc;
+
+err_destroy_worker:
+    ucp_worker_destroy(ompi_pml_ucx.ucp_worker);
+    ompi_pml_ucx.ucp_worker = NULL;
+err:
+    return OMPI_ERROR;
 }
 
 int mca_pml_ucx_cleanup(void)
 {
+    int i;
+
     PML_UCX_VERBOSE(1, "mca_pml_ucx_cleanup");
 
     opal_progress_unregister(mca_pml_ucx_progress);
 
+    if (ompi_pml_ucx.datatype_attr_keyval != MPI_KEYVAL_INVALID) {
+        ompi_attr_free_keyval(TYPE_ATTR, &ompi_pml_ucx.datatype_attr_keyval, false);
+    }
+
+    for (i = 0; i < OMPI_DATATYPE_MAX_PREDEFINED; ++i) {
+        if (ompi_pml_ucx.predefined_types[i] != PML_UCX_DATATYPE_INVALID) {
+            ucp_dt_destroy(ompi_pml_ucx.predefined_types[i]);
+            ompi_pml_ucx.predefined_types[i] = PML_UCX_DATATYPE_INVALID;
+        }
+    }
+
     ompi_pml_ucx.completed_send_req.req_state = OMPI_REQUEST_INVALID;
     OMPI_REQUEST_FINI(&ompi_pml_ucx.completed_send_req);
     OBJ_DESTRUCT(&ompi_pml_ucx.completed_send_req);
@@ -338,6 +427,7 @@ int mca_pml_ucx_add_procs(struct ompi_proc_t **procs, size_t nprocs)
         }
     }
 
+    opal_common_ucx_mca_proc_added();
     return OMPI_SUCCESS;
 }
 
@@ -365,93 +455,52 @@ static inline ucp_ep_h mca_pml_ucx_get_ep(ompi_communicator_t *comm, int rank)
     return NULL;
 }
 
-static void mca_pml_ucx_waitall(void **reqs, size_t *count_p)
-{
-    ucs_status_t status;
-    size_t i;
-
-    PML_UCX_VERBOSE(2, "waiting for %d disconnect requests", (int)*count_p);
-    for (i = 0; i < *count_p; ++i) {
-        do {
-            opal_progress();
-            status = ucp_request_test(reqs[i], NULL);
-        } while (status == UCS_INPROGRESS);
-        if (status != UCS_OK) {
-            PML_UCX_ERROR("disconnect request failed: %s",
-                          ucs_status_string(status));
-        }
-        ucp_request_free(reqs[i]);
-        reqs[i] = NULL;
-    }
-
-    *count_p = 0;
-}
-
-static void mca_pml_fence_complete_cb(int status, void *fenced)
-{
-    *(int*)fenced = 1;
-}
-
 int mca_pml_ucx_del_procs(struct ompi_proc_t **procs, size_t nprocs)
 {
-    int fenced = 0;
     ompi_proc_t *proc;
-    size_t num_reqs, max_reqs;
-    void *dreq, **dreqs;
-    ucp_ep_h ep;
+    opal_common_ucx_del_proc_t *del_procs;
     size_t i;
+    int ret;
 
-    max_reqs = ompi_pml_ucx.num_disconnect;
-    if (max_reqs > nprocs) {
-        max_reqs = nprocs;
-    }
-
-    dreqs = malloc(sizeof(*dreqs) * max_reqs);
-    if (dreqs == NULL) {
+    del_procs = malloc(sizeof(*del_procs) * nprocs);
+    if (del_procs == NULL) {
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
-    num_reqs = 0;
-
     for (i = 0; i < nprocs; ++i) {
-        proc = procs[(i + OMPI_PROC_MY_NAME->vpid) % nprocs];
-        ep   = proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML];
-        if (ep == NULL) {
-            continue;
-        }
-
-        PML_UCX_VERBOSE(2, "disconnecting from rank %d", proc->super.proc_name.vpid);
-        dreq = ucp_disconnect_nb(ep);
-        if (dreq != NULL) {
-            if (UCS_PTR_IS_ERR(dreq)) {
-                PML_UCX_ERROR("ucp_disconnect_nb(%d) failed: %s",
-                              proc->super.proc_name.vpid,
-                              ucs_status_string(UCS_PTR_STATUS(dreq)));
-            } else {
-                dreqs[num_reqs++] = dreq;
-            }
-        }
+        proc = procs[i];
+        del_procs[i].ep   = proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML];
+        del_procs[i].vpid = proc->super.proc_name.vpid;
 
+        /* mark peer as disconnected */
         proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML] = NULL;
-
-        if ((int)num_reqs >= ompi_pml_ucx.num_disconnect) {
-            mca_pml_ucx_waitall(dreqs, &num_reqs);
-        }
     }
 
-    mca_pml_ucx_waitall(dreqs, &num_reqs);
-    free(dreqs);
+    ret = opal_common_ucx_del_procs(del_procs, nprocs, OMPI_PROC_MY_NAME->vpid,
+                                    ompi_pml_ucx.num_disconnect, ompi_pml_ucx.ucp_worker);
+    free(del_procs);
 
-    opal_pmix.fence_nb(NULL, 0, mca_pml_fence_complete_cb, &fenced);
-    while (!fenced) {
-        ucp_worker_progress(ompi_pml_ucx.ucp_worker);
-    }
-
-    return OMPI_SUCCESS;
+    return ret;
 }
 
 int mca_pml_ucx_enable(bool enable)
 {
+    ompi_attribute_fn_ptr_union_t copy_fn;
+    ompi_attribute_fn_ptr_union_t del_fn;
+    int ret;
+
+    /* Create a key for adding custom attributes to datatypes */
+    copy_fn.attr_datatype_copy_fn  =
+                    (MPI_Type_internal_copy_attr_function*)MPI_TYPE_NULL_COPY_FN;
+    del_fn.attr_datatype_delete_fn = mca_pml_ucx_datatype_attr_del_fn;
+    ret = ompi_attr_create_keyval(TYPE_ATTR, copy_fn, del_fn,
+                                  &ompi_pml_ucx.datatype_attr_keyval, NULL, 0,
+                                  NULL);
+    if (ret != OMPI_SUCCESS) {
+        PML_UCX_ERROR("Failed to create keyval for UCX datatypes: %d", ret);
+        return ret;
+    }
+
     PML_UCX_FREELIST_INIT(&ompi_pml_ucx.persistent_reqs,
                           mca_pml_ucx_persistent_request_t,
                           128, -1, 128);
@@ -491,11 +540,12 @@ int mca_pml_ucx_irecv_init(void *buf, size_t count, ompi_datatype_t *datatype,
     PML_UCX_TRACE_RECV("irecv_init request *%p=%p", buf, count, datatype, src,
                        tag, comm, (void*)request, (void*)req);
 
-    req->ompi.req_state = OMPI_REQUEST_INACTIVE;
-    req->flags          = 0;
-    req->buffer         = buf;
-    req->count          = count;
-    req->datatype       = mca_pml_ucx_get_datatype(datatype);
+    req->ompi.req_state           = OMPI_REQUEST_INACTIVE;
+    req->ompi.req_mpi_object.comm = comm;
+    req->flags                    = 0;
+    req->buffer                   = buf;
+    req->count                    = count;
+    req->datatype.datatype        = mca_pml_ucx_get_datatype(datatype);
 
     PML_UCX_MAKE_RECV_TAG(req->tag, req->recv.tag_mask, tag, src, comm);
 
@@ -524,7 +574,8 @@ int mca_pml_ucx_irecv(void *buf, size_t count, ompi_datatype_t *datatype,
     }
 
     PML_UCX_VERBOSE(8, "got request %p", (void*)req);
-    *request = req;
+    req->req_mpi_object.comm = comm;
+    *request                 = req;
     return OMPI_SUCCESS;
 }
 
@@ -539,19 +590,19 @@ int mca_pml_ucx_recv(void *buf, size_t count, ompi_datatype_t *datatype, int src
 
     PML_UCX_TRACE_RECV("%s", buf, count, datatype, src, tag, comm, "recv");
 
+    /* coverity[bad_alloc_arithmetic] */
     PML_UCX_MAKE_RECV_TAG(ucp_tag, ucp_tag_mask, tag, src, comm);
     req = PML_UCX_REQ_ALLOCA();
     status = ucp_tag_recv_nbr(ompi_pml_ucx.ucp_worker, buf, count,
                               mca_pml_ucx_get_datatype(datatype),
                               ucp_tag, ucp_tag_mask, req);
 
-    for (;;) {
+    MCA_COMMON_UCX_PROGRESS_LOOP(ompi_pml_ucx.ucp_worker) {
         status = ucp_request_test(req, &info);
         if (status != UCS_INPROGRESS) {
             mca_pml_ucx_set_recv_status_safe(mpi_status, status, &info);
             return OMPI_SUCCESS;
         }
-        opal_progress();
     }
 }
 
@@ -596,18 +647,20 @@ int mca_pml_ucx_isend_init(const void *buf, size_t count, ompi_datatype_t *datat
         return OMPI_ERROR;
     }
 
-    req->ompi.req_state = OMPI_REQUEST_INACTIVE;
-    req->flags          = MCA_PML_UCX_REQUEST_FLAG_SEND;
-    req->buffer         = (void *)buf;
-    req->count          = count;
-    req->tag            = PML_UCX_MAKE_SEND_TAG(tag, comm);
-    req->send.mode      = mode;
-    req->send.ep        = ep;
+    req->ompi.req_state           = OMPI_REQUEST_INACTIVE;
+    req->ompi.req_mpi_object.comm = comm;
+    req->flags                    = MCA_PML_UCX_REQUEST_FLAG_SEND;
+    req->buffer                   = (void *)buf;
+    req->count                    = count;
+    req->tag                      = PML_UCX_MAKE_SEND_TAG(tag, comm);
+    req->send.mode                = mode;
+    req->send.ep                  = ep;
+
     if (MCA_PML_BASE_SEND_BUFFERED == mode) {
-        req->ompi_datatype = datatype;
+        req->datatype.ompi_datatype = datatype;
         OBJ_RETAIN(datatype);
     } else {
-        req->datatype = mca_pml_ucx_get_datatype(datatype);
+        req->datatype.datatype = mca_pml_ucx_get_datatype(datatype);
     }
 
     *request = &req->ompi;
@@ -720,7 +773,8 @@ int mca_pml_ucx_isend(const void *buf, size_t count, ompi_datatype_t *datatype,
         return OMPI_SUCCESS;
     } else if (!UCS_PTR_IS_ERR(req)) {
         PML_UCX_VERBOSE(8, "got request %p", (void*)req);
-        *request = req;
+        req->req_mpi_object.comm = comm;
+        *request                 = req;
         return OMPI_SUCCESS;
     } else {
         PML_UCX_ERROR("ucx send failed: %s", ucs_status_string(UCS_PTR_STATUS(req)));
@@ -738,16 +792,12 @@ mca_pml_ucx_send_nb(ucp_ep_h ep, const void *buf, size_t count,
 
     req = (ompi_request_t*)mca_pml_ucx_common_send(ep, buf, count, datatype,
                                                    mca_pml_ucx_get_datatype(datatype),
-                                                   tag, mode,
-                                                   mca_pml_ucx_send_completion);
-
+                                                   tag, mode, cb);
     if (OPAL_LIKELY(req == NULL)) {
         return OMPI_SUCCESS;
     } else if (!UCS_PTR_IS_ERR(req)) {
         PML_UCX_VERBOSE(8, "got request %p", (void*)req);
-        ucp_worker_progress(ompi_pml_ucx.ucp_worker);
-        ompi_request_wait(&req, MPI_STATUS_IGNORE);
-        return OMPI_SUCCESS;
+        MCA_COMMON_UCX_WAIT_LOOP(req, ompi_pml_ucx.ucp_worker, "ucx send", ompi_request_free(&req));
     } else {
         PML_UCX_ERROR("ucx send failed: %s", ucs_status_string(UCS_PTR_STATUS(req)));
         return OMPI_ERROR;
@@ -760,21 +810,17 @@ mca_pml_ucx_send_nbr(ucp_ep_h ep, const void *buf, size_t count,
                      ucp_datatype_t ucx_datatype, ucp_tag_t tag)
 
 {
-    void *req;
+    ucs_status_ptr_t req;
     ucs_status_t status;
 
+    /* coverity[bad_alloc_arithmetic] */
     req    = PML_UCX_REQ_ALLOCA();
     status = ucp_tag_send_nbr(ep, buf, count, ucx_datatype, tag, req);
     if (OPAL_LIKELY(status == UCS_OK)) {
         return OMPI_SUCCESS;
     }
 
-    ucp_worker_progress(ompi_pml_ucx.ucp_worker);
-    while ((status = ucp_request_check_status(req)) == UCS_INPROGRESS) {
-        opal_progress();
-    }
-
-    return OPAL_LIKELY(UCS_OK == status) ? OMPI_SUCCESS : OMPI_ERROR;
+    MCA_COMMON_UCX_WAIT_LOOP(req, ompi_pml_ucx.ucp_worker, "ucx send", (void)0);
 }
 #endif
 
@@ -808,8 +854,10 @@ int mca_pml_ucx_send(const void *buf, size_t count, ompi_datatype_t *datatype, i
 }
 
 int mca_pml_ucx_iprobe(int src, int tag, struct ompi_communicator_t* comm,
-                         int *matched, ompi_status_public_t* mpi_status)
+                       int *matched, ompi_status_public_t* mpi_status)
 {
+    static unsigned progress_count = 0;
+
     ucp_tag_t ucp_tag, ucp_tag_mask;
     ucp_tag_recv_info_t info;
     ucp_tag_message_h ucp_msg;
@@ -822,15 +870,16 @@ int mca_pml_ucx_iprobe(int src, int tag, struct ompi_communicator_t* comm,
     if (ucp_msg != NULL) {
         *matched = 1;
         mca_pml_ucx_set_recv_status_safe(mpi_status, UCS_OK, &info);
-    } else {
-        opal_progress();
+    } else  {
+        (++progress_count % opal_common_ucx.progress_iterations) ?
+            (void)ucp_worker_progress(ompi_pml_ucx.ucp_worker) : opal_progress();
         *matched = 0;
     }
     return OMPI_SUCCESS;
 }
 
 int mca_pml_ucx_probe(int src, int tag, struct ompi_communicator_t* comm,
-                        ompi_status_public_t* mpi_status)
+                      ompi_status_public_t* mpi_status)
 {
     ucp_tag_t ucp_tag, ucp_tag_mask;
     ucp_tag_recv_info_t info;
@@ -839,22 +888,23 @@ int mca_pml_ucx_probe(int src, int tag, struct ompi_communicator_t* comm,
     PML_UCX_TRACE_PROBE("probe", src, tag, comm);
 
     PML_UCX_MAKE_RECV_TAG(ucp_tag, ucp_tag_mask, tag, src, comm);
-    for (;;) {
-        ucp_msg = ucp_tag_probe_nb(ompi_pml_ucx.ucp_worker, ucp_tag, ucp_tag_mask,
-                                   0, &info);
+
+    MCA_COMMON_UCX_PROGRESS_LOOP(ompi_pml_ucx.ucp_worker) {
+        ucp_msg = ucp_tag_probe_nb(ompi_pml_ucx.ucp_worker, ucp_tag,
+                                   ucp_tag_mask, 0, &info);
         if (ucp_msg != NULL) {
             mca_pml_ucx_set_recv_status_safe(mpi_status, UCS_OK, &info);
             return OMPI_SUCCESS;
         }
-
-        opal_progress();
     }
 }
 
 int mca_pml_ucx_improbe(int src, int tag, struct ompi_communicator_t* comm,
-                          int *matched, struct ompi_message_t **message,
-                          ompi_status_public_t* mpi_status)
+                        int *matched, struct ompi_message_t **message,
+                        ompi_status_public_t* mpi_status)
 {
+    static unsigned progress_count = 0;
+
     ucp_tag_t ucp_tag, ucp_tag_mask;
     ucp_tag_recv_info_t info;
     ucp_tag_message_h ucp_msg;
@@ -870,7 +920,8 @@ int mca_pml_ucx_improbe(int src, int tag, struct ompi_communicator_t* comm,
         *matched         = 1;
         mca_pml_ucx_set_recv_status_safe(mpi_status, UCS_OK, &info);
     } else  {
-        opal_progress();
+        (++progress_count % opal_common_ucx.progress_iterations) ?
+            (void)ucp_worker_progress(ompi_pml_ucx.ucp_worker) : opal_progress();
         *matched = 0;
     }
     return OMPI_SUCCESS;
@@ -887,7 +938,7 @@ int mca_pml_ucx_mprobe(int src, int tag, struct ompi_communicator_t* comm,
     PML_UCX_TRACE_PROBE("mprobe", src, tag, comm);
 
     PML_UCX_MAKE_RECV_TAG(ucp_tag, ucp_tag_mask, tag, src, comm);
-    for (;;) {
+    MCA_COMMON_UCX_PROGRESS_LOOP(ompi_pml_ucx.ucp_worker) {
         ucp_msg = ucp_tag_probe_nb(ompi_pml_ucx.ucp_worker, ucp_tag, ucp_tag_mask,
                                    1, &info);
         if (ucp_msg != NULL) {
@@ -896,8 +947,6 @@ int mca_pml_ucx_mprobe(int src, int tag, struct ompi_communicator_t* comm,
             mca_pml_ucx_set_recv_status_safe(mpi_status, UCS_OK, &info);
             return OMPI_SUCCESS;
         }
-
-        opal_progress();
     }
 }
 
@@ -969,8 +1018,8 @@ int mca_pml_ucx_start(size_t count, ompi_request_t** requests)
             tmp_req = (ompi_request_t*)mca_pml_ucx_common_send(preq->send.ep,
                                                                preq->buffer,
                                                                preq->count,
-                                                               preq->ompi_datatype,
-                                                               preq->datatype,
+                                                               preq->datatype.ompi_datatype,
+                                                               preq->datatype.datatype,
                                                                preq->tag,
                                                                preq->send.mode,
                                                                mca_pml_ucx_psend_completion);
@@ -978,7 +1027,8 @@ int mca_pml_ucx_start(size_t count, ompi_request_t** requests)
             PML_UCX_VERBOSE(8, "start recv request %p", (void*)preq);
             tmp_req = (ompi_request_t*)ucp_tag_recv_nb(ompi_pml_ucx.ucp_worker,
                                                        preq->buffer, preq->count,
-                                                       preq->datatype, preq->tag,
+                                                       preq->datatype.datatype,
+                                                       preq->tag,
                                                        preq->recv.tag_mask,
                                                        mca_pml_ucx_precv_completion);
         }
