@@ -15,12 +15,18 @@
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/pml/base/base.h"
 #include "ompi/datatype/ompi_datatype.h"
+#include "ompi/datatype/ompi_datatype_internal.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/request/request.h"
+#include "opal/mca/common/ucx/common_ucx.h"
 
 #include <ucp/api/ucp.h>
 #include "pml_ucx_freelist.h"
 
+#define PML_UCX_ASSERT  MCA_COMMON_UCX_ASSERT
+#define PML_UCX_ERROR   MCA_COMMON_UCX_ERROR
+#define PML_UCX_VERBOSE MCA_COMMON_UCX_VERBOSE
+
 
 typedef struct mca_pml_ucx_module           mca_pml_ucx_module_t;
 typedef struct pml_ucx_persistent_request   mca_pml_ucx_persistent_request_t;
@@ -37,55 +43,28 @@ struct mca_pml_ucx_module {
     ucp_context_h             ucp_context;
     ucp_worker_h              ucp_worker;
 
+    /* Datatypes */
+    int                       datatype_attr_keyval;
+    ucp_datatype_t            predefined_types[OMPI_DATATYPE_MPI_MAX_PREDEFINED];
+
     /* Requests */
     mca_pml_ucx_freelist_t    persistent_reqs;
     ompi_request_t            completed_send_req;
     size_t                    request_size;
     int                       num_disconnect;
-    bool                      opal_mem_hooks;
 
     /* Converters pool */
     mca_pml_ucx_freelist_t    convs;
 
     int                       priority;
-    int                       verbose;
-    int                       output;
 };
 
 extern mca_pml_base_component_2_0_0_t mca_pml_ucx_component;
 extern mca_pml_ucx_module_t ompi_pml_ucx;
 
-
-/* Debugging */
-#define PML_UCX_ENABLE_DEBUG   OPAL_ENABLE_DEBUG
-#if PML_UCX_ENABLE_DEBUG
-#  define PML_UCX_MAX_VERBOSE  9
-#  define PML_UCX_ASSERT(_x)   assert(_x)
-#else
-#  define PML_UCX_MAX_VERBOSE  2
-#  define PML_UCX_ASSERT(_x)
-#endif
-
-#define _PML_UCX_QUOTE(_x) \
-    # _x
-#define PML_UCX_QUOTE(_x) \
-    _PML_UCX_QUOTE(_x)
-
-#define PML_UCX_ERROR(...) \
-    opal_output_verbose(0, ompi_pml_ucx.output, \
-                        __FILE__ ":" PML_UCX_QUOTE(__LINE__) \
-                        " Error: " __VA_ARGS__)
-
-#define PML_UCX_VERBOSE(_level, ... ) \
-    if (((_level) <= PML_UCX_MAX_VERBOSE) && ((_level) <= ompi_pml_ucx.verbose)) { \
-        opal_output_verbose(_level, ompi_pml_ucx.output, \
-                            __FILE__ ":" PML_UCX_QUOTE(__LINE__) " " \
-                            __VA_ARGS__); \
-    }
-
 int mca_pml_ucx_open(void);
 int mca_pml_ucx_close(void);
-int mca_pml_ucx_init(void);
+int mca_pml_ucx_init(int enable_mpi_threads);
 int mca_pml_ucx_cleanup(void);
 
 int mca_pml_ucx_add_procs(struct ompi_proc_t **procs, size_t nprocs);
