@@ -80,14 +80,15 @@ static void mca_pml_ucx_persistent_request_detach(mca_pml_ucx_persistent_request
     preq->tmp_req                 = NULL;
 }
 
-void mca_pml_ucx_persistent_request_complete(mca_pml_ucx_persistent_request_t *preq,
-                                             ompi_request_t *tmp_req)
+inline void
+mca_pml_ucx_persistent_request_complete(mca_pml_ucx_persistent_request_t *preq,
+                                        ompi_request_t *tmp_req)
 {
     preq->ompi.req_status = tmp_req->req_status;
-    ompi_request_complete(&preq->ompi, true);
-    mca_pml_ucx_persistent_request_detach(preq, tmp_req);
     mca_pml_ucx_request_reset(tmp_req);
+    mca_pml_ucx_persistent_request_detach(preq, tmp_req);
     ucp_request_free(tmp_req);
+    ompi_request_complete(&preq->ompi, true);
 }
 
 static inline void mca_pml_ucx_preq_completion(ompi_request_t *tmp_req)
@@ -135,6 +136,7 @@ static void mca_pml_ucx_request_init_common(ompi_request_t* ompi_req,
     OMPI_REQUEST_INIT(ompi_req, req_persistent);
     ompi_req->req_type             = OMPI_REQUEST_PML;
     ompi_req->req_state            = state;
+    ompi_req->req_start            = mca_pml_ucx_start;
     ompi_req->req_free             = req_free;
     ompi_req->req_cancel           = req_cancel;
     /* This field is used to attach persistant request to a temporary req.
@@ -174,7 +176,7 @@ static int mca_pml_ucx_persistent_request_free(ompi_request_t **rptr)
     }
     if ((preq->flags & MCA_PML_UCX_REQUEST_FLAG_SEND) &&
          (MCA_PML_BASE_SEND_BUFFERED == preq->send.mode)) {
-        OBJ_RELEASE(preq->ompi_datatype);
+        OBJ_RELEASE(preq->datatype.ompi_datatype);
     }
     PML_UCX_FREELIST_RETURN(&ompi_pml_ucx.persistent_reqs, &preq->ompi.super);
     *rptr = MPI_REQUEST_NULL;
@@ -226,7 +228,7 @@ void mca_pml_ucx_completed_request_init(ompi_request_t *ompi_req)
     mca_pml_ucx_request_init_common(ompi_req, false, OMPI_REQUEST_ACTIVE,
                                     mca_pml_completed_request_free,
                                     mca_pml_completed_request_cancel);
+    ompi_req->req_mpi_object.comm = &ompi_mpi_comm_world.comm;
     ompi_request_complete(ompi_req, false);
-
 }
 
