@@ -3,6 +3,8 @@
 ! Copyright (c) 2006-2014 Cisco Systems, Inc.  All rights reserved.
 ! Copyright (c) 2009-2012 Los Alamos National Security, LLC.
 !                         All rights reserved.
+! Copyright (c) 2019      Research Organization for Information Science
+!                         and Technology (RIST).  All rights reserved.
 !
 ! $COPYRIGHT$
 !
@@ -38,11 +40,4 @@
 #define OMPI_PRIVATE
 #endif
 
-! PROTECTED or not
-#if OMPI_FORTRAN_HAVE_PROTECTED
-#define OMPI_PROTECTED , PROTECTED
-#else
-#define OMPI_PROTECTED
-#endif
-
 #endif
