@@ -3,6 +3,7 @@
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -31,9 +32,9 @@ int MPI_T_category_get_num (int *num_cat)
         return MPI_ERR_ARG;
     }
 
-    mpit_lock ();
+    ompi_mpit_lock ();
     *num_cat = mca_base_var_group_get_count ();
-    mpit_unlock ();
+    ompi_mpit_unlock ();
 
     return MPI_SUCCESS;
 }
