@@ -3,6 +3,7 @@
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -30,9 +31,9 @@ int MPI_T_cvar_get_num (int *num_cvar) {
         return MPI_ERR_ARG;
     }
 
-    mpit_lock ();
+    ompi_mpit_lock ();
     *num_cvar = mca_base_var_get_count();
-    mpit_unlock ();
+    ompi_mpit_unlock ();
 
     return MPI_SUCCESS;
 }
