@@ -3,6 +3,7 @@
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,14 +30,14 @@ int MPI_T_pvar_session_create(MPI_T_pvar_session *session)
         return MPI_T_ERR_NOT_INITIALIZED;
     }
 
-    mpit_lock ();
+    ompi_mpit_lock ();
 
     *session = OBJ_NEW(mca_base_pvar_session_t);
     if (NULL == *session) {
         ret = MPI_ERR_NO_MEM;
     }
 
-    mpit_unlock ();
+    ompi_mpit_unlock ();
 
     return ret;
 }
