@@ -37,10 +37,11 @@
 
 /* by default the debuging is turned off */
 int opal_datatype_dfd = -1;
-bool opal_unpack_debug = false;
-bool opal_pack_debug = false;
-bool opal_position_debug = false;
-bool opal_copy_debug = false;
+bool opal_ddt_unpack_debug = false;
+bool opal_ddt_pack_debug = false;
+bool opal_ddt_position_debug = false;
+bool opal_ddt_copy_debug = false;
+bool opal_ddt_raw_debug = false;
 int opal_ddt_verbose = -1;  /* Has the datatype verbose it's own output stream */
 
 extern int opal_cuda_verbose;
@@ -148,35 +149,43 @@ int opal_datatype_register_params(void)
     int ret;
 
     ret = mca_base_var_register ("opal", "mpi", NULL, "ddt_unpack_debug",
-				 "Whether to output debugging information in the ddt unpack functions (nonzero = enabled)",
-				 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_3,
-				 MCA_BASE_VAR_SCOPE_LOCAL, &opal_unpack_debug);
+                                 "Whether to output debugging information in the ddt unpack functions (nonzero = enabled)",
+                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_3,
+                                 MCA_BASE_VAR_SCOPE_LOCAL, &opal_ddt_unpack_debug);
     if (0 > ret) {
-	return ret;
+        return ret;
     }
 
     ret = mca_base_var_register ("opal", "mpi", NULL, "ddt_pack_debug",
-				 "Whether to output debugging information in the ddt pack functions (nonzero = enabled)",
-				 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_3,
-				 MCA_BASE_VAR_SCOPE_LOCAL, &opal_pack_debug);
+                                 "Whether to output debugging information in the ddt pack functions (nonzero = enabled)",
+                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_3,
+                                 MCA_BASE_VAR_SCOPE_LOCAL, &opal_ddt_pack_debug);
     if (0 > ret) {
-	return ret;
+        return ret;
+    }
+
+    ret = mca_base_var_register ("opal", "mpi", NULL, "ddt_raw_debug",
+                                 "Whether to output debugging information in the ddt raw functions (nonzero = enabled)",
+                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_3,
+                                 MCA_BASE_VAR_SCOPE_LOCAL, &opal_ddt_raw_debug);
+    if (0 > ret) {
+        return ret;
     }
 
     ret = mca_base_var_register ("opal", "mpi", NULL, "ddt_position_debug",
-				 "Non zero lead to output generated by the datatype position functions",
-				 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_3,
-				 MCA_BASE_VAR_SCOPE_LOCAL, &opal_position_debug);
+                                 "Non zero lead to output generated by the datatype position functions",
+                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_3,
+                                 MCA_BASE_VAR_SCOPE_LOCAL, &opal_ddt_position_debug);
     if (0 > ret) {
-	return ret;
+        return ret;
     }
 
     ret = mca_base_var_register ("opal", "mpi", NULL, "ddt_copy_debug",
-				 "Whether to output debugging information in the ddt copy functions (nonzero = enabled)",
-				 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_3,
-				 MCA_BASE_VAR_SCOPE_LOCAL, &opal_copy_debug);
+                                 "Whether to output debugging information in the ddt copy functions (nonzero = enabled)",
+                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_3,
+                                 MCA_BASE_VAR_SCOPE_LOCAL, &opal_ddt_copy_debug);
     if (0 > ret) {
-	return ret;
+        return ret;
     }
 
     ret = mca_base_var_register ("opal", "opal", NULL, "ddt_verbose",
@@ -195,7 +204,7 @@ int opal_datatype_register_params(void)
                                  OPAL_INFO_LVL_8, MCA_BASE_VAR_SCOPE_LOCAL,
                                  &opal_cuda_verbose);
     if (0 > ret) {
-	return ret;
+        return ret;
     }
 #endif
 
@@ -224,8 +233,8 @@ int32_t opal_datatype_init( void )
                                                    OPAL_DATATYPE_FLAG_CONTIGUOUS |
                                                    OPAL_DATATYPE_FLAG_NO_GAPS;
         datatype->desc.desc[0].elem.common.type  = i;
-        /* datatype->desc.desc[0].elem.blocklen XXX not set at the moment, it will be needed later */
         datatype->desc.desc[0].elem.count        = 1;
+        datatype->desc.desc[0].elem.blocklen     = 1;
         datatype->desc.desc[0].elem.disp         = 0;
         datatype->desc.desc[0].elem.extent       = datatype->size;
 
