@@ -10,7 +10,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2018 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Laboratory
  * Copyright (c) 2012-2015 Los Alamos National Security, LLC.  All rights
@@ -361,9 +361,9 @@ static int mca_btl_tcp_component_open(void)
 #if OPAL_ENABLE_IPV6
     mca_btl_tcp_component.tcp6_listen_sd = -1;
 #endif
-    mca_btl_tcp_component.tcp_num_btls=0;
+    mca_btl_tcp_component.tcp_num_btls = 0;
     mca_btl_tcp_component.tcp_addr_count = 0;
-    mca_btl_tcp_component.tcp_btls=NULL;
+    mca_btl_tcp_component.tcp_btls = NULL;
 
     /* initialize objects */
     OBJ_CONSTRUCT(&mca_btl_tcp_component.tcp_lock, opal_mutex_t);
@@ -511,6 +511,17 @@ static int mca_btl_tcp_create(int if_kindex, const char* if_name)
         btl->tcp_send_handler = 0;
 #endif
 
+       struct sockaddr_storage addr;
+       opal_ifkindextoaddr(if_kindex, (struct sockaddr*) &addr,
+                                          sizeof (struct sockaddr_storage));
+#if OPAL_ENABLE_IPV6
+        if (addr.ss_family == AF_INET6) {
+            btl->tcp_ifaddr_6 =  addr;
+        }
+#endif
+        if (addr.ss_family == AF_INET) {
+           btl->tcp_ifaddr = addr;
+        }
         /* allow user to specify interface bandwidth */
         sprintf(param, "bandwidth_%s", if_name);
         mca_btl_tcp_param_register_uint(param, NULL, btl->super.btl_bandwidth, OPAL_INFO_LVL_5, &btl->super.btl_bandwidth);
@@ -1172,8 +1183,8 @@ static int mca_btl_tcp_component_exchange(void)
                  if ((AF_INET == my_ss.ss_family) &&
                      (4 != mca_btl_tcp_component.tcp_disable_family)) {
                      memcpy(&addrs[current_addr].addr_inet,
-                             &((struct sockaddr_in*)&my_ss)->sin_addr,
-                             sizeof(addrs[0].addr_inet));
+                            &((struct sockaddr_in*)&my_ss)->sin_addr,
+                            sizeof(struct in_addr));
                      addrs[current_addr].addr_port =
                          mca_btl_tcp_component.tcp_listen_port;
                      addrs[current_addr].addr_family = MCA_BTL_TCP_AF_INET;
@@ -1352,7 +1363,6 @@ static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
     mca_btl_tcp_endpoint_hs_msg_t hs_msg;
     struct timeval save, tv;
     socklen_t rcvtimeo_save_len = sizeof(save);
-    char str[128];
 
     /* Note, Socket will be in blocking mode during intial handshake
      * hence setting SO_RCVTIMEO to say 2 seconds here to avoid waiting 
@@ -1365,20 +1375,22 @@ static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
         if (ENOPROTOOPT == errno) {
             sockopt = false;
         } else {
-            opal_output_verbose(20, opal_btl_base_framework.framework_output,
-                                "Cannot get current recv timeout value of the socket"
-                                "Local_host:%s PID:%d",
-                                opal_process_info.nodename, getpid());
+            opal_show_help("help-mpi-btl-tcp.txt", "socket flag fail",
+                           true, opal_process_info.nodename,
+                           getpid(),
+                           "getsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, ...)",
+                           strerror(opal_socket_errno), opal_socket_errno);
             return;
         }
     } else {
         tv.tv_sec = 2;
         tv.tv_usec = 0;
         if (0 != setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv))) {
-            opal_output_verbose(20, opal_btl_base_framework.framework_output,
-                                "Cannot set new recv timeout value of the socket"
-                                "Local_host:%s PID:%d",
-                                opal_process_info.nodename, getpid());
+            opal_show_help("help-mpi-btl-tcp.txt", "socket flag fail",
+                           true, opal_process_info.nodename,
+                           getpid(),
+                           "setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, ...)",
+                           strerror(opal_socket_errno), opal_socket_errno);
            return;
        }
     }
@@ -1386,39 +1398,38 @@ static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
     OBJ_RELEASE(event);
     retval = mca_btl_tcp_recv_blocking(sd, (void *)&hs_msg, sizeof(hs_msg));
 
-    /* An unknown process attempted to connect to Open MPI via TCP.
-     * Open MPI uses a "magic" string to trivially verify that the connecting
-     * process is a fellow Open MPI process.  An MPI process accepted a TCP
-     * connection but did not receive the correct magic string.  This might
-     * indicate an Open MPI version mismatch between different MPI processes
-     * in the same job, or it may indicate that some other agent is
-     * mistakenly attempting to connect to Open MPI's TCP listening sockets.
+    /* If we get a zero-length message back, it's likely that we
+       connected to Open MPI peer process X simultaneously, and the
+       peer closed its connection to us (in favor of our connection to
+       them).  This is not an error -- just close it and move on.
 
-     * This attempted connection will be ignored; your MPI job may or may not
-     * continue properly.
-     */
-     if (sizeof(hs_msg) != retval) {
+       Similarly, if we get less than sizeof(hs_msg) bytes, it
+       probably wasn't an Open MPI peer.  But we don't really care,
+       because the peer closed the socket.  So just close it and move
+       on. */
+    if (retval < sizeof(hs_msg)) {
+         const char *peer = opal_fd_get_peer_name(sd);
          opal_output_verbose(20, opal_btl_base_framework.framework_output,
-                            "process did not receive full connect ACK "
-                            "Local_host:%s PID:%d String_received:%s Test_fail:%s",
-                            opal_process_info.nodename,
-                            getpid(),
-                            (retval > 0) ? hs_msg.magic_id : "<nothing>",
-                            "handshake message length");
-
-         /* The other side probably isn't OMPI, so just hang up */
+                             "Peer %s closed socket without sending BTL TCP magic ID handshake (we received %d bytes out of the expected %d) -- closing/ignoring this connection",
+                             peer, (int) retval, (int) sizeof(hs_msg));
+         free((char*) peer);
          CLOSE_THE_SOCKET(sd);
          return;
     }
 
+    /* Open MPI uses a "magic" string to trivially verify that the
+       connecting process is a fellow Open MPI process.  See if we got
+       the correct magic string. */
     guid = hs_msg.guid;
     if (0 != strncmp(hs_msg.magic_id, mca_btl_tcp_magic_id_string, len)) {
-        opal_output_verbose(20, opal_btl_base_framework.framework_output,
-                            "process did not receive right magic string. "
-                            "Local_host:%s PID:%d String_received:%s Test_fail:%s",
-                            opal_process_info.nodename,
-                            getpid(), hs_msg.magic_id,
-                            "string value");
+         const char *peer = opal_fd_get_peer_name(sd);
+         opal_output_verbose(20, opal_btl_base_framework.framework_output,
+                             "Peer %s send us an incorrect Open MPI magic ID string (i.e., this was not a connection from the same version of Open MPI; expected \"%s\", received \"%s\")",
+                             peer,
+                             mca_btl_tcp_magic_id_string,
+                             hs_msg.magic_id);
+         free((char*) peer);
+
         /* The other side probably isn't OMPI, so just hang up */
         CLOSE_THE_SOCKET(sd);
         return;
@@ -1427,10 +1438,11 @@ static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
     if (sockopt) {
        /* reset RECVTIMEO option to its original state */
        if (0 != setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, &save, sizeof(save))) {
-           opal_output_verbose(20, opal_btl_base_framework.framework_output,
-                               "Cannot reset recv timeout value"
-                               "Local_host:%s PID:%d",
-                               opal_process_info.nodename, getpid());
+           opal_show_help("help-mpi-btl-tcp.txt", "socket flag fail",
+                          true, opal_process_info.nodename,
+                          getpid(),
+                          "setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, ...)",
+                          strerror(opal_socket_errno), opal_socket_errno);
           return;
        }
     }
@@ -1481,24 +1493,9 @@ static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
     /* are there any existing peer instances willing to accept this connection */
     (void)mca_btl_tcp_proc_accept(btl_proc, (struct sockaddr*)&addr, sd);
 
-    switch (addr.ss_family) {
-        case AF_INET:
-            inet_ntop(AF_INET, &(((struct sockaddr_in*) &addr)->sin_addr), str, sizeof(str));
-            break;
-
-    #if OPAL_ENABLE_IPV6
-        case AF_INET6:
-            inet_ntop(AF_INET6, &(((struct sockaddr_in6*) &addr)->sin6_addr), str, sizeof(str));
-            break;
-    #endif
-
-        default:
-            BTL_ERROR(("Got an accept() from an unknown address family -- this shouldn't happen"));
-            CLOSE_THE_SOCKET(sd);
-            return;
-
-    }
+    const char *str = opal_fd_get_peer_name(sd);
     opal_output_verbose(10, opal_btl_base_framework.framework_output,
                         "btl:tcp: now connected to %s, process %s", str,
                         OPAL_NAME_PRINT(btl_proc->proc_opal->proc_name));
+    free((char*) str);
 }
