@@ -15,11 +15,13 @@
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2010-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2018 Intel, Inc. All rights reserved.
  * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2017      Amazon.com, Inc. or its affiliates.
  *                         All Rights reserved.
+ * Copyright (c) 2018      Mellanox Technologies, Inc.
+ *                         All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -305,6 +307,10 @@ opal_err2str(int errnum, const char **errmsg)
     case OPAL_ERR_MODEL_DECLARED:
         retval = "Model declared";
         break;
+    case OPAL_PMIX_LAUNCH_DIRECTIVE:
+        retval = "Launch directive";
+        break;
+
     default:
         retval = "UNRECOGNIZED";
     }
@@ -475,12 +481,16 @@ opal_init_util(int* pargc, char*** pargv)
         goto return_error;
     }
 
+    OPAL_TIMING_ENV_NEXT(otmng, "opal_dss_open");
+
     /* initialize the mca */
     if (OPAL_SUCCESS != (ret = mca_base_open())) {
         error = "mca_base_open";
         goto return_error;
     }
 
+    OPAL_TIMING_ENV_NEXT(otmng, "mca_base_open");
+
     /* initialize if framework */
     if (OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_if_base_framework, 0))) {
         fprintf(stderr, "opal_if_base_open() failed -- process will likely abort (%s:%d, returned %d instead of OPAL_SUCCESS)\n",
