@@ -302,6 +302,15 @@ static int close_open_file_descriptors(int write_fd,
         return ORTE_ERR_FILE_OPEN_FAILURE;
     }
     struct dirent *files;
+
+    /* grab the fd of the opendir above so we don't close in the 
+     * middle of the scan. */
+    int dir_scan_fd = dirfd(dir);
+    if(dir_scan_fd < 0 ) {
+        return ORTE_ERR_FILE_OPEN_FAILURE;
+    }
+
+    
     while (NULL != (files = readdir(dir))) {
         if (!isdigit(files->d_name[0])) {
             continue;
@@ -315,7 +324,8 @@ static int close_open_file_descriptors(int write_fd,
 #if OPAL_PMIX_V1
             fd != opts.p_internal[1] &&
 #endif
-            fd != write_fd) {
+            fd != write_fd && 
+            fd != dir_scan_fd) {
             close(fd);
         }
     }
@@ -444,11 +454,22 @@ static int do_child(orte_odls_spawn_caddy_t *cd, int write_fd)
 
     /* Exec the new executable */
     execve(cd->cmd, cd->argv, cd->env);
-    getcwd(dir, sizeof(dir));
+    /* If we get here, an error has occurred. */
+    (void) getcwd(dir, sizeof(dir));
+    struct stat stats;
+    char* msg;
+    /* If errno is ENOENT, that indicates either cd->cmd does not exist, or
+     * cd->cmd is a script, but has a bad interpreter specified. */
+    if (ENOENT == errno && 0 == stat(cd->app->app, &stats)) {
+        asprintf(&msg, "%s has a bad interpreter on the first line.",
+                 cd->app->app);
+    } else {
+        msg = strdup(strerror(errno));
+    }
     send_error_show_help(write_fd, 1,
                          "help-orte-odls-default.txt", "execve error",
-                         orte_process_info.nodename, dir, cd->app->app, strerror(errno));
-    /* Does not return */
+                         orte_process_info.nodename, dir, cd->app->app, msg);
+    free(msg);
 }
 
 
