@@ -52,6 +52,8 @@
 #include <netdb.h>
 #endif
 #include <ctype.h>
+#include <sys/socket.h>
+#include <arpa/inet.h>
 
 #include "opal/util/show_help.h"
 #include "opal/util/error.h"
@@ -84,6 +86,8 @@
 #include "orte/mca/oob/tcp/oob_tcp_peer.h"
 #include "orte/mca/oob/tcp/oob_tcp_connection.h"
 #include "orte/mca/oob/tcp/oob_tcp_listener.h"
+#include "oob_tcp_peer.h"
+
 /*
  * Local utility functions
  */
@@ -143,12 +147,8 @@ mca_oob_tcp_component_t mca_oob_tcp_component = {
  */
 static int tcp_component_open(void)
 {
-    mca_oob_tcp_component.next_base = 0;
     OBJ_CONSTRUCT(&mca_oob_tcp_component.peers, opal_hash_table_t);
     opal_hash_table_init(&mca_oob_tcp_component.peers, 32);
-    OBJ_CONSTRUCT(&mca_oob_tcp_component.ev_bases, opal_pointer_array_t);
-    opal_pointer_array_init(&mca_oob_tcp_component.ev_bases,
-                            orte_oob_base.num_threads, 256, 8);
 
     OBJ_CONSTRUCT(&mca_oob_tcp_component.listeners, opal_list_t);
     if (ORTE_PROC_IS_HNP) {
@@ -184,9 +184,6 @@ static int tcp_component_open(void)
  */
 static int tcp_component_close(void)
 {
-    /* cleanup listen event list */
-    OPAL_LIST_DESTRUCT(&mca_oob_tcp_component.listeners);
-
     OBJ_DESTRUCT(&mca_oob_tcp_component.peers);
 
     if (NULL != mca_oob_tcp_component.ipv4conns) {
@@ -205,8 +202,6 @@ static int tcp_component_close(void)
     }
 #endif
 
-    OBJ_DESTRUCT(&mca_oob_tcp_component.ev_bases);
-
     return ORTE_SUCCESS;
 }
 static char *static_port_string;
@@ -663,27 +658,11 @@ static orte_rml_pathway_t* component_query_transports(void)
 static int component_startup(void)
 {
     int rc = ORTE_SUCCESS;
-    int i;
-    char *tmp;
-    opal_event_base_t *evb;
 
     opal_output_verbose(2, orte_oob_base_framework.framework_output,
                         "%s TCP STARTUP",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
 
-    /* initialize state */
-    if (0 == orte_oob_base.num_threads) {
-        opal_pointer_array_add(&mca_oob_tcp_component.ev_bases, orte_oob_base.ev_base);
-    } else {
-        for (i=0; i < orte_oob_base.num_threads; i++) {
-            asprintf(&tmp, "OOB-TCP-%d", i);
-            evb = opal_progress_thread_init(tmp);
-            opal_pointer_array_add(&mca_oob_tcp_component.ev_bases, evb);
-            opal_argv_append_nosize(&mca_oob_tcp_component.ev_threads, tmp);
-            free(tmp);
-        }
-    }
-
     /* if we are a daemon/HNP, or we are a standalone app,
      * then it is possible that someone else may initiate a
      * connection to us. In these cases, we need to start the
@@ -711,14 +690,6 @@ static void component_shutdown(void)
                         "%s TCP SHUTDOWN",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
 
-    if (0 < orte_oob_base.num_threads) {
-        for (i=0; i < orte_oob_base.num_threads; i++) {
-            opal_progress_thread_finalize(mca_oob_tcp_component.ev_threads[i]);
-            opal_pointer_array_set_item(&mca_oob_tcp_component.ev_bases, i, NULL);
-        }
-        opal_argv_free(mca_oob_tcp_component.ev_threads);
-    }
-
     if (ORTE_PROC_IS_HNP && mca_oob_tcp_component.listen_thread_active) {
         mca_oob_tcp_component.listen_thread_active = false;
         /* tell the thread to exit */
@@ -744,6 +715,9 @@ static void component_shutdown(void)
                                                  (void **) &peer, node, &node);
     }
 
+    /* cleanup listen event list */
+    OPAL_LIST_DESTRUCT(&mca_oob_tcp_component.listeners);
+
     opal_output_verbose(2, orte_oob_base_framework.framework_output,
                         "%s TCP SHUTDOWN done",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
@@ -843,6 +817,8 @@ static int parse_uri(const uint16_t af_family,
             opal_output (0, "oob_tcp_parse_uri: Could not convert %s\n", host);
             return ORTE_ERR_BAD_PARAM;
         }
+        in6->sin6_family = AF_INET6;
+        in6->sin6_port =  htons(atoi(port));
     }
 #endif
     else {
@@ -973,6 +949,7 @@ static int component_set_addr(orte_process_name_t *peer,
             }
 
             maddr = OBJ_NEW(mca_oob_tcp_addr_t);
+            ((struct sockaddr_storage*) &(maddr->addr))->ss_family = af_family;
             if (ORTE_SUCCESS != (rc = parse_uri(af_family, host, ports, (struct sockaddr_storage*) &(maddr->addr)))) {
                 ORTE_ERROR_LOG(rc);
                 OBJ_RELEASE(maddr);
@@ -1359,7 +1336,6 @@ static char **split_and_resolve(char **orig_str, char *name)
 
 static void peer_cons(mca_oob_tcp_peer_t *peer)
 {
-    peer->ev_base = NULL;
     peer->auth_method = NULL;
     peer->sd = -1;
     OBJ_CONSTRUCT(&peer->addrs, opal_list_t);
