@@ -13,9 +13,9 @@
  * Copyright (c) 2007-2012 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2011-2017 IBM Corporation.  All rights reserved.
- * Copyright (c) 2014-2017 Intel, Inc.  All rights reserved.
- * Copyright (c) 2015-2017 Research Organization for Information Science
+ * Copyright (c) 2011-2019 IBM Corporation.  All rights reserved.
+ * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
@@ -87,6 +87,7 @@
 #include "orte/mca/ess/base/base.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/grpcomm/base/base.h"
+#include "orte/mca/oob/base/base.h"
 #include "orte/mca/rmaps/rmaps.h"
 #include "orte/mca/routed/routed.h"
 #include "orte/mca/rml/base/rml_contact.h"
@@ -99,7 +100,7 @@
 
 static int rsh_init(void);
 static int rsh_launch(orte_job_t *jdata);
-static int remote_spawn(opal_buffer_t *launch);
+static int remote_spawn(void);
 static int rsh_terminate_orteds(void);
 static int rsh_finalize(void);
 
@@ -258,10 +259,12 @@ static int rsh_init(void)
 /**
  * Callback on daemon exit.
  */
-static void rsh_wait_daemon(orte_proc_t *daemon, void* cbdata)
+static void rsh_wait_daemon(int sd, short flags, void *cbdata)
 {
     orte_job_t *jdata;
-    orte_plm_rsh_caddy_t *caddy=(orte_plm_rsh_caddy_t*)cbdata;
+    orte_wait_tracker_t *t2 = (orte_wait_tracker_t*)cbdata;
+    orte_plm_rsh_caddy_t *caddy=(orte_plm_rsh_caddy_t*)t2->cbdata;
+    orte_proc_t *daemon = caddy->daemon;
     char *rtmod;
 
     if (orte_orteds_term_ordered || orte_abnormal_term_ordered) {
@@ -269,6 +272,7 @@ static void rsh_wait_daemon(orte_proc_t *daemon, void* cbdata)
          * session attached, e.g., while debugging
          */
         OBJ_RELEASE(caddy);
+        OBJ_RELEASE(t2);
         return;
     }
 
@@ -322,7 +326,7 @@ static void rsh_wait_daemon(orte_proc_t *daemon, void* cbdata)
         opal_event_active(&launch_event, EV_WRITE, 1);
     }
     /* cleanup */
-    OBJ_RELEASE(caddy);
+    OBJ_RELEASE(t2);
 }
 
 static int setup_launch(int *argcptr, char ***argvptr,
@@ -339,11 +343,12 @@ static int setup_launch(int *argcptr, char ***argvptr,
     char *orted_cmd, *orted_prefix, *final_cmd;
     int orted_index;
     int rc;
-    int i, j;
+    int i, j, cnt;
     bool found;
     char *lib_base=NULL, *bin_base=NULL;
     char *opal_prefix = getenv("OPAL_PREFIX");
     char* full_orted_cmd = NULL;
+    char * rtmod;
 
     /* Figure out the basenames for the libdir and bindir.  This
        requires some explanation:
@@ -496,10 +501,13 @@ static int setup_launch(int *argcptr, char ***argvptr,
              * we have to insert the orted_prefix in the right place
              */
             (void)asprintf (&final_cmd,
-                            "%s%s%s PATH=%s%s$PATH ; export PATH ; "
+                            "%s%s%s%s%s%s PATH=%s%s$PATH ; export PATH ; "
                             "LD_LIBRARY_PATH=%s%s$LD_LIBRARY_PATH ; export LD_LIBRARY_PATH ; "
                             "DYLD_LIBRARY_PATH=%s%s$DYLD_LIBRARY_PATH ; export DYLD_LIBRARY_PATH ; "
                             "%s %s",
+                            (NULL != mca_plm_rsh_component.chdir ? "cd " : " "),
+                            (NULL != mca_plm_rsh_component.chdir ? mca_plm_rsh_component.chdir : " "),
+                            (NULL != mca_plm_rsh_component.chdir ? " ; " : " "),
                             (opal_prefix != NULL ? "OPAL_PREFIX=" : " "),
                             (opal_prefix != NULL ? opal_prefix : " "),
                             (opal_prefix != NULL ? " ; export OPAL_PREFIX;" : " "),
@@ -526,7 +534,7 @@ static int setup_launch(int *argcptr, char ***argvptr,
              * we have to insert the orted_prefix in the right place
              */
             (void)asprintf (&final_cmd,
-                            "%s%s%s set path = ( %s $path ) ; "
+                            "%s%s%s%s%s%s set path = ( %s $path ) ; "
                             "if ( $?LD_LIBRARY_PATH == 1 ) "
                             "set OMPI_have_llp ; "
                             "if ( $?LD_LIBRARY_PATH == 0 ) "
@@ -540,6 +548,9 @@ static int setup_launch(int *argcptr, char ***argvptr,
                             "if ( $?OMPI_have_dllp == 1 ) "
                             "setenv DYLD_LIBRARY_PATH %s%s$DYLD_LIBRARY_PATH ; "
                             "%s %s",
+                            (NULL != mca_plm_rsh_component.chdir ? "cd " : " "),
+                            (NULL != mca_plm_rsh_component.chdir ? mca_plm_rsh_component.chdir : " "),
+                            (NULL != mca_plm_rsh_component.chdir ? " ; " : " "),
                             (opal_prefix != NULL ? "setenv OPAL_PREFIX " : " "),
                             (opal_prefix != NULL ? opal_prefix : " "),
                             (opal_prefix != NULL ? " ;" : " "),
@@ -603,7 +614,18 @@ static int setup_launch(int *argcptr, char ***argvptr,
          (mca_plm_rsh_component.using_qrsh && mca_plm_rsh_component.daemonize_qrsh)) &&
         ((!mca_plm_rsh_component.using_llspawn) ||
          (mca_plm_rsh_component.using_llspawn && mca_plm_rsh_component.daemonize_llspawn))) {
-        opal_argv_append(&argc, &argv, "--daemonize");
+    }
+
+    if (!mca_plm_rsh_component.no_tree_spawn) {
+        // Remove problematic and/or conflicting command line arguments that
+        // should not be passed on to our children.
+        cnt = opal_argv_count(orted_cmd_line);
+        for (i=0; i < cnt; i+=3) {
+            if (0 == strcmp(orted_cmd_line[i+1], "routed")) {
+                opal_argv_delete(&cnt, &orted_cmd_line, i, 3);
+                break;
+            }
+        }
     }
 
     /*
@@ -615,9 +637,30 @@ static int setup_launch(int *argcptr, char ***argvptr,
                                           proc_vpid_index);
 
     /* ensure that only the ssh plm is selected on the remote daemon */
-    opal_argv_append_nosize(&argv, "-"OPAL_MCA_CMD_LINE_ID);
-    opal_argv_append_nosize(&argv, "plm");
-    opal_argv_append_nosize(&argv, "rsh");
+    opal_argv_append(&argc, &argv, "-"OPAL_MCA_CMD_LINE_ID);
+    opal_argv_append(&argc, &argv, "plm");
+    opal_argv_append(&argc, &argv, "rsh");
+
+    /* if we are tree-spawning, tell our child daemons the
+     * uri of their parent (me) */
+    if (!mca_plm_rsh_component.no_tree_spawn) {
+        opal_argv_append(&argc, &argv, "--tree-spawn");
+        orte_oob_base_get_addr(&param);
+
+        // When tree-spawn'ing we need to force the remote daemons to use
+        // the routing component that was used to setup the launch tree.
+        // Otherwise the orte_parent_uri will not match the orted they
+        // expect to find in the routing tree.
+        rtmod = orte_rml.get_routed(orte_coll_conduit);
+        opal_argv_append(&argc, &argv, "-"OPAL_MCA_CMD_LINE_ID);
+        opal_argv_append(&argc, &argv, "routed");
+        opal_argv_append(&argc, &argv, rtmod);
+
+        opal_argv_append(&argc, &argv, "-"OPAL_MCA_CMD_LINE_ID);
+        opal_argv_append(&argc, &argv, "orte_parent_uri");
+        opal_argv_append(&argc, &argv, param);
+        free(param);
+    }
 
     /* unless told otherwise... */
     if (mca_plm_rsh_component.pass_environ_mca_params) {
@@ -769,7 +812,7 @@ static void ssh_child(int argc, char **argv)
 /*
  * launch a set of daemons from a remote daemon
  */
-static int remote_spawn(opal_buffer_t *launch)
+static int remote_spawn(void)
 {
     int node_name_index1;
     int proc_vpid_index;
@@ -778,7 +821,6 @@ static int remote_spawn(opal_buffer_t *launch)
     int argc;
     int rc=ORTE_SUCCESS;
     bool failed_launch = true;
-    orte_std_cntr_t n;
     orte_process_name_t target;
     orte_plm_rsh_caddy_t *caddy;
     orte_job_t *daemons;
@@ -793,11 +835,13 @@ static int remote_spawn(opal_buffer_t *launch)
     /* if we hit any errors, tell the HNP it was us */
     target.vpid = ORTE_PROC_MY_NAME->vpid;
 
-    /* extract the prefix from the launch buffer */
-    n = 1;
-    if (ORTE_SUCCESS != (rc = opal_dss.unpack(launch, &prefix, &n, OPAL_STRING))) {
-        ORTE_ERROR_LOG(rc);
-        goto cleanup;
+    /* check to see if enable-orterun-prefix-by-default was given - if
+     * this is being done by a singleton, then orterun will not be there
+     * to put the prefix in the app. So make sure we check to find it */
+    if ((bool)ORTE_WANT_ORTERUN_PREFIX_BY_DEFAULT) {
+        prefix = strdup(opal_install_dirs.prefix);
+    } else {
+        prefix = NULL;
     }
 
     /* get the updated routing list */
@@ -872,6 +916,10 @@ static int remote_spawn(opal_buffer_t *launch)
         opal_list_append(&launch_list, &caddy->super);
     }
     OPAL_LIST_DESTRUCT(&coll);
+    /* we NEVER use tree-spawn for secondary launches - e.g.,
+     * due to a dynamic launch requesting add_hosts - so be
+     * sure to turn it off here */
+    mca_plm_rsh_component.no_tree_spawn = true;
 
     /* trigger the event to start processing the launch list */
     OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
@@ -937,7 +985,7 @@ static void process_launch_list(int fd, short args, void *cbdata)
         caddy = (orte_plm_rsh_caddy_t*)item;
         /* register the sigchild callback */
         ORTE_FLAG_SET(caddy->daemon, ORTE_PROC_FLAG_ALIVE);
-        orte_wait_cb(caddy->daemon, rsh_wait_daemon, (void*)caddy);
+        orte_wait_cb(caddy->daemon, rsh_wait_daemon, orte_event_base, (void*)caddy);
 
         /* fork a child to exec the rsh/ssh session */
         pid = fork();
@@ -1155,24 +1203,8 @@ static void launch_daemons(int fd, short args, void *cbdata)
 
     /* if we are tree launching, find our children and create the launch cmd */
     if (!mca_plm_rsh_component.no_tree_spawn) {
-        orte_daemon_cmd_flag_t command = ORTE_DAEMON_TREE_SPAWN;
         orte_job_t *jdatorted;
 
-        /* get the tree spawn buffer */
-        orte_tree_launch_cmd = OBJ_NEW(opal_buffer_t);
-        /* insert the tree_spawn cmd */
-        if (ORTE_SUCCESS != (rc = opal_dss.pack(orte_tree_launch_cmd, &command, 1, ORTE_DAEMON_CMD))) {
-            ORTE_ERROR_LOG(rc);
-            OBJ_RELEASE(orte_tree_launch_cmd);
-            goto cleanup;
-        }
-        /* pack the prefix since this will be needed by the next wave */
-        if (ORTE_SUCCESS != (rc = opal_dss.pack(orte_tree_launch_cmd, &prefix_dir, 1, OPAL_STRING))) {
-            ORTE_ERROR_LOG(rc);
-            OBJ_RELEASE(orte_tree_launch_cmd);
-            goto cleanup;
-        }
-
         /* get the orted job data object */
         if (NULL == (jdatorted = orte_get_job_data_object(ORTE_PROC_MY_NAME->jobid))) {
             ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
@@ -1184,6 +1216,10 @@ static void launch_daemons(int fd, short args, void *cbdata)
         OBJ_CONSTRUCT(&coll, opal_list_t);
         rtmod = orte_rml.get_routed(orte_coll_conduit);
         orte_routed.get_routing_list(rtmod, &coll);
+
+        OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
+                             "%s plm:rsh:launch Tree Launch using routed/%s",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), rtmod));
     }
 
     /* setup the launch */
@@ -1281,6 +1317,10 @@ static void launch_daemons(int fd, short args, void *cbdata)
         OBJ_RETAIN(caddy->daemon);
         opal_list_append(&launch_list, &caddy->super);
     }
+    /* we NEVER use tree-spawn for secondary launches - e.g.,
+     * due to a dynamic launch requesting add_hosts - so be
+     * sure to turn it off here */
+    mca_plm_rsh_component.no_tree_spawn = true;
 
     /* set the job state to indicate the daemons are launched */
     state->jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
