@@ -13,7 +13,7 @@
  * Copyright (c) 2011-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013-2015 Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2018 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -186,14 +186,15 @@ int orte_dt_print_job(char **output, char *prefix, orte_job_t *src, opal_data_ty
         asprintf(&pfx2, "%s", prefix);
     }
 
-    asprintf(&tmp, "\n%sData for job: %s\tPersonality: %s\tRecovery: %s(%s)\n%s\tNum apps: %ld\tMPI allowed: %s\tStdin target: %s\tState: %s\tAbort: %s", pfx2,
-             ORTE_JOBID_PRINT(src->jobid), src->personality,
+    tmp2 = opal_argv_join(src->personality, ',');
+    asprintf(&tmp, "\n%sData for job: %s\tPersonality: %s\tRecovery: %s(%s)\n%s\tNum apps: %ld\tStdin target: %s\tState: %s\tAbort: %s", pfx2,
+             ORTE_JOBID_PRINT(src->jobid), tmp2,
              (ORTE_FLAG_TEST(src, ORTE_JOB_FLAG_RECOVERABLE)) ? "ENABLED" : "DISABLED",
              (orte_get_attribute(&src->attributes, ORTE_JOB_RECOVER_DEFINED, NULL, OPAL_BOOL)) ? "DEFINED" : "DEFAULT",
              pfx2,
-             (long)src->num_apps,
-             (ORTE_FLAG_TEST(src, ORTE_JOB_FLAG_GANG_LAUNCHED)) ? "YES" : "NO", ORTE_VPID_PRINT(src->stdin_target),
+             (long)src->num_apps, ORTE_VPID_PRINT(src->stdin_target),
               orte_job_state_to_str(src->state), (ORTE_FLAG_TEST(src, ORTE_JOB_FLAG_ABORTED)) ? "True" : "False");
+    free(tmp2);
     asprintf(&pfx, "%s\t", pfx2);
     free(pfx2);
 
@@ -343,8 +344,8 @@ int orte_dt_print_node(char **output, char *prefix, orte_node_t *src, opal_data_
         goto PRINT_PROCS;
     }
 
-    asprintf(&tmp, "\n%sData for node: %s\tState: %0x",
-             pfx2, (NULL == src->name) ? "UNKNOWN" : src->name, src->state);
+    asprintf(&tmp, "\n%sData for node: %s\tState: %0x\tFlags: %02x",
+             pfx2, (NULL == src->name) ? "UNKNOWN" : src->name, src->state, src->flags);
     /* does this node have any aliases? */
     tmp3 = NULL;
     if (orte_get_attribute(&src->attributes, ORTE_NODE_ALIAS, (void**)&tmp3, OPAL_STRING)) {
@@ -441,9 +442,12 @@ int orte_dt_print_node(char **output, char *prefix, orte_node_t *src, opal_data_
  */
 int orte_dt_print_proc(char **output, char *prefix, orte_proc_t *src, opal_data_type_t type)
 {
-    char *tmp, *tmp2, *pfx2;
-    hwloc_obj_t loc=NULL, bd=NULL;
-    char locale[1024], bind[1024];
+    char *tmp, *tmp3, *pfx2;
+    hwloc_obj_t loc=NULL;
+    char locale[1024], tmp1[1024], tmp2[1024];
+    hwloc_cpuset_t mycpus;
+    char *str=NULL, *cpu_bitmap=NULL;
+
 
     /* set default result */
     *output = NULL;
@@ -469,18 +473,14 @@ int orte_dt_print_proc(char **output, char *prefix, orte_proc_t *src, opal_data_
     }
 
     if (!orte_devel_level_output) {
-        hwloc_cpuset_t mycpus;
-        char tmp1[1024], tmp2[1024];
-        char *str=NULL, *cpu_bitmap=NULL;
-
         if (orte_get_attribute(&src->attributes, ORTE_PROC_CPU_BITMAP, (void**)&cpu_bitmap, OPAL_STRING) &&
-            NULL != src->node->topology) {
+            NULL != src->node->topology && NULL != src->node->topology->topo) {
             mycpus = hwloc_bitmap_alloc();
             hwloc_bitmap_list_sscanf(mycpus, cpu_bitmap);
-            if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1), src->node->topology, mycpus)) {
+            if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1), src->node->topology->topo, mycpus)) {
                 str = strdup("UNBOUND");
             } else {
-                opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), src->node->topology, mycpus);
+                opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), src->node->topology->topo, mycpus);
                 asprintf(&str, "%s:%s", tmp1, tmp2);
             }
             hwloc_bitmap_free(mycpus);
@@ -508,14 +508,14 @@ int orte_dt_print_proc(char **output, char *prefix, orte_proc_t *src, opal_data_
 
     asprintf(&tmp, "\n%sData for proc: %s", pfx2, ORTE_NAME_PRINT(&src->name));
 
-    asprintf(&tmp2, "%s\n%s\tPid: %ld\tLocal rank: %lu\tNode rank: %lu\tApp rank: %d", tmp, pfx2,
+    asprintf(&tmp3, "%s\n%s\tPid: %ld\tLocal rank: %lu\tNode rank: %lu\tApp rank: %d", tmp, pfx2,
              (long)src->pid, (unsigned long)src->local_rank, (unsigned long)src->node_rank, src->app_rank);
     free(tmp);
-    tmp = tmp2;
+    tmp = tmp3;
 
     if (orte_get_attribute(&src->attributes, ORTE_PROC_HWLOC_LOCALE, (void**)&loc, OPAL_PTR)) {
         if (NULL != loc) {
-            if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2mapstr(locale, sizeof(locale), src->node->topology, loc->cpuset)) {
+            if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2mapstr(locale, sizeof(locale), src->node->topology->topo, loc->cpuset)) {
                 strcpy(locale, "NODE");
             }
         } else {
@@ -524,23 +524,26 @@ int orte_dt_print_proc(char **output, char *prefix, orte_proc_t *src, opal_data_
     } else {
         strcpy(locale, "UNKNOWN");
     }
-    if (orte_get_attribute(&src->attributes, ORTE_PROC_HWLOC_BOUND, (void**)&bd, OPAL_PTR)) {
-        if (NULL != bd) {
-            if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2mapstr(bind, sizeof(bind), src->node->topology, bd->cpuset)) {
-                strcpy(bind, "UNBOUND");
-            }
-        } else {
-            strcpy(bind, "UNBOUND");
-        }
+    if (orte_get_attribute(&src->attributes, ORTE_PROC_CPU_BITMAP, (void**)&cpu_bitmap, OPAL_STRING) &&
+        NULL != src->node->topology && NULL != src->node->topology->topo) {
+        mycpus = hwloc_bitmap_alloc();
+        hwloc_bitmap_list_sscanf(mycpus, cpu_bitmap);
+        opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), src->node->topology->topo, mycpus);
     } else {
-        strcpy(bind, "UNBOUND");
+        snprintf(tmp2, sizeof(tmp2), "UNBOUND");
     }
-    asprintf(&tmp2, "%s\n%s\tState: %s\tApp_context: %ld\n%s\tLocale:  %s\n%s\tBinding: %s", tmp, pfx2,
-             orte_proc_state_to_str(src->state), (long)src->app_idx, pfx2, locale, pfx2, bind);
+    asprintf(&tmp3, "%s\n%s\tState: %s\tApp_context: %ld\n%s\tLocale:  %s\n%s\tBinding: %s", tmp, pfx2,
+             orte_proc_state_to_str(src->state), (long)src->app_idx, pfx2, locale, pfx2,  tmp2);
     free(tmp);
+    if (NULL != str) {
+        free(str);
+    }
+    if (NULL != cpu_bitmap) {
+        free(cpu_bitmap);
+    }
 
     /* set the return */
-    *output = tmp2;
+    *output = tmp3;
 
     free(pfx2);
     return ORTE_SUCCESS;
@@ -675,7 +678,7 @@ int orte_dt_print_map(char **output, char *prefix, orte_job_map_t *src, opal_dat
                  orte_rmaps_base_print_mapping(src->mapping),
                  orte_rmaps_base_print_ranking(src->ranking),
                  pfx2, opal_hwloc_base_print_binding(src->binding),
-                 (NULL == opal_hwloc_base_cpu_set) ? "NULL" : opal_hwloc_base_cpu_set,
+                 (NULL == opal_hwloc_base_cpu_list) ? "NULL" : opal_hwloc_base_cpu_list,
                  (NULL == src->ppr) ? "NULL" : src->ppr,
                  (int)src->cpus_per_rank);
 
@@ -851,13 +854,13 @@ int orte_dt_print_sig(char **output, char *prefix, orte_grpcomm_signature_t *src
     }
 
     if (NULL == src->signature) {
-        asprintf(output, "%sORTE_SIG  SeqNumber:%d  Procs: NULL", prefx, src->seq_num);
+        asprintf(output, "%sORTE_SIG  Procs: NULL", prefx);
         free(prefx);
         return ORTE_SUCCESS;
     }
 
     /* there must be at least one proc in the signature */
-    asprintf(&tmp, "%sORTE_SIG  SeqNumber:%d  Procs: ", prefx, src->seq_num);
+    asprintf(&tmp, "%sORTE_SIG  Procs: ", prefx);
 
     for (i=0; i < src->sz; i++) {
         asprintf(&tmp2, "%s%s", tmp, ORTE_NAME_PRINT(&src->signature[i]));
@@ -867,4 +870,3 @@ int orte_dt_print_sig(char **output, char *prefix, orte_grpcomm_signature_t *src
     *output = tmp;
     return ORTE_SUCCESS;
 }
-
