@@ -17,11 +17,18 @@
 #if OSHMEM_PROFILING
 #include "oshmem/include/pshmem.h"
 #pragma weak shmem_quiet = pshmem_quiet
+#pragma weak shmem_ctx_quiet = pshmem_ctx_quiet
 #include "oshmem/shmem/c/profile/defines.h"
 #endif
 
 void shmem_quiet(void)
 {
 
-    MCA_SPML_CALL(fence());
+    MCA_SPML_CALL(quiet(oshmem_ctx_default));
+}
+
+void shmem_ctx_quiet(shmem_ctx_t ctx)
+{
+
+    MCA_SPML_CALL(quiet(ctx));
 }
